package cn.wise.sc.pay.common.core.model;

import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * 封装公共的接口
 *
 * @param <T> service对象
 * @param <M> 实体对象
 * @author qh
 */
@Slf4j
public class BaseController<T extends IService<M>, M> {

    T iService;

    public BaseController(T iService) {
        this.iService = iService;
    }

    @PostMapping("/list")
    public BaseResponse list(@RequestBody PageWrapperQuery pageWrapperQuery) {
        QueryWrapper queryWrapper = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);
        try {
            List<M> list = iService.list(queryWrapper);
            return BaseResponse.ok(list, iService.getClass());
        } catch (Exception ex) {
            log.error("查询操作失败 ===== " + iService.toString() + ".list()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("查询失败", iService.getClass());
        }
    }

    @PostMapping("/page")
    public BaseResponse page(@RequestBody PageWrapperQuery page) {

        QueryWrapper queryWrapper = PageWrapperQueryUtil.getQueryWrapper(page);
        IPage page1 = new Page(page.getPageQuery().getPageNo(), page.getPageQuery().getPageSize());
        try {
            page1 = iService.page(page1, queryWrapper);
            BaseResponse<M> baseResponse = BaseResponse.ok();
            baseResponse.setPage((Page) page1);
            baseResponse.setClassName(iService.getClass().getName());
            return baseResponse;
        } catch (Exception ex) {
            log.error("查询操作失败 ===== " + iService.toString() + ".page()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("分页查询失败", iService.getClass());
        }
    }

    @GetMapping("/{id}")
    public BaseResponse getById(@PathVariable("id") long id) {
        try {
            M model = iService.getById(id);
            return BaseResponse.ok(model);
        } catch (Exception ex) {
            log.error("查询操作失败 ===== " + iService.toString() + ".create()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("获取详情信息失败", iService.getClass());
        }
    }

    @PostMapping
    public BaseResponse create(@RequestBody M model) {
        try {
            iService.save(model);
            return BaseResponse.okMsg("添加成功", model.getClass());
        } catch (Exception ex) {
            log.error("查询操作失败 ===== " + iService.toString() + ".create()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("添加失败", model.getClass());
        }
    }

    @PutMapping
    public BaseResponse update(@RequestBody M model) {
        try {
            iService.updateById(model);
            return BaseResponse.okMsg("更新成功", model.getClass());
        } catch (Exception ex) {
            log.error("查询操作失败 ===== " + iService.toString() + ".create()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("更新记录失败", model.getClass());
        }
    }

    @DeleteMapping("/{id}")
    public BaseResponse delById(@PathVariable("id") long id) {
        try {
            boolean removeById = iService.removeById(id);
            return BaseResponse.ok(removeById);
        } catch (Exception ex) {
            log.error("查询操作失败 ===== " + iService.toString() + ".create()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("获取详情信息失败", iService.getClass());
        }
    }

}
