package cn.wise.sc.pay.common.core.util;

import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.util.Arrays;
import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import org.apache.commons.codec.binary.Base64;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

/**
 * @description: 小程序用户信息解密
 * @author: qh
 * @create: 2020-01-09 16:58
 **/
public class DecryptUtil {

    // 算法名称
    final String KEY_ALGORITHM = "AES";

    // 加解密算法/模式/填充方式
    final String algorithmStr = "AES/CBC/PKCS7Padding";
    private Key key;
    private Cipher cipher;

    public void init(byte[] keyBytes) {

        // 如果密钥不足16位，那么就补足. 这个if 中的内容很重要
        int base = 16;
        if (keyBytes.length % base != 0) {
            int groups = keyBytes.length / base + (keyBytes.length % base != 0 ? 1 : 0);
            byte[] temp = new byte[groups * base];
            Arrays.fill(temp, (byte) 0);
            System.arraycopy(keyBytes, 0, temp, 0, keyBytes.length);
            keyBytes = temp;
        }
        // 初始化
        Security.addProvider(new BouncyCastleProvider());
        // 转化成JAVA的密钥格式
        key = new SecretKeySpec(keyBytes, KEY_ALGORITHM);
        try {
            // 初始化cipher
            cipher = Cipher.getInstance(algorithmStr);
        } catch (NoSuchAlgorithmException e) {
            //Auto-generated catch block
            e.printStackTrace();
        } catch (NoSuchPaddingException e) {
            //Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * 解密方法
     *
     * @param encryptedDataStr 要解密的字符串
     * @param keyBytesStr      解密密钥
     * @param ivStr            初始位移
     * @return
     */
    public byte[] decrypt(String encryptedDataStr, String keyBytesStr, String ivStr) {
        byte[] encryptedText = null;
        byte[] encryptedData = null;
        byte[] sessionkey = null;
        byte[] iv = null;

        try {
            sessionkey = Base64.decodeBase64(keyBytesStr);
            encryptedData = Base64.decodeBase64(encryptedDataStr);
            iv = Base64.decodeBase64(ivStr);
            int base = 16;
            if (iv.length % base != 0) {
                int groups = iv.length / base + (iv.length % base != 0 ? 1 : 0);
                byte[] temp = new byte[groups * base];
                Arrays.fill(temp, (byte) 0);
                System.arraycopy(iv, 0, temp, 0, iv.length);
                iv = temp;
            }
            init(sessionkey);

            cipher.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(iv));
            encryptedText = cipher.doFinal(encryptedData);
        } catch (Exception e) {
            //Auto-generated catch block
            e.printStackTrace();
        }
        return encryptedText;
    }

    public static void main(String[] args) throws UnsupportedEncodingException {
        DecryptUtil d = new DecryptUtil();
        String content = "5wR9yKMKPzXctqgvilFZWzIx5sdDzP008tSpQXs8vXT8O4Lwygxjob/v7f3JyZgelvEQRwuB2oZOYfAQHRk2/kuJVJ84KzGZpGpTugfr5q11a0cBp7ejgIyUu7sNjOx843snqi4NfEnbZNcqbXwtMWQRrMkdF4eB386vINoQKdhnEGpowMNLXQXpAL+eK7duCT0VULLgTCcalPg58sAZeg==";
        String key = "dOcYH99TxkutoPEMje0BbQ==";
        String iv = "KBb/+kqfZ9BmILjdIqMqAw==";
        byte[] result = d.decrypt(content, key, iv);
        System.out.println(new String(result, StandardCharsets.UTF_8));
    }

}
