package cn.wise.sc.pay.common.mq.server.service;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import cn.wise.sc.pay.common.mq.server.enums.TransactionMessageStatusEnum;
import cn.wise.sc.pay.common.mq.server.mapper.TransactionMessageMapper;
import cn.wise.sc.pay.common.mq.server.model.TransactionMessage;
import cn.wise.sc.pay.common.mq.server.query.MessageQuery;
import cn.wise.sc.pay.common.mq.server.util.DateUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;


/**
 * @description:
 * @author: qh
 * @create: 2019-11-05 15:40
 **/
@Service
public class TransactionMessageServiceImpl extends ServiceImpl<TransactionMessageMapper, TransactionMessage> implements TransactionMessageService {

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean sendMessage(TransactionMessage message) {
        if (check(message)) {
            super.save(message);
            return true;
        }
        return false;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean sendMessage(List<TransactionMessage> messages) {
        for (TransactionMessage message : messages) {
            if (!check(message)) {
                return false;
            }
        }
        super.saveBatch(messages);
        return true;
    }

    private boolean check(TransactionMessage message) {
        if (!StringUtils.hasText(message.getMessage()) || !StringUtils.hasText(message.getQueue())
                || !StringUtils.hasText(message.getSendSystem())) {
            return false;
        }
        return message.getCreateTime() != null;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean confirmCustomerMessage(String customerSystem, Long messageId) {
        TransactionMessage message = this.getById(messageId);
        if (message == null) {
            return false;
        }
        message.setCustomerDate(LocalDateTime.now());
        message.setStatus(TransactionMessageStatusEnum.OVER.getStatus());
        message.setCustomerSystem(customerSystem);

        this.updateById(message);

        return true;
    }

    @Override
    public List<TransactionMessage> findByWatingMessage(int limit) {
        if (limit > 1000) {
            limit = 1000;
        }
        IPage page = new Page(0, limit);
        QueryWrapper qw = new QueryWrapper();
        qw.eq("status", TransactionMessageStatusEnum.WATING.getStatus());
        qw.orderByAsc("id");

        return this.page(page, qw).getRecords();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean confirmDieMessage(Long messageId) {
        TransactionMessage message = this.getById(messageId);
        if (message == null) {
            return false;
        }
        message.setStatus(TransactionMessageStatusEnum.DIE.getStatus());
        message.setDieDate(LocalDateTime.now());

        this.updateById(message);

        return true;
    }

    @Override
    public boolean incrSendCount(Long messageId, Date sendDate) {
        TransactionMessage message = super.getById(messageId);
        if (message == null) {
            return false;
        }
        message.setSendDate(DateUtils.date2LocalDate(new Date()));
        message.setSendCount(message.getSendCount() + 1);

        this.updateById(message);

        return false;
    }

    @Override
    public boolean retrySendDieMessage() {

        UpdateWrapper uw = new UpdateWrapper();
        uw.eq("status", TransactionMessageStatusEnum.DIE.getStatus());
        uw.eq("send_count", 0);

        TransactionMessage message = new TransactionMessage();
        message.setStatus(TransactionMessageStatusEnum.WATING.getStatus());

        this.update(message, uw);

        return true;
    }

    @Override
    public List<TransactionMessage> findMessageByPage(MessageQuery query, TransactionMessageStatusEnum status) {

        IPage page = new Page(query.getPageNo(), query.getPageSize());

        QueryWrapper qw = new QueryWrapper();
        qw.eq("status", status.getStatus());
        qw.orderByAsc("id");

        return this.page(page, qw).getRecords();
    }
}