package cn.wise.sc.pay.common.security.configration;

import lombok.SneakyThrows;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.annotation.Order;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.builders.WebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.crypto.factory.PasswordEncoderFactories;
import org.springframework.security.crypto.password.PasswordEncoder;

/**
 * 认证相关配置
 */
@Primary
@Order(90)
@Configuration
@EnableGlobalMethodSecurity(prePostEnabled = true)
public class WebSecurityConfigurer extends WebSecurityConfigurerAdapter {

    @Override
    @SneakyThrows
    protected void configure(HttpSecurity http) {
        http
                .authorizeRequests()
                .antMatchers(
                        "/asso/**",
                        "/association/**",
                        "/cashiers/**",
                        "/merchant/**",
                        "/sendSms",
                        "/app/**",
                        "/manager/**",
                        "/manage/**",
                        "/actuator/**",
                        "/token/**", "/login", "/open/**", "/swagger-ui.html",//让他们可以通过spring的加密体系 (没用）
                        "/swagger-resources/**",
                        "/v2/api-docs",
                        "/doc.html",
                        "/webjars/**").permitAll()
                .anyRequest().authenticated()
                .and().csrf().disable();
    }

    @Override
    public void configure(WebSecurity web) throws Exception {
        // web.ignoring是直接绕开spring security的所有filter，直接跳过验证
        web.ignoring().antMatchers("/v2/**", "swagger-ui.html","doc.html");

    }

    @Bean
    @Override
    @SneakyThrows
    public AuthenticationManager authenticationManagerBean() {
        return super.authenticationManagerBean();
    }

    //@Bean
    //public AuthenticationSuccessHandler mobileLoginSuccessHandler() {
    //	return MobileLoginSuccessHandler.builder()
    //		.objectMapper(objectMapper)
    //		.clientDetailsService(clientDetailsService)
    //		.passwordEncoder(passwordEncoder())
    //		.defaultAuthorizationServerTokenServices(defaultAuthorizationServerTokenServices).build();
    //}


    /**
     * https://spring.io/blog/2017/11/01/spring-security-5-0-0-rc1-released#password-storage-updated
     * Encoded password does not look like BCrypt
     *
     * @return PasswordEncoder
     */
    @Bean
    public PasswordEncoder passwordEncoder() {
        return PasswordEncoderFactories.createDelegatingPasswordEncoder();
    }

}
