package cn.wise.sc.pay.common.security2.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.file.FileWriter;
import cn.wise.sc.pay.common.security2.util.JwtUtils;
import io.swagger.annotations.ApiOperation;
import org.redisson.api.RedissonClient;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import java.io.File;

/**
 * @description: 获取token的页面
 * @author: qh
 * @create: 2020-04-21 16:06
 **/
@CrossOrigin
@RestController
@RequestMapping("/common/security")
public class TokenIndexController {

    final
    RedissonClient redissonClient;
    final
    AuthenticationManager authenticationManager;
    final
    UserDetailsService userDetailsService;

    public TokenIndexController(AuthenticationManager authenticationManager,
                                UserDetailsService userDetailsService,
                                RedissonClient redissonClient) {
        this.authenticationManager = authenticationManager;
        this.userDetailsService = userDetailsService;
        this.redissonClient = redissonClient;
    }

    /**
     * 创建token
     *
     * @return
     * @throws Exception
     */
    @PostMapping("/token")
    @ApiOperation("获取token信息")
    public String createAuthenticationToken(String username, String password) throws Exception {

        authenticate(username, password);

        UserDetails userDetails = userDetailsService
                .loadUserByUsername(username);
        String token = JwtUtils.generateUsersToken(userDetails);

        return token;
    }

    /**
     * 创建token
     *
     * @return
     * @throws Exception
     */
    @PostMapping("/log")
    @ApiOperation("日志")
    public void log(@RequestBody String json) throws Exception {

        String path = new File("").getAbsolutePath();
        FileWriter fileWriter = new FileWriter(path + File.separator + "request.log");
        fileWriter.append("时间：" + DateUtil.now());
        fileWriter.append("\r\n");
        fileWriter.append(json);
        fileWriter.append("\r\n");
    }

    /**
     * 将本次获取到的token存入Security的上下文中
     *
     * @param username
     * @param password
     * @throws Exception
     */
    private void authenticate(String username, String password) throws Exception {
        try {
            //authenticate()会调用 userDetailsService去验证用户
            authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(username, password));
        } catch (DisabledException e) {
            throw new Exception("USER_DISABLED", e);
        } catch (BadCredentialsException e) {
            throw new Exception("INVALID_CREDENTIALS", e);
        }
    }
}
