package cn.wise.sc.pay.common.security2.core;

import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @ClassNameCustomAuthenticationEntryPoint
 * @Description 认证入口点  它在用户请求处理过程中遇到认证异常时，
 * 被ExceptionTranslationFilter用于开启特定认证方案(authentication schema)的认证流程。
 * 自定义的授权不通过返回信息
 * @Author qh
 * @Date2020/3/5 14:28
 * @Version V1.0
 **/
@Component
public class JwtAuthenticationEntryPoint implements AuthenticationEntryPoint {

    @Override
    public void commence(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, AuthenticationException e) throws IOException, ServletException {

        if (e instanceof BadCredentialsException) {
            httpServletResponse.sendError(HttpServletResponse.SC_UNAUTHORIZED, "密码账号验证不通过!");
        } else if(e instanceof LockedException){
            httpServletResponse.sendError(HttpServletResponse.SC_UNAUTHORIZED, "账户已锁定请联系管理员!");
        }else {
            httpServletResponse.sendError(HttpServletResponse.SC_UNAUTHORIZED, "授权不通过！");
        }

    }
}

