package cn.wise.sc.pay.common.security2.core;

import cn.wise.sc.pay.common.security2.SysUser;
import cn.wise.sc.pay.common.security2.util.JwtUtils;
import io.jsonwebtoken.ExpiredJwtException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.www.BasicAuthenticationFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;


/**
 * @description: token验证过滤器  每次请求都会调
 * BasicAuthenticationFilter  用于验证请求header的过滤器
 * @author: qh
 * @create: 2020-03-09 23:04
 **/
@Slf4j
@Deprecated
public class JwtTokenVerifyFilter extends BasicAuthenticationFilter {

    public JwtTokenVerifyFilter(AuthenticationManager authenticationManager) {
        super(authenticationManager);
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request,
                                    HttpServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        String srt = request.getRequestURI();
        String header = request.getHeader(JwtUtils.TOKEN_HEADER);
        //如果token获取失败
        if (header == null || !header.startsWith(JwtUtils.TOKEN_PREFIX)) {
            response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "未授权，请先登录!");
            chain.doFilter(request, response);
            return;
        }

        //解析token
        String token = header.substring(7);
        try {
            SysUser sysUser = JwtUtils.decodeToken(token);
            //根据token生成用户信息凭证
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(sysUser.getUsername(),
                    null, sysUser.getAuthorities());
            SecurityContextHolder.getContext().setAuthentication(authenticationToken);
        } catch (ExpiredJwtException e) {
            response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "token过期,请重新获取!");
            SecurityContextHolder.clearContext();
        } catch (Exception e) {
            response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "无效token,请重新获取!");
            SecurityContextHolder.clearContext();
        } finally {
            chain.doFilter(request, response);
        }
    }

}
