package cn.wise.sc.pay.business.comment.controller;

import cn.wise.sc.pay.business.comment.bean.Reply;
import cn.wise.sc.pay.business.comment.service.IReplyService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-12-7
 */
public class CommonReplyController {

    @Autowired
    IReplyService replyService;

    @PostMapping("/addReply")
    @ApiOperation("评论回复")
    @ApiImplicitParam(name = "reply", value = "评论回复实体", paramType = "body", dataType = "Reply")
    public BaseResponse inserReply(@RequestBody Reply reply){
        Integer result = replyService.insertReply(reply);
        return result == 1 ? BaseResponse.ok(result) : BaseResponse.error("回复失败!",CommonReplyController.class);
    }

    @DeleteMapping("/deleteById")
    @ApiOperation("删除评论回复")
    @ApiImplicitParam(name = "id", value = "回复id", paramType = "path", dataType = "Long")
    public BaseResponse deleteById(Long id){
        if(id == null){
            return BaseResponse.error("删除回复失败!",CommonReplyController.class);
        }
        Integer result = replyService.deleteById(id);
        return BaseResponse.ok(result);
    }

    @DeleteMapping("/deleteByCommentIds")
    @ApiOperation("删除评论回复根据评论id列表")
    @ApiImplicitParam(name = "commentIdStr", value = "评论id 列表", paramType = "body", dataType = "List")
    public BaseResponse deleteByCommentIds(String commentIdStr){
        List<String> commentIds = new ArrayList<>();
        if(StringUtils.isNotBlank(commentIdStr)){
            String[] comIdStrs = commentIdStr.split(",");
            for (String id : comIdStrs) {
                commentIds.add(id);
            }
        }
        Integer result = replyService.deleteByCommentId(commentIds);
        return BaseResponse.ok(result);
    }

    @PutMapping("/updatePariseNum")
    @ApiOperation("点赞数+1")
    @ApiImplicitParam(name = "id", value = "回复id", paramType = "path", dataType = "Long")
    public BaseResponse updatePraiseNum(Long id){
        if(id == null){
            return BaseResponse.error("点赞失败!",CommonReplyController.class);
        }
        Integer result = replyService.updatePraiseNum(id);
        return BaseResponse.ok(result);
    }
}
