package cn.wise.sc.pay.business.comment.service.impl;

import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.business.comment.bean.Reply;
import cn.wise.sc.pay.business.comment.bean.Source;
import cn.wise.sc.pay.business.comment.mapper.CommentMapper;
import cn.wise.sc.pay.business.comment.mapper.ReplyMapper;
import cn.wise.sc.pay.business.comment.mapper.SourceMapper;
import cn.wise.sc.pay.business.comment.service.ICommentService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.util.FastDFSUtils;
import cn.wise.sc.pay.common.core.util.FileTypeUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@Service
public class CommentServiceImpl implements ICommentService {

    @Autowired
    CommentMapper commentMapper;

    @Autowired
    ReplyMapper replyMapper;

    @Autowired
    SourceMapper sourceMapper;

    @Autowired
     FastFileStorageClient fastFileStorageClient;

    @Autowired
     Environment environment;

    @Override
    public BaseResponse insertComment(Comment comment) {
        comment.setCreateTime(System.currentTimeMillis());
        comment.setUpdateTime(System.currentTimeMillis());
        Integer res = commentMapper.insert(comment);
        List<String> sources = comment.getSources();
        if (sources != null && sources.size() > 0) {
            QueryWrapper<Comment> wrapper = new QueryWrapper<>();
            wrapper.eq("target_id", comment.getTargetId());
            wrapper.eq("critic_id", comment.getCriticId());
            wrapper.orderByDesc("id");
            wrapper.last("limit 1");
            Comment com = commentMapper.selectOne(wrapper);
            if (com == null) {
                com = commentMapper.selectOne(wrapper);
            }
            if (com != null) {
                int order = 1;
                for (String url : sources) {
                    String extension = FilenameUtils.getExtension(url);
                    Source sour = Source.builder()
                            .aocId(com.getId())
                            .url(url)
                            .orderNum(order)
                            .type(FileTypeUtils.checkFileExtension(extension))
                            .build();
                    sourceMapper.insert(sour);
                    order++;
                    if (order > 9) {
                        break;
                    }
                }
            }
        }
        return res == 1 ? BaseResponse.ok("评论发表成功") :
                BaseResponse.error("评论发表失败", Comment.class);
    }

    @Override
    public BaseResponse deleteByIds(List<String> ids) {
        UpdateWrapper<Comment> wrapper = new UpdateWrapper<>();
        wrapper.in("id", ids);
        int res = commentMapper.delete(wrapper);
        UpdateWrapper<Reply> delWrapper = new UpdateWrapper<>();
        delWrapper.in("cor_id", ids);
        replyMapper.delete(delWrapper);
        return res >= 1 ? BaseResponse.ok("删除成功") : BaseResponse.error("删除失败", Comment.class);
    }

    @Override
    public BaseResponse getCommentList(PageQuery pageQuery) {

        FastDFSUtils fastDFSUtils = new FastDFSUtils(fastFileStorageClient, environment);

        Page<Comment> page = new Page<>();
        if (pageQuery != null) {
            page.setSize(pageQuery.getPageSize());
            page.setCurrent(pageQuery.getPageNo());
        }
        //评论主体信息
        IPage<Comment> commentIPage = commentMapper.selectPage(page, null);
        if (commentIPage == null) {
            return BaseResponse.ok((Comment) null, this.getClass());
        }
        //
        List<Long> commentIds = null;
        List<Comment> records = commentIPage.getRecords();
        if (!CollectionUtils.isEmpty(records)) {
            commentIds = records.stream().map(re -> re.getId()).collect(Collectors.toList());
        }
        //评论资源信息
        List<Source> sources = null;
        if (!CollectionUtils.isEmpty(commentIds)) {
            QueryWrapper<Source> sourceQueryWrapper = new QueryWrapper<>();
            sourceQueryWrapper.in("aoc_id", commentIds);
            sources = sourceMapper.selectList(sourceQueryWrapper);
        }
        //遍历设值评论资源
        if (!CollectionUtils.isEmpty(sources)) {
            List<Source> finalSources = sources;

            records.stream().forEach(re -> {

                List<Source> sourceArrayList = new ArrayList<>();
                finalSources.stream().forEach(sr -> {

                    if (re.getId().equals(sr.getAocId())) {
                        sourceArrayList.add(sr);
                    }
                });
                //填充图片前缀
                sourceArrayList.stream().forEach(re1 -> re1.setUrl(fastDFSUtils.fullUrl(re1.getUrl())));
                re.setSourceVo(sourceArrayList);
            });
        }
        page.setRecords(records);
        return BaseResponse.ok(page);
    }

    @Override
    public BaseResponse updatePraiseNum(Long id) {
        Comment comment = commentMapper.selectById(id);
        comment.setPraiseNum(comment.getPraiseNum() + 1);
        comment.setUpdateTime(System.currentTimeMillis());
        commentMapper.updateById(comment);
        return BaseResponse.ok("点赞成功");
    }

    @Override
    public BaseResponse<Page<Comment>> selectByTargetId(Long targetId, PageQuery pageQuery) {

        FastDFSUtils fastDFSUtils = new FastDFSUtils(fastFileStorageClient, environment);

        Page<Comment> page = new Page<>();
        if (targetId != null) {
            if (pageQuery != null) {
                page.setCurrent(pageQuery.getPageNo());
                page.setSize(pageQuery.getPageSize());
            }
            QueryWrapper<Comment> wrapper = new QueryWrapper<>();
            wrapper.eq("target_id", targetId);
            wrapper.orderByDesc("update_time");
            page = (Page<Comment>) commentMapper.selectPage(page, wrapper);
            List<Comment> comments = page.getRecords();
            if (comments == null) {
                return BaseResponse.ok(page, this.getClass());
            }
            List<Long> ids = new ArrayList<>();
            for (Comment comment : comments) {
                ids.add(comment.getId());
            }
            if (ids.size() > 0) {
                //添加评论回复数据
                QueryWrapper<Reply> queryWrapper = new QueryWrapper<>();
                queryWrapper.in("cor_id", ids);
                queryWrapper.orderByAsc("create_time");
                List<Reply> replies = replyMapper.selectList(queryWrapper);
                //添加评论资源图片
                QueryWrapper<Source> sourceQueryWrapper = new QueryWrapper<>();
                sourceQueryWrapper.in("aoc_id", ids);
                List<Source> sources = sourceMapper.selectList(sourceQueryWrapper);

                for (Comment comment : comments) {
                    List<Reply> commentReplys = new ArrayList<>();
                    List<Source> sourceList = new ArrayList<>();

                    if (!CollectionUtils.isEmpty(replies)) {
                        for (Reply re : replies) {
                            if (comment.getId().longValue() == re.getCorId().longValue()) {
                                commentReplys.add(re);
                            }
                        }
                    }
                    if (!CollectionUtils.isEmpty(sources)) {
                        sources.stream().forEach(sr -> {
                            if (sr.getAocId().equals(comment.getId())) {
                                sourceList.add(sr);
                            }
                        });
                    }
                    //添加图片前缀
                    sourceList.stream().forEach(rel -> rel.setUrl(fastDFSUtils.fullUrl(rel.getUrl())));
                    comment.setSourceVo(sourceList);
                    comment.setReplies(commentReplys);
                }
                page.setRecords(comments);
            }
        }

        return BaseResponse.ok(page);
    }

    @Override
    public int getCommentCount(QueryWrapper qw) {
        return commentMapper.selectCount(qw);
    }
}
