package cn.wise.sc.pay.business.goods.configure;

import cn.hutool.core.util.StrUtil;
import com.github.tobato.fastdfs.domain.fdfs.MetaData;
import com.github.tobato.fastdfs.domain.fdfs.StorePath;
import com.github.tobato.fastdfs.domain.proto.storage.DownloadByteArray;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.HashSet;
import java.util.Set;

/**
 * @description: 文件上传工具类
 * @author: qh
 * @create: 2019-11-04 15:08
 **/
@Component(value = "fastdfs")
public class FastDFSUtils {

    @Autowired
    private FastFileStorageClient fastFileStorageClient;

    @Autowired
    Environment environment;

    /**
     * 文件上传
     *
     * @param file
     * @return
     * @throws IOException
     */
    public StorePath uploadFile(MultipartFile file) throws IOException {

        // 设置文件信息
        Set<MetaData> mataData = new HashSet<>();

        // 上传   （文件上传可不填文件信息，填入null即可）
        StorePath storePath = fastFileStorageClient.uploadFile(file.getInputStream(),
                file.getSize(), FilenameUtils.getExtension(file.getOriginalFilename()), mataData);

        return storePath;
    }

    /**
     * 获取完整路径
     *
     * @param relativeUrl
     * @return
     */
    public String fullUrl(String relativeUrl) {
        if (StrUtil.isNotEmpty(relativeUrl)) {
            String host = environment.getProperty("fdfs.imgaddress");
            return host + relativeUrl;
        }else {
            return "";
        }
    }

    /**
     * 文件删除
     *
     * @param path
     * @return
     */
    public String delete(String path) {

        // 第一种删除：参数：完整地址
        fastFileStorageClient.deleteFile(path);

        // 第二种删除：参数：组名加文件路径
        // fastFileStorageClient.deleteFile(group,path);

        return "删除成功！";
    }


    /**
     * 文件下载
     *
     * @param path
     * @return
     */
    public void downLoad(String group, String path, String fileName, HttpServletResponse response) throws IOException {

        // 获取文件
        byte[] bytes = fastFileStorageClient.downloadFile(group, path, new DownloadByteArray());

        //设置相应类型application/octet-stream        （注：applicatoin/octet-stream 为通用，一些其它的类型苹果浏览器下载内容可能为空）
        response.reset();
        response.setContentType("applicatoin/octet-stream");
        //设置头信息                 Content-Disposition为属性名  附件形式打开下载文件   指定名称  为 设定的fileName
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        // 写入到流
        ServletOutputStream out = response.getOutputStream();
        out.write(bytes);

        out.close();
    }
}
