package cn.wise.sc.pay.business.goods.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.business.goods.configure.FastDFSUtils;
import cn.wise.sc.pay.business.goods.mapper.GoodsMapper;
import cn.wise.sc.pay.business.goods.model.GoodsResponse;
import cn.wise.sc.pay.business.goods.model.GoodsStatus;
import cn.wise.sc.pay.business.goods.model.bean.Goods;
import cn.wise.sc.pay.business.goods.model.query.GoodsQuery;
import cn.wise.sc.pay.business.goods.model.query.GoodsScanQuery;
import cn.wise.sc.pay.business.goods.model.query.SpecificationQuery;
import cn.wise.sc.pay.business.goods.model.vo.GoodsExcel;
import cn.wise.sc.pay.business.goods.model.vo.GoodsVO;
import cn.wise.sc.pay.business.goods.service.IGoodsService;
import cn.wise.sc.pay.business.goods.service.ISpecificationService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 商品 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-12-03
 */
@Service
@SuppressWarnings("all")
public class GoodsServiceImpl extends ServiceImpl<GoodsMapper, Goods> implements IGoodsService {

    @Autowired
    ISpecificationService iSpecService;
    @Autowired
    FastDFSUtils fastDFSUtils;
    @Autowired
    IMerchantService merchantService;

    @Override
    public GoodsResponse newGoods(GoodsQuery goodsQuery) {

        Goods goods = Goods.builder()
                .name(goodsQuery.getName())
                .cover(goodsQuery.getCover())
                .field1(goodsQuery.getPrice())
                .field2(goodsQuery.getUnit())
                .status(GoodsStatus.WaitAudit.getCode())
                .type(goodsQuery.getType())
                .field5(goodsQuery.getShopName())
                .sellerId(goodsQuery.getSellerId())
                .build();

        if (goodsQuery.getStatus() != null){
            goods.setStatus(GoodsStatus.Selling.getCode());
        }
        long id = goodsQuery.getSellerId();
        BaseResponse<Merchant> merchantInfo = merchantService.getMerchantInfo(id);
        Merchant body = merchantInfo.getData();
        if (body == null) {
            return GoodsResponse.error("商家不存在");
        }
        if (body.getWhiteList() == 1) {
            goods.setStatus(GoodsStatus.Selling.getCode());
        }
        List<SpecificationQuery> spcs = goodsQuery.getSpcQuery();


        if (spcs == null || spcs.size() == 0) {
            goods.setField3("0");
            boolean isSuccess = this.save(goods);
        } else {
            goods.setField3("1");
            boolean isSuccess = this.save(goods);
            for (SpecificationQuery spc : spcs) {
                spc.setGoodsId(goods.getId());
                spc.setSellerId(goods.getSellerId());
                iSpecService.newSpecification(spc);
            }
        }

        GoodsVO goodsVO = new GoodsVO(goods);
        GoodsResponse goodsResponse = GoodsResponse.ok("商品添加成功", goodsVO);
        return goodsResponse;
    }

    @Override
    public GoodsResponse editGoods(GoodsQuery goodsQuery) {

        if (goodsQuery.getId() == 0) {
            return GoodsResponse.error("商品id不能为空");
        }

        if (goodsQuery.getSellerId() == 0) {
            return GoodsResponse.error("商户id不能为空");
        }

        UpdateWrapper ew = new UpdateWrapper();
        ew.eq("seller_id", goodsQuery.getSellerId());
        ew.eq("id", goodsQuery.getId());

        Goods upGoods = Goods.builder()
                .sellerId(goodsQuery.getSellerId())
                .type(goodsQuery.getType())
                .status(goodsQuery.getStatus())
                .cover(goodsQuery.getCover())
                .name(goodsQuery.getName())
                .field1(goodsQuery.getPrice())
                .field2(goodsQuery.getUnit())
                .field3(goodsQuery.getSpc())
                .field5(goodsQuery.getShopName())
                .id(goodsQuery.getId())
                .build();

        boolean isSuccess = this.update(upGoods, ew);


        List<SpecificationQuery> spcs = goodsQuery.getSpcQuery();
        for (SpecificationQuery spc : spcs) {
            isSuccess = iSpecService.editSpecification(spc).getCode() == 200 ? true : false;
        }

        GoodsVO goodsVO = new GoodsVO(upGoods);
        return isSuccess ? GoodsResponse.ok("更新成功", goodsVO) : GoodsResponse.error("更新失败");
    }

    @Override
    public GoodsResponse delGoods(long id, long sellerId) {

        boolean isSuccess = checkIdAndSeller(id, sellerId);

        if (isSuccess) {

            QueryWrapper qw = new QueryWrapper();
            qw.eq("id", id);
            qw.eq("seller_id", sellerId);
            qw.eq("status", 0);

            isSuccess = this.remove(qw);
        }
        return isSuccess ? GoodsResponse.ok("删除成功") : GoodsResponse.error("删除失败");

    }

    @Override
    public GoodsResponse unshelveGoods(long id, long sellerId) {

        boolean isSuccess = checkIdAndSeller(id, sellerId);

        if (isSuccess) {

            QueryWrapper qw = new QueryWrapper();
            qw.eq("id", id);
            qw.eq("seller_id", sellerId);
            qw.eq("status", 1);

            Goods updateGoods = Goods.builder()
                    .status(0)
                    .build();

            isSuccess = this.update(updateGoods, qw);
        }

        return isSuccess ? GoodsResponse.ok("下架成功") : GoodsResponse.error("下架失败");
    }

    @Override
    public GoodsResponse shelveGoods(long id, long sellerId) {
        boolean isSuccess = checkIdAndSeller(id, sellerId);

        if (isSuccess) {

            QueryWrapper qw = new QueryWrapper();
            qw.eq("id", id);
            qw.eq("seller_id", sellerId);
            qw.eq("status", GoodsStatus.Unshelve.getCode());

            Goods updateGoods = Goods.builder()
                    .status(1)
                    .build();

            isSuccess = this.update(updateGoods, qw);
        }

        return isSuccess ? GoodsResponse.ok("上架成功") : GoodsResponse.error("上架失败");
    }

    @Override
    public BaseResponse list(long sellerId, String key, String type, int status) {

        QueryWrapper qw = new QueryWrapper();

        if (StrUtil.isNotEmpty(type)) {
            qw.like("type", type);
        }
        if (sellerId != 0) {
            qw.eq("seller_id", sellerId);
        }
        if (StrUtil.isNotEmpty(key)) {
            try {
                int id = Integer.valueOf(key);
                qw.eq("id", id);
            } catch (Exception ex) {
                qw.like("name", key);
            }
        }
        if (status != -1) {
            qw.eq("status", status);
        }
        try {

            List<Goods> list = this.list(qw);
            List<GoodsVO> goodsVOList = new ArrayList<>(list.size());
            for (Goods goods : list) {
                GoodsVO goodsVO = new GoodsVO(goods);
                goodsVO.setCover(goods.getCover());
                goodsVOList.add(goodsVO);
            }

            BaseResponse baseResponse = BaseResponse.ok("查询集合成功");
            baseResponse.setData(goodsVOList);
            return baseResponse;
        } catch (Exception ex) {
            BaseResponse baseResponse = BaseResponse.error("查询集合异常", GoodsVO.class.getName());
            return baseResponse;
        }
    }

    @Override
    public GoodsResponse getGoods(int goodsId) {

        try {

            Goods goods = this.getById(goodsId);
            GoodsVO goodsVO = new GoodsVO(goods);
            goodsVO.setCover(fastDFSUtils.fullUrl(goods.getCover()));
            return GoodsResponse.ok("找到了", goodsVO);

        } catch (Exception ex) {

            return GoodsResponse.error("没找着呢");
        }
    }

    @Override
    public BaseResponse getList(String key, String shopName, String goodsType) {

        QueryWrapper qw = new QueryWrapper();
        if (StrUtil.isNotEmpty(key)) {
            qw.like("name", key);
        }
        if (StrUtil.isNotEmpty(shopName)) {
            qw.like("field4", shopName);
        }
        if (StrUtil.isNotEmpty(goodsType)) {
            qw.eq("type", goodsType);
        }

        try {
            List<Goods> list = this.list(qw);
            List<GoodsVO> voList = new ArrayList<>(list.size());
            for (Goods goods : list) {
                GoodsVO goodsVO = new GoodsVO(goods);
                voList.add(goodsVO);
            }
            BaseResponse response = BaseResponse.ok("查询成功");
            response.setData(voList);
            return response;

        } catch (Exception ex) {
            return GoodsResponse.error("查询失败");
        }
    }

    @Override
    public GoodsResponse auditGoods(long goodsId, String cause, int status) {

        if (goodsId == 0) {
            return GoodsResponse.error("参数不错误");
        }
        if (status > GoodsStatus.NotAllow.getCode() || status < 0) {
            status = GoodsStatus.NotAllow.getCode();
        }
        UpdateWrapper uw = new UpdateWrapper();

        Goods goods = Goods.builder()
                .status(status)
                .id(goodsId)
                .field4(cause)
                .build();
        try {
            this.updateById(goods);
            return GoodsResponse.ok("审核完成");
        } catch (Exception ex) {
            return GoodsResponse.error("请重新审核");
        }
    }

    @Override
    public BaseResponse getWaitingAuditList(String key, String shopName, String goodsType) {

        QueryWrapper qw = new QueryWrapper();
        qw.eq("status", GoodsStatus.WaitAudit.getCode());
        if (StrUtil.isNotEmpty(key)) {
            qw.like("name", key);
        }
        if (StrUtil.isNotEmpty(shopName)) {
            qw.like("field5", shopName);
        }
        if (StrUtil.isNotEmpty(goodsType)) {
            qw.eq("type", goodsType);
        }
        try {
            List<Goods> list = this.list(qw);
            List<GoodsVO> voList = new ArrayList<>(list.size());
            for (Goods goods : list) {
                GoodsVO goodsVO = new GoodsVO(goods);
                voList.add(goodsVO);
            }
            BaseResponse response = BaseResponse.ok("查询成功");
            response.setData(voList);
            return response;

        } catch (Exception ex) {
            return GoodsResponse.error("查询失败");
        }
    }

    @Override
    public BaseResponse getPage(PageQuery pageQuery, String key, long shopName, String goodsType) {

        IPage page = new Page(pageQuery.getPageNo(), pageQuery.getPageSize());
        QueryWrapper qw = new QueryWrapper();
        qw.orderByDesc("id");
        qw.ne("status", GoodsStatus.NotAllow.getCode());
        qw.ne("status", GoodsStatus.WaitAudit.getCode());
        if (StrUtil.isNotEmpty(key)) {
            qw.like("name", key);
        }
        if (shopName != 0) {
            qw.like("seller_id", shopName);
        }
        if (StrUtil.isNotEmpty(goodsType)) {
            qw.eq("type", goodsType);
        }

        try {
            page = this.page(page, qw);
            List<Goods> list = page.getRecords();

            List<GoodsVO> voList = new ArrayList<>(list.size());
            for (Goods goods : list) {
                GoodsVO goodsVO = new GoodsVO(goods);
                voList.add(goodsVO);
            }
            page.setRecords(voList);
            BaseResponse response = BaseResponse.ok("查询成功");
            response.setPage((Page) page);
            return response;

        } catch (Exception ex) {
            return GoodsResponse.error("查询失败");
        }
    }

    @Override
    public BaseResponse getWaitingAuditPage(PageQuery pageQuery, String key, Long shopId, String goodsType) {

        IPage page = new Page(pageQuery.getPageNo(), pageQuery.getPageSize());
        QueryWrapper qw = new QueryWrapper();
        qw.orderByDesc("id");
        qw.eq("status", GoodsStatus.WaitAudit.getCode());
        if (StrUtil.isNotEmpty(key)) {
            qw.like("name", key);
        }
        if (shopId != null) {
            qw.like("seller_id", shopId);
        }
        if (StrUtil.isNotEmpty(goodsType)) {
            qw.eq("type", goodsType);
        }
        try {
            page = this.page(page, qw);
            List<Goods> list = page.getRecords();
            List<GoodsVO> voList = new ArrayList<>(list.size());
            for (Goods goods : list) {
                GoodsVO goodsVO = new GoodsVO(goods);
                voList.add(goodsVO);
            }
            page.setRecords(voList);
            BaseResponse response = BaseResponse.ok("查询成功");
            response.setPage((Page) page);
            return response;

        } catch (Exception ex) {
            return GoodsResponse.error("查询失败");
        }
    }

    @Override
    public GoodsResponse unshelveGoods(int id, int sellerId, String cause) {
        boolean isSuccess = checkIdAndSeller(id, sellerId);

        if (isSuccess) {

            QueryWrapper qw = new QueryWrapper();
            qw.eq("id", id);
            qw.eq("seller_id", sellerId);
            qw.eq("status", GoodsStatus.Selling.getCode());

            Goods updateGoods = Goods.builder()
                    .status(GoodsStatus.Unshelve.getCode())
                    .field4(cause)
                    .build();

            isSuccess = this.update(updateGoods, qw);
        }

        return isSuccess ? GoodsResponse.ok("下架成功") : GoodsResponse.error("下架失败");
    }


    /**
     * 判断商品id和卖家id是否有效
     *
     * @param id
     * @param sellerId
     * @return
     */
    private boolean checkIdAndSeller(long id, long sellerId) {
        if (id == 0 || sellerId == 0) {
            return false;
        }
        return true;
    }

    @Override
    public List<GoodsExcel> getOneGoodsExcel(String typeName) {
        GoodsExcel goodsSpecExcel = GoodsExcel.builder()
                .name("风味奶茶")
                .cover("M00/00/02/rBEAEF8j00-AdIcKAAALfVm4HKQ084.png")
                .type(typeName)
                .price("20.5")
                .unit("杯")
                .spc("无")
                .build();
        List<GoodsExcel> list = new ArrayList<>();
        list.add(goodsSpecExcel);
        return list;
    }

    @Override
    public void saveGoods(List<GoodsExcel> datas) {
        long l = datas.get(0).getSellerId().longValue();
        BaseResponse<Merchant> merchantInfo = merchantService.getMerchantInfo(l);
        Merchant body = merchantInfo.getData();
        if (body == null) {
            throw new RuntimeException("商户不存在");
        }
        List<Goods> goodsList = new ArrayList<>();
        for (GoodsExcel excel : datas) {
            int status = 0;
            if (body.getWhiteList() == 1) {
                status = 1;
            }
            Goods goods = Goods.builder()
                    .cover(excel.getCover())
                    .status(status)
                    .name(excel.getName())
                    .sellerId(body.getId())
                    .type(excel.getType())
                    .field1(excel.getPrice())
                    .field2(excel.getUnit())
                    .field3(excel.getSpc())
                    .code(excel.getCode())
                    .build();
            goodsList.add(goods);
        }
        this.saveBatch(goodsList);
    }

    @Override
    public BaseResponse<Goods> scanAddGoods(GoodsScanQuery goodsScanQuery) {

        Goods goods = Goods.builder()
                .field1(goodsScanQuery.getPrice().doubleValue() + "")
                .type(goodsScanQuery.getType())
                .name(goodsScanQuery.getName())
                .code(goodsScanQuery.getCode())
                .sellerId(goodsScanQuery.getSellerId())
                .status(1)
                .build();

        QueryWrapper qw = new QueryWrapper();
        qw.eq("code", goods.getCode());
        qw.eq("seller_id",goods.getSellerId());
        int count = this.count(qw);
        if (count != 0) {
            return BaseResponse.error("当前条形码已被使用!", this.getClass().getName());
        }

        boolean rts = this.save(goods);

        return rts ? BaseResponse.ok() : BaseResponse.error("扫码录入失败!", this.getClass().getName());
    }

    @Override
    public BaseResponse<Goods> getGoodsInfoByCode(String code) {

        if (StrUtil.isEmpty(code)) {
            return BaseResponse.error("二维码获取失败!", this.getClass().getName());
        }

        QueryWrapper<Goods> qw = new QueryWrapper<>();
        qw.eq("code", code);
        Goods goods = this.getOne(qw);

        return BaseResponse.ok(goods);
    }

    @Override
    public BaseResponse<Boolean> batchAdd(List<Goods> goods) {
        return BaseResponse.ok(this.saveBatch(goods));
    }

    @Override
    public List<Goods> getByGoodIds(List<Long> ids) {

        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.in("id",ids);

        return this.list(queryWrapper);
    }
}
