package cn.wise.sc.pay.business.goods.service.impl;

import cn.wise.sc.pay.business.goods.mapper.SpecificationMapper;
import cn.wise.sc.pay.business.goods.model.SpcResponse;
import cn.wise.sc.pay.business.goods.model.bean.SpcDetail;
import cn.wise.sc.pay.business.goods.model.bean.SpcDetailBO;
import cn.wise.sc.pay.business.goods.model.bean.Specification;
import cn.wise.sc.pay.business.goods.model.query.SpcDetailQuery;
import cn.wise.sc.pay.business.goods.model.query.SpecificationQuery;
import cn.wise.sc.pay.business.goods.model.vo.SpcDetailVO;
import cn.wise.sc.pay.business.goods.model.vo.SpecificationVO;
import cn.wise.sc.pay.business.goods.service.ISpcDetailService;
import cn.wise.sc.pay.business.goods.service.ISpecificationService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 商品规格 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-12-03
 */
@Service
public class SpecificationServiceImpl extends ServiceImpl<SpecificationMapper, Specification> implements ISpecificationService {


    @Autowired
    SpecificationMapper specificationMapper;
    @Autowired
    ISpcDetailService iSpcDetailService;

    /**
     * 判断商品id和卖家id是否有效
     *
     * @param id
     * @param sellerId
     * @return
     */
    private boolean checkIdAndSeller(long id, long sellerId) {
        return id != 0 && sellerId != 0;
    }

    @Override
    @Transactional
    public SpcResponse newSpecification(SpecificationQuery specificationQuery) {

        Specification newSpecification = Specification.builder()
                .goodsId(specificationQuery.getGoodsId())
                .name(specificationQuery.getName())
                .sellerId(specificationQuery.getSellerId())
                .status(specificationQuery.getStatus())
                .build();

        QueryWrapper qw = new QueryWrapper();
        qw.eq("name", specificationQuery.getName());
        qw.eq("seller_id", specificationQuery.getSellerId());
        qw.eq("goods_id", specificationQuery.getGoodsId());
        int count = this.count(qw);
        if (count > 0) {
            return SpcResponse.error("当前商品规格已存在!");
        }

        boolean isSalve = this.save(newSpecification);

        if (isSalve) {
            List<SpcDetailQuery> spcDetailQuerys = specificationQuery.getSpcDetailQuery();
            List<SpcDetail> spcDetails = new ArrayList<>(spcDetailQuerys.size());
            for (SpcDetailQuery spcDetailQuery : spcDetailQuerys) {
                SpcDetail spcDetail = SpcDetail.builder()
                        .name(spcDetailQuery.getName())
                        .price(new BigDecimal(spcDetailQuery.getPrice()))
                        .store(spcDetailQuery.getStore())
                        .spcId(newSpecification.getId())
                        .build();
                spcDetails.add(spcDetail);
            }

            isSalve = iSpcDetailService.saveBatch(spcDetails);
        }

        return isSalve ? SpcResponse.ok("添加成功") : SpcResponse.error("添加失败");
    }

    @Override
    public SpcResponse editSpecification(SpecificationQuery specificationQuery) {

        if (!checkIdAndSeller(specificationQuery.getId(), specificationQuery.getSellerId())) {
            return SpcResponse.error("参数错误");
        }
        Specification updateSpec = Specification.builder()
                .name(specificationQuery.getName())
                .status(specificationQuery.getStatus())
                .build();
        UpdateWrapper uw = new UpdateWrapper();
        uw.eq("seller_id", specificationQuery.getSellerId());
        uw.eq("id", specificationQuery.getId());
        boolean isUpdate = this.update(updateSpec, uw);

        //批量更新
        if (isUpdate) {
            List<SpcDetailQuery> spcDetailQuerys = specificationQuery.getSpcDetailQuery();
            List<SpcDetail> updateSpcDetails = new ArrayList<>(spcDetailQuerys.size());
            for (SpcDetailQuery spcDetailQuery : spcDetailQuerys) {
                SpcDetail spcDetail = SpcDetail.builder()
                        .name(spcDetailQuery.getName())
                        .price(new BigDecimal(spcDetailQuery.getPrice()))
                        .store(spcDetailQuery.getStore())
                        .id(spcDetailQuery.getId())
                        .build();
                updateSpcDetails.add(spcDetail);
            }

            isUpdate = iSpcDetailService.updateBatchById(updateSpcDetails);
        }

        return isUpdate ? SpcResponse.ok("更新成功") : SpcResponse.error("更新失败");
    }

    @Override
    public SpcResponse delSpecification(long id, long sellerId) {

        if (!checkIdAndSeller(id, sellerId)) {
            return SpcResponse.error("参数错误");
        }
        boolean isRemove = this.removeById(id);

        QueryWrapper qw = new QueryWrapper();
        qw.eq("spc_id", id);

        isRemove = iSpcDetailService.remove(qw);

        return isRemove ? SpcResponse.ok("删除成功") : SpcResponse.error("删除失败");
    }

    @Override
    public BaseResponse getList(long goodsId) {

        if (goodsId == 0) {
            return SpcResponse.error("参数错误");
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("goods_id", goodsId);

        try {
            List<Specification> specs = this.list(qw);
            List<Long> spcIds = specs.stream().map(arg -> arg.getId()).collect(Collectors.toList());

            QueryWrapper qwSpcDetail = new QueryWrapper();
            qwSpcDetail.in("spc_id", spcIds);
            List<SpcDetail> spcDetails = iSpcDetailService.list(qwSpcDetail);

            List<SpecificationVO> specificationVOS = new ArrayList<>(specs.size());
            for (Specification spec : specs) {

                SpecificationVO specificationVO = SpecificationVO.builder()
                        .name(spec.getName())
                        .id(spec.getId())
                        .sellerId(spec.getSellerId())
                        .build();

                List<SpcDetail> oneSpcDetails = spcDetails.stream()
                        .filter(arg -> arg.getSpcId().longValue() == spec.getId())
                        .collect(Collectors.toList());

                if (oneSpcDetails.size() != 0) {
                    List<SpcDetailVO> spcDetailVOS = new ArrayList<>(oneSpcDetails.size());
                    for (SpcDetail spcDetail : oneSpcDetails) {

                        SpcDetailVO spcDetailVO = SpcDetailVO.builder()
                                .id(spcDetail.getId())
                                .name(spcDetail.getName())
                                .price(spcDetail.getPrice())
                                .store(spcDetail.getStore())
                                .build();
                        spcDetailVOS.add(spcDetailVO);
                    }

                    specificationVO.setSpcDetailVO(spcDetailVOS);
                }
                specificationVOS.add(specificationVO);
            }
            BaseResponse baseResponse = BaseResponse.ok("查询成功");
            baseResponse.setList(specificationVOS);
            return baseResponse;
        } catch (Exception ex) {

            return SpcResponse.error("查询失败");
        }
    }


    @Override
    public BaseResponse getGoodsSpc(int goodsId) {

        if (goodsId == 0) {
            return SpcResponse.error("参数有误");
        }

        List<SpcDetailBO> spcDetailBOS = this.getSpcAllList(goodsId);

        Set<Long> spcIds = spcDetailBOS.stream().map(arg -> arg.getSpecification().getId()).collect(Collectors.toSet());

        List<SpecificationVO> spcVOs = new ArrayList<>(spcIds.size());

        for (Long spcId : spcIds) {

            List<SpcDetailBO> oneSpcDetailBOs = spcDetailBOS.stream()
                    .filter(arg -> arg.getSpecification().getId().longValue() == spcId)
                    .collect(Collectors.toList());

            SpecificationVO spcVO = SpecificationVO.builder()
                    .id(spcId)
                    .name(oneSpcDetailBOs.get(0).getSpecification().getName())
                    .sellerId(oneSpcDetailBOs.get(0).getSpecification().getSellerId())
                    .build();
            List<SpcDetailVO> spcDetailVOS = new ArrayList<>(oneSpcDetailBOs.size());

            for (SpcDetailBO spcDetailBO : oneSpcDetailBOs) {
                SpcDetailVO spcDetailVO = SpcDetailVO.builder()
                        .store(spcDetailBO.getSpcDetail().getStore())
                        .price(spcDetailBO.getSpcDetail().getPrice())
                        .name(spcDetailBO.getSpcDetail().getName())
                        .id(spcDetailBO.getSpcDetail().getId())
                        .build();
                spcDetailVOS.add(spcDetailVO);
            }
            spcVO.setSpcDetailVO(spcDetailVOS);
            spcVOs.add(spcVO);
        }

        BaseResponse baseResponse = BaseResponse.ok("操作成功");
        baseResponse.setData(spcVOs);
        return baseResponse;

    }

    private List<SpcDetailBO> getSpcAllList(int goodsId) {
        return specificationMapper.getSpcAllList(goodsId);
    }
}
