package cn.wise.sc.pay.laas.userbac.business.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserDTO;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserInfo;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import cn.wise.sc.pay.laas.userbac.domain.vo.UserVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;


public interface UserService extends IService<SysUser> {
    /**
     * 查询用户信息
     *
     * @param sysUser 用户
     * @return userInfo
     */
    UserInfo getUserInfo(SysUser sysUser);

    /**
     * 分页查询用户信息（含有角色信息）
     *
     * @param page    分页对象
     * @param userDTO 参数列表
     * @return
     */
    IPage getUserWithRolePage(Page page, UserDTO userDTO);

    /**
     * 删除用户
     *
     * @param sysUser 用户
     * @return boolean
     */
    Boolean removeUserById(SysUser sysUser);

    /**
     * 删除app用户 manage专用
     *
     * @param sysUser 用户
     * @return boolean
     */
    Boolean removeAppUserById(SysUser sysUser);

    /**
     * 更新当前用户基本信息
     *
     * @param userDto 用户信息
     * @return Boolean
     */
    R<Boolean> updateUserInfo(UserDTO userDto);

    /**
     * 更新指定用户信息
     *
     * @param userDto 用户信息
     * @return
     */
    Boolean updateUser(UserDTO userDto);

    /**
     * 更新指定用户信息
     *
     * @param userDto 用户信息
     * @return
     */
    Boolean updateAppUser(UserDTO userDto);

    /**
     * 通过ID查询用户信息
     *
     * @param id 用户ID
     * @return 用户信息
     */
    UserVO getUserVoById(Integer id);

    /**
     * 查询上级部门的用户信息
     *
     * @param username 用户名
     * @return R
     */
    List<SysUser> listAncestorUsersByUsername(String username);

    /**
     * 保存用户信息
     *
     * @param userDto DTO 对象
     * @return success/fail
     */
    Boolean saveUser(UserDTO userDto);

    /**
     * 分页查询会员列表
     *
     * @param username
     * @param phone
     * @param level
     * @param pageQuery
     * @return
     */
    R getUserList(String username, String phone, String level, PageQuery pageQuery);

    /**
     * 分页查询app端的用户会员列表
     *
     * @param username
     * @param phone
     * @param level
     * @param pageQuery
     * @return
     */
    R getAppUserList(String username, String phone, String level, PageQuery pageQuery);


    /**
     * 锁定用户
     *
     * @param userId
     * @return
     */
    Boolean userLock(Integer userId);

    /**
     * 锁定app用户 manage端使用
     *
     * @param userId
     * @return
     */
    Boolean appUserLock(Integer userId);

    /**
     * @param userName
     * @return
     */
    UserVO getUserByName(String userName);

    /**
     * 获取app用户id
     *
     * @param id
     * @return
     */
    SysUser getAppUserById(Integer id);

    /**
     * 获取所有用户集合
     *
     * @return list
     */
    List<SysUser> getUserList();

    /**
     * 天机app端虚拟用户
     *
     * @param sysUser 添加虚拟用户信息
     * @return 结果
     */
    BaseResponse<Integer> saveVirtualAppUser(SysUser sysUser);
}
