package cn.wise.sc.pay.laas.userbac.business.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.wise.sc.pay.laas.userbac.business.mapper.SysDeptRelationMapper;
import cn.wise.sc.pay.laas.userbac.business.config.DTNFIX;
import cn.wise.sc.pay.laas.userbac.business.service.DeptRelationService;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysDept;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysDeptRelation;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>

 */
@Service
@AllArgsConstructor
public class DeptRelationServiceImpl extends ServiceImpl<SysDeptRelationMapper, SysDeptRelation> implements DeptRelationService {

	@Autowired
	DTNFIX dtnfix;

	private final SysDeptRelationMapper sysDeptRelationMapper;

	/**
	 * 维护部门关系
	 *
	 * @param sysDept 部门
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void saveDeptRelation(SysDept sysDept) {
		//增加部门关系表
		SysDeptRelation condition = new SysDeptRelation();
		condition.setDescendant(sysDept.getParentId());
		List<SysDeptRelation> relationList = sysDeptRelationMapper
			.selectList(Wrappers.<SysDeptRelation>query().lambda()
				.eq(SysDeptRelation::getDescendant, sysDept.getParentId()))
			.stream().map(relation -> {
				relation.setDescendant(sysDept.getDeptId());
				return relation;
			}).collect(Collectors.toList());
		if (CollUtil.isNotEmpty(relationList)) {
			this.saveBatch(relationList);
		}

		//自己也要维护到关系表中
		SysDeptRelation own = new SysDeptRelation();
		own.setDescendant(sysDept.getDeptId());
		own.setAncestor(sysDept.getDeptId());
		sysDeptRelationMapper.insert(own);
	}

	/**
	 * 通过ID删除部门关系
	 *
	 * @param id
	 */
	@Override
	public void removeDeptRelationById(Integer id) {
		baseMapper.deleteDeptRelationsById(id,dtnfix.prefix(),dtnfix.suffix());
	}

	/**
	 * 更新部门关系
	 *
	 * @param relation
	 */
	@Override
	public void updateDeptRelation(SysDeptRelation relation) {
		baseMapper.updateDeptRelations(relation,dtnfix.prefix(),dtnfix.suffix());
	}

}
