package cn.wise.sc.pay.laas.userbac.business.service.impl;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.points.service.IUserPointsMShipExtraService;
import cn.wise.sc.pay.common.core.constant.CommonConstants;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.laas.userbac.business.config.DTNFIX;
import cn.wise.sc.pay.laas.userbac.business.mapper.SysUserMapper;
import cn.wise.sc.pay.laas.userbac.business.service.DeptRelationService;
import cn.wise.sc.pay.laas.userbac.business.service.DeptService;
import cn.wise.sc.pay.laas.userbac.business.service.MenuService;
import cn.wise.sc.pay.laas.userbac.business.service.RoleDeptService;
import cn.wise.sc.pay.laas.userbac.business.service.RoleService;
import cn.wise.sc.pay.laas.userbac.business.service.UserRoleService;
import cn.wise.sc.pay.laas.userbac.business.service.UserService;
import cn.wise.sc.pay.laas.userbac.domain.dto.MenuTree;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserDTO;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserInfo;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysDept;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysDeptRelation;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysRole;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUserRole;
import cn.wise.sc.pay.laas.userbac.domain.vo.MenuVO;
import cn.wise.sc.pay.laas.userbac.domain.vo.TreeUtil;
import cn.wise.sc.pay.laas.userbac.domain.vo.UserVO;
import com.baomidou.mybatisplus.annotation.SqlParser;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

// import cn.wise.sc.pay.common.security.util.SecurityUtils;

@Slf4j
@Service
@AllArgsConstructor
public class UserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements UserService {
    private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();
    private final MenuService menuService;
    private final RoleService roleService;
    private final DeptService deptService;
    private final UserRoleService userRoleService;
    private final DeptRelationService deptRelationService;
    private final RoleDeptService roleDeptService;

    @Autowired
    DTNFIX dtnfix;

    @Autowired
    IUserPointsMShipExtraService userPointsMShipExtraService;

    /**
     * 保存用户信息
     *
     * @param userDto DTO 对象
     * @return success/fail
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean saveUser(UserDTO userDto) {
        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(userDto, sysUser);
        sysUser.setDelFlag(CommonConstants.STATUS_NORMAL);
        sysUser.setPassword(ENCODER.encode(userDto.getPassword()));
        baseMapper.insert(sysUser);
        List<SysUserRole> userRoleList =
                userDto.getRole().stream()
                        .map(
                                roleId -> {
                                    SysUserRole userRole = new SysUserRole();
                                    userRole.setUserId(sysUser.getUserId());
                                    userRole.setRoleId(roleId);
                                    return userRole;
                                })
                        .collect(Collectors.toList());
        return userRoleService.saveBatch(userRoleList);
    }

    @Override
    public R getUserList(String username, String phone, String level, PageQuery pageQuery) {
        //		QueryWrapper wrapper = new QueryWrapper();
        //		if(StringUtils.isNotBlank(username)){
        //			wrapper.like("username",username);
        //		}
        //		if(StringUtils.isNotBlank(phone)){
        //			wrapper.like("phone",phone);
        //		}
        //		if(StringUtils.isNotBlank(level)){
        //			wrapper.eq("level",level);
        //		}
        Page page = new Page();
        page.setCurrent(pageQuery.getPageNo());
        page.setSize(pageQuery.getPageSize());
        return R.ok(
                baseMapper.getUserVosPageTest(page, username, phone, dtnfix.prefix(), dtnfix.getSuffix()));
        //		if(pageQuery != null){
        //			page.setCurrent(pageQuery.getPageNo());
        //			page.setSize(pageQuery.getPageSize());
        //		}else{
        //			return R.ok(this.list(wrapper));
        //		}
        //		return R.ok(this.page(page,wrapper));
    }

    @Override
    public R getAppUserList(String username, String phone, String level, PageQuery pageQuery) {

        Page page = new Page();
        page.setCurrent(pageQuery.getPageNo());
        page.setSize(pageQuery.getPageSize());
        return R.ok(baseMapper.getUserVosPageTest(page, username, phone, "app_", dtnfix.getSuffix()));
    }

    @Override
    public Boolean userLock(Integer userId) {
        if (userId != null) {
            SysUser user = this.getById(userId);
            if (user != null) {
                user.setLockFlag("0".equals(user.getLockFlag()) ? "9" : "0");
                return this.updateById(user);
            }
        }
        return false;
    }

    @Override
    @SqlParser(filter = true)
    public Boolean appUserLock(Integer userId) {
        if (userId != null) {
            SysUser user = this.getAppUserById(userId);
            if (user != null) {
                user.setLockFlag("0".equals(user.getLockFlag()) ? "9" : "0");
                try {
                    baseMapper.updateAppUser(user);
                    return true;
                } catch (Exception e) {
                    return false;
                }
            }
        }
        return false;
    }

    /**
     * 通过查用户的全部信息
     *
     * @param sysUser 用户
     * @return
     */
    @Override
    public UserInfo getUserInfo(SysUser sysUser) {
        UserInfo userInfo = new UserInfo();
        userInfo.setSysUser(sysUser);
        // 设置角色列表  （ID）
        List<Integer> roleIds =
                roleService.listRolesByUserId(sysUser.getUserId()).stream()
                        .map(SysRole::getRoleId)
                        .collect(Collectors.toList());
        userInfo.setRoles(ArrayUtil.toArray(roleIds, Integer.class));

        // 设置权限列表（menu.permission）
        Set<String> permissions = new HashSet<>();
        roleIds.forEach(
                roleId -> {
                    List<String> permissionList =
                            menuService.getMenuByRoleId(roleId).stream()
                                    .filter(menuVo -> StringUtils.isNotEmpty(menuVo.getPermission()))
                                    .map(MenuVO::getPermission)
                                    .collect(Collectors.toList());
                    permissions.addAll(permissionList);
                });
        //        roleDeptService.getRoleDepts(sysUser.getDeptId()).stream().forEach(sysRoleDept -> {
        //            roleIds.add(sysRoleDept.getRoleId());
        //        });
        Set<MenuVO> menus = new HashSet<>();
        roleIds.forEach(
                roleId -> {
                    List<MenuVO> menuList =
                            menuService.getMenuByRoleId(roleId).stream()
                                    .filter(menuVO -> ("0".equals(menuVO.getDelFlag())))
                                    .collect(Collectors.toList());
                    menus.addAll(menuList);
                });
        List<MenuTree> treeList =
                menus.stream()
                        .filter(menu -> !menu.getMenuId().equals(menu.getParentId()))
                        .map(
                                dept -> {
                                    MenuTree node = new MenuTree();
                                    BeanUtils.copyProperties(dept, node);
                                    node.setId(dept.getMenuId());
                                    return node;
                                })
                        .collect(Collectors.toList());
        userInfo.setMenus(TreeUtil.buildByLoop(treeList, -1));
        userInfo.setPermissions(ArrayUtil.toArray(permissions, String.class));
        return userInfo;
    }

    /**
     * 分页查询用户信息（含有角色信息）
     *
     * @param page    分页对象
     * @param userDTO 参数列表
     * @return
     */
    @Override
    public IPage getUserWithRolePage(Page page, UserDTO userDTO) {
        //		return baseMapper.getUserVosPage(page, userDTO, dtnfix.prefix(),dtnfix.suffix());
        return baseMapper.getUserVosPageTest(
                page, userDTO.getUsername(), userDTO.getPhone(), dtnfix.prefix(), dtnfix.suffix());
    }

    /**
     * 通过ID查询用户信息
     *
     * @param id 用户ID
     * @return 用户信息
     */
    @Override
    public UserVO getUserVoById(Integer id) {
        return baseMapper.getUserVoById(id, dtnfix.prefix(), dtnfix.suffix());
    }

    /**
     * 删除用户
     *
     * @param sysUser 用户
     * @return Boolean
     */
    @Override
    // @CacheEvict(value = "user_details", key = "#sysUser.username")
    public Boolean removeUserById(SysUser sysUser) {
        userRoleService.removeRoleByUserId(sysUser.getUserId());
        this.removeById(sysUser.getUserId());
        return Boolean.TRUE;
    }

    /**
     * 删除用户
     *
     * @param sysUser 用户
     * @return Boolean
     */
    @Override
    // @CacheEvict(value = "user_details", key = "#sysUser.username")
    public Boolean removeAppUserById(SysUser sysUser) {
        userRoleService.removeRoleByAppUserId(sysUser.getUserId());
        baseMapper.removeAppById(sysUser.getUserId());
        return Boolean.TRUE;
    }

    @Override
    public UserVO getUserByName(String userName) {
        return baseMapper.getUserVoByUsername(userName, "manager_", "");
    }

    @Override
    public SysUser getAppUserById(Integer id) {
        return baseMapper.getAppUserById(id, "app_", "");
    }

    @Override
    public List<SysUser> getUserList() {
        return baseMapper.list();
    }

    @Override
    public BaseResponse<Integer> saveVirtualAppUser(SysUser sysUser) {
        baseMapper.saveVirtualAppUser(sysUser);
        if (sysUser.getUserId() == null) {
            return BaseResponse.error("添加虚拟用户失败,请稍后再试!", this.getClass().getName());
        }
        return BaseResponse.ok(sysUser.getUserId());
    }

    @Override
    // @CacheEvict(value = "user_details", key = "#userDto.username")
    public R<Boolean> updateUserInfo(UserDTO userDto) {
        UserVO userVO =
                baseMapper.getUserVoByUsername(userDto.getUsername(), dtnfix.prefix(), dtnfix.suffix());
        SysUser sysUser = new SysUser();
        if (StrUtil.isNotBlank(userDto.getPassword())
                && StrUtil.isNotBlank(userDto.getNewpassword1())) {
            if (ENCODER.matches(userDto.getPassword(), userVO.getPassword())) {
                sysUser.setPassword(ENCODER.encode(userDto.getNewpassword1()));
            } else {
                log.warn("原密码错误，修改密码失败:{}", userDto.getUsername());
                return R.failed("原密码错误，修改失败");
            }
        }
        sysUser.setPhone(userDto.getPhone());
        sysUser.setUserId(userVO.getUserId());
        sysUser.setAvatar(userDto.getAvatar());
        return R.ok(this.updateById(sysUser));
    }

    @Override
    // @CacheEvict(value = "user_details", key = "#userDto.username")
    public Boolean updateUser(UserDTO userDto) {
        if (userDto.getUserId() == null) {
            return Boolean.FALSE;
        }
        SysUser sysUser = this.getById(userDto.getUserId());
        if (sysUser == null) {
            return Boolean.FALSE;
        }
        BeanUtils.copyProperties(userDto, sysUser);
        if (StringUtils.isNotBlank(sysUser.getPassword())) {
            sysUser.setPassword(ENCODER.encode(sysUser.getPassword()));
        }
        sysUser.setUpdateTime(new Date(System.currentTimeMillis()));

        this.updateById(sysUser);

        if (userDto.getRole() != null && userDto.getRole().size() > 0) {
            userRoleService.remove(
                    Wrappers.<SysUserRole>update().lambda().eq(SysUserRole::getUserId, userDto.getUserId()));
            userDto
                    .getRole()
                    .forEach(
                            roleId -> {
                                SysUserRole userRole = new SysUserRole();
                                userRole.setUserId(sysUser.getUserId());
                                userRole.setRoleId(roleId);
                                userRole.insert();
                            });
        }
        return Boolean.TRUE;
    }

    @Override
    // @CacheEvict(value = "user_details", key = "#userDto.username")
    public Boolean updateAppUser(UserDTO userDto) {
        if (userDto.getUserId() == null) {
            return Boolean.FALSE;
        }
        SysUser sysUser = baseMapper.getAppUserById(userDto.getUserId(), "app_", "");
        if (sysUser == null) {
            return Boolean.FALSE;
        }
        BeanUtils.copyProperties(userDto, sysUser);
        if (StringUtils.isNotBlank(sysUser.getPassword())) {
            sysUser.setPassword(ENCODER.encode(sysUser.getPassword()));
        }
        sysUser.setUpdateTime(new Date(System.currentTimeMillis()));

        baseMapper.updateAppUser(sysUser);

        //        if (userDto.getRole() != null && userDto.getRole().size() > 0) {
        //            userRoleService.remove(Wrappers.<SysUserRole>update().lambda()
        //                    .eq(SysUserRole::getUserId, userDto.getUserId()));
        //            userDto.getRole().forEach(roleId -> {
        //                SysUserRole userRole = new SysUserRole();
        //                userRole.setUserId(sysUser.getUserId());
        //                userRole.setRoleId(roleId);
        //                userRole.insert();
        //            });
        //        }
        return Boolean.TRUE;
    }

    /**
     * 查询上级部门的用户信息
     *
     * @param username 用户名
     * @return R
     */
    @Override
    public List<SysUser> listAncestorUsersByUsername(String username) {
        SysUser sysUser =
                this.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));

        SysDept sysDept = deptService.getById(sysUser.getDeptId());
        if (sysDept == null) {
            return null;
        }

        Integer parentId = sysDept.getParentId();
        return this.list(Wrappers.<SysUser>query().lambda().eq(SysUser::getDeptId, parentId));
    }

    /**
     * 获取当前用户的子部门信息
     *
     * @return 子部门列表
     */
    private List<Integer> getChildDepts() {
        //        Integer deptId = SecurityUtils.getUser().getDeptId();

        Integer deptId = 1;
        // 获取当前部门的子部门
        return deptRelationService
                .list(Wrappers.<SysDeptRelation>query().lambda().eq(SysDeptRelation::getAncestor, deptId))
                .stream()
                .map(SysDeptRelation::getDescendant)
                .collect(Collectors.toList());
    }
}
