package cn.wise.sc.pay.pass.business.app.controller.eshop;

import cn.wise.sc.pay.api.eshopping.service.IEshopTrolleyService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.query.TrolleyGoodsDetail;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import java.util.List;

/**
 * @description: 购物车
 * @author: qh
 * @create: 2020-03-27 16:10
 **/
@Validated
@CrossOrigin
@RestController
@Api(tags = "购物车")
@RequestMapping("/app/eshop/shoppingTrolley")
public class EshopTrolleyController {

    final
    IEshopTrolleyService iEshopTrolleyService;

    public EshopTrolleyController(IEshopTrolleyService iEshopTrolleyService) {
        this.iEshopTrolleyService = iEshopTrolleyService;
    }

    /**
     * 加入购物车
     *
     * @return List<TrolleyGoodsDetail>
     */
    @PostMapping
    @ApiOperation("加入购物车")
    @ApiImplicitParam(value = "购物车商品", name = "TrolleyGoodsDetail", paramType = "body", dataType = "TrolleyGoodsDetail")
    public BaseResponse<List<TrolleyGoodsDetail>> joinTrolley(@RequestBody TrolleyGoodsDetail trolleyGoodsDetail) {
        return iEshopTrolleyService.joinTrolley(trolleyGoodsDetail);
    }


    /**
     * 移除购物车
     *
     * @param goodId 商品id
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @DeleteMapping
    @ApiOperation("移除购物车")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "商品id", name = "goodId", paramType = "query", dataType = "Long", required = true),
            @ApiImplicitParam(value = "用户id", name = "userId", paramType = "query", dataType = "Long", required = true),
            @ApiImplicitParam(value = "规格id", name = "skuId", paramType = "query", dataType = "Long", required = true)
    })
    public BaseResponse<List<TrolleyGoodsDetail>> removeTrolley(@RequestParam("goodId") @Min(value = 1, message = "无效的商品号") Long goodId,
                                                                @RequestParam("userId") @Min(value = 1, message = "无效的用户id") Long userId,
                                                                @RequestParam("skuId") Long skuId) {
        return iEshopTrolleyService.removeTrolley(goodId, userId, skuId);
    }

    /**
     * 获取购物车里的所有商品
     *
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @GetMapping
    @ApiOperation("获取购物车里的所有商品")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "用户id", name = "userId", paramType = "query", dataType = "Long", required = true)
    })
    public BaseResponse<List<TrolleyGoodsDetail>> getGoods(@RequestParam("userId") @Min(value = 1, message = "无效的用户id") Long userId) {
        return iEshopTrolleyService.getGoods(userId);
    }

    /**
     * 编辑购物车
     *
     * @param list   购物车信息
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @PostMapping("/update")
    @ApiOperation("移除购物车")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "购物车信息", name = "list", paramType = "body", dataType = "List"),
            @ApiImplicitParam(value = "用户id", name = "userId", paramType = "query", dataType = "Long", required = true)
    })

    public BaseResponse<List<TrolleyGoodsDetail>> edit(@RequestBody List<TrolleyGoodsDetail> list,
                                                       @RequestParam("userId") @Min(value = 1, message = "无效的用户id") Long userId) {
        return iEshopTrolleyService.edit(list, userId);
    }

}
