package cn.wise.sc.pay.pass.business.app.controller.fastdfs;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.pass.business.app.config.FastDfsUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/app/fastdfs")
@Api(value = "上传资源到FastDFS服务器接口")
public class FastDfsController {
    @Autowired
    FastDfsUtil fastDFSUtils;

    @PostMapping("/upload")
    @ApiOperation(value = "上传接口", notes = "可上传图片、视频、文件，返回上传地址信息")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "file", value = "需要上传的资源文件", dataType = "MultipartFile", paramType = "body")
    )
    public BaseResponse<Map<String, String>> upload(MultipartFile file) {
        Map<String, String> map = new HashMap<>();

        map.put("host", fastDFSUtils.fullUrl(""));
        String path = "";
        if (file != null && StringUtils.isNotEmpty(file.getOriginalFilename())) {
            try {
                path = fastDFSUtils.uploadFile(file).getPath();
            } catch (IOException e) {
                return BaseResponse.error(e.getMessage(), this.getClass().getName());
            }
        }
        map.put("uri", path);
        map.put("fullPath", fastDFSUtils.fullUrl(path));
        return BaseResponse.ok(map);
    }

    /*@DeleteMapping("/remove")
    @ApiOperation(value = "删除fastdfs资源接口")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "fullPath", value = "资源全路径", dataType = "String", paramType = "query")
    )
    public BaseResponse remove(String fullPath) {
        String delete = null;
        try {
            delete = fastDFSUtils.delete(fullPath);
        } catch (Exception e) {
            e.printStackTrace();
            return BaseResponse.error(e.getMessage(), this.getClass().getName());
        }
        return BaseResponse.ok(delete);
    }*/

    /*@GetMapping("/download")
    @ApiOperation(value = "删除fastdfs资源接口")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "fullPath", value = "资源全路径", dataType = "String", paramType = "query")
    )
    public BaseResponse download(String fullPath) {
        fastDFSUtils.downLoad();
    }*/
}
