package cn.wise.sc.pay.pass.business.app.controller.orders;

import cn.hutool.core.map.MapUtil;
import cn.wise.sc.pay.api.order.service.IXpenseTrackerService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.core.util.DateTimeUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

@RestController
@Api(description = "消费记录")
@CrossOrigin
@RequestMapping("/app/XpenseTracker")
public class XpenseTrackerAppController {

    final
    IXpenseTrackerService xpenseTrackerService;

    public XpenseTrackerAppController(IXpenseTrackerService xpenseTrackerService) {
        this.xpenseTrackerService = xpenseTrackerService;
    }

    @GetMapping("/page")
    @ApiOperation("分页集合查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "start", value = "开始时间段", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间段", paramType = "query", dataType = "long"),
    })
    public BaseResponse page(long userId, long start, long end, PageQuery page) {

        PageWrapperQuery pageWrapperQuery = constructionCondition(userId, start, end);
        pageWrapperQuery.setPageQuery(page);
        String lastSql = "order by create_time desc";
        pageWrapperQuery.setSql(lastSql);
        return xpenseTrackerService.page(pageWrapperQuery);
    }

    @GetMapping("/list")
    @ApiOperation("普通集合查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "start", value = "开始时间段", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间段", paramType = "query", dataType = "long"),
    })
    public BaseResponse list(@RequestParam("userId") long userId,
                             @RequestParam("start") long start,
                             @RequestParam("end") long end) {

        PageWrapperQuery pageWrapperQuery = constructionCondition(userId, start, end);
        String lastSql = "order by create_time desc";
        pageWrapperQuery.setSql(lastSql);
        return xpenseTrackerService.list(pageWrapperQuery);
    }

    /**
     * 构造消费记录查询条件
     *
     * @param userId 用户id
     * @param start  起始时间
     * @param end    结束时间
     * @return
     */
    private PageWrapperQuery constructionCondition(long userId, long start, long end) {
        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        Map eq = MapUtil.newConcurrentHashMap(1);
        if (userId != 0) {
            eq.put("user_id", userId);
            pageWrapperQuery.setEq(eq);
        }
        Map le = new HashMap(1);
        Map ge = new HashMap(1);
        if (DateTimeUtils.check(start, end)) {
            le.put("create_time", end);
            ge.put("create_time", start);
            pageWrapperQuery.setLe(le);
            pageWrapperQuery.setGe(ge);
        }
        return pageWrapperQuery;
    }
}
