package cn.wise.sc.pay.pass.business.cashiers.controller.merchant;

import cn.wise.sc.pay.api.merchant.service.IBannerImageService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.BannerImage;
import cn.wise.sc.pay.domain.merchant.query.BannerImageQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

/**
 * @author lqk
 * @since 2019/10/08 09:09
 */
@RestController
@CrossOrigin
@Api(description = "广告图展示")
@RequestMapping("/manage/merchant/banner/")
@Slf4j
public class BannerImageController {

    final
    IBannerImageService bannerImageService;

    public BannerImageController(IBannerImageService bannerImageService) {
        this.bannerImageService = bannerImageService;
    }

    @GetMapping("advertImages")
    public BaseResponse getAdvertImageList() {
        return bannerImageService.getAdvertImageList();
    }

    /**
     * 添加收银端广告图
     *
     * @param bannerImageQuery
     * @return
     */
    @PostMapping("cashierAdvert")
    @ApiOperation("添加收银端广告图")
    public BaseResponse<BannerImage> addCashierAdvert(@RequestBody BannerImageQuery bannerImageQuery) {
        return bannerImageService.addCashierAdvert(bannerImageQuery);
    }

    /**
     * 收银端根据商家id获取商家广告图
     *
     * @param merchantId
     * @return
     */
    @GetMapping("cashierAdvert")
    @ApiOperation(value = "根据商户id获取商家广告图")
    @ApiImplicitParam(name = "merchantId", value = "商家id", dataType = "Long", paramType = "query")
    public BaseResponse getCashierAdvert(@NotNull Long merchantId) {
        BaseResponse response = bannerImageService.getCashierAdvert(merchantId);
        return response;
    }

    /**
     * 分页查询店铺与店铺的广告图
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/list/merchants")
    @ApiOperation(value = "根据店铺名分页筛选店铺与店铺的广告图")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "pageNo", value = "第几页", dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "name", value = "店铺名称", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页显示数量", dataType = "int", paramType = "query")
    })
    public BaseResponse listBannerByMerchants(Integer pageNo, Integer pageSize, String name) {
        BaseResponse response = bannerImageService.listBannerByMerchants(pageNo, pageSize, name);
        return response;
    }
}
