package cn.wise.sc.pay.pass.business.cashiers.controller.merchant;

import cn.wise.sc.pay.api.merchant.service.IMerchDiscountService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.MerchDiscount;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @description: 店铺折扣
 * @author: qh
 * @create: 2020-09-07 13:36
 **/
@RestController
@CrossOrigin
@RequestMapping("/merchant/discount")
@Slf4j
public class MerchDiscountController {

    final
    IMerchDiscountService iMerchDiscountService;

    public MerchDiscountController(IMerchDiscountService iMerchDiscountService) {
        this.iMerchDiscountService = iMerchDiscountService;
    }

    @GetMapping("/{seller}")
    @ApiOperation("根据商户id获取折扣配置")
    @ApiImplicitParam(name = "seller", value = "商户id")
    BaseResponse<MerchDiscount> getMerchDiscountBySeller(@PathVariable("seller") Long seller){
        return iMerchDiscountService.getMerchDiscountBySeller(seller);
    }

    @PutMapping
    @ApiOperation("更新商户折扣信息")
    BaseResponse<Boolean> updateMerchDiscountBySeller(@RequestBody MerchDiscount merchDiscount){
        return iMerchDiscountService.updateMerchDiscountBySeller(merchDiscount);
    }

    @PostMapping
    @ApiOperation("新增一个折扣信息")
    BaseResponse<MerchDiscount> addNewMerchDiscount(@RequestBody MerchDiscount merchDiscount){
        return iMerchDiscountService.addNewMerchDiscount(merchDiscount);
    }
}
