package cn.wise.sc.pay.pass.business.manager.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.api.points.service.IMemberShipService;
import cn.wise.sc.pay.api.points.service.IUserPointsMShipExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.security2.userDetail.JwtUserDetailsService;
import cn.wise.sc.pay.common.security2.util.JwtUtils;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.query.UserExreaQuery;
import cn.wise.sc.pay.laas.userbac.business.controller.UserController;
import cn.wise.sc.pay.laas.userbac.business.service.UserService;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserDTO;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserInfo;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import cn.wise.sc.pay.laas.userbac.domain.vo.UserVO;
import cn.wise.sc.pay.pass.business.manager.model.SysUserInfoVO;
import cn.wise.sc.pay.pass.business.manager.model.VirtualUserQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author neo.shu
 * @since 2019/12/9 18:27
 */
@CrossOrigin
@RestController
@RequestMapping("/manager/user")
@Api(value = "manager/user", tags = "管理端用户管理")
public class UserControler extends UserController {

    @Autowired
    IUserPointsMShipExtraService iUserPointsMShipExtraService;
    @Autowired
    IMemberShipService memberShipService;
    @Autowired
    UserService userService;
    private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();
    final AuthenticationManager authenticationManager;
    final JwtUserDetailsService userDetailsService;
    private final IUserPointsMShipExtraService userPointsMShipExtraService;

    public UserControler(
            UserService userService,
            IUserPointsMShipExtraService userPointsMShipExtraService,
            AuthenticationManager authenticationManager,
            JwtUserDetailsService userDetailsService) {
        super(userService);
        this.userPointsMShipExtraService = userPointsMShipExtraService;
        this.authenticationManager = authenticationManager;
        this.userDetailsService = userDetailsService;
    }

    /**
     * 创建token
     *
     * @return
     * @throws Exception
     */
    @PostMapping("/token")
    @ApiOperation("获取token信息")
    public String createAuthenticationToken(String username, String password) throws Exception {
        authenticate(username, password);
        UserDetails userDetails = userDetailsService.loadUserByUsername(username);
        String token = JwtUtils.generateUsersToken(userDetails);
        return token;
    }

    private void authenticate(String username, String password) throws Exception {
        try {
            authenticationManager.authenticate(
                    new UsernamePasswordAuthenticationToken(username, password));
        } catch (DisabledException e) {
            throw new Exception("USER_DISABLED", e);
        } catch (BadCredentialsException e) {
            throw new Exception("INVALID_CREDENTIALS", e);
        }
    }

    @ApiOperation("管理端账号密码登录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "username", value = "用户名", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "password", value = "用户密码", dataType = "String", paramType = "query")
    })
    @GetMapping("/login")
    public R loginSysUser(
            @RequestParam("username") String username, @RequestParam("password") String password) {
        UserInfo sysUser = (UserInfo) info(username).getData();
        if (sysUser != null) {
            String pw = sysUser.getSysUser().getPassword();
            if (ENCODER.matches(password, pw)) {
                return R.ok(sysUser);
            }
            return R.failed("密码错误");
        }
        return R.failed("用户不存在");
    }

    @ApiOperation("修改密码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", dataType = "int", paramType = "path"),
            @ApiImplicitParam(
                    name = "oldPassword",
                    value = "原先的密码",
                    dataType = "String",
                    paramType = "query"),
            @ApiImplicitParam(
                    name = "newPassword",
                    value = "新设置密码",
                    dataType = "String",
                    paramType = "query")
    })
    @PostMapping("/password/change/{id}")
    public R changePassword(@PathVariable Integer id, String oldPassword, String newPassword) {
        if (StringUtils.isEmpty(oldPassword)) {
            return R.failed("没有输入密码");
        }
        if (StringUtils.isEmpty(newPassword)) {
            return R.failed("没有输入新密码");
        }
        if (id == null) {
            return R.failed("没有输入主键id");
        }
        Object user = super.user(id).getData();
        UserDTO userDTO = new UserDTO();
        userDTO.setNewpassword1(newPassword);
        userDTO.setUsername(((SysUser) user).getUsername());
        userDTO.setPassword(oldPassword);
        R r = super.updateUserInfo(userDTO);
        return r;
    }

    @GetMapping("/list/all")
    @ApiOperation("获取所有用户集合")
    public R userList() {
        List<SysUser> list = super.getUserList();
        return R.ok(list);
    }

    @ApiOperation("会员用户条件分页列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "username", value = "用户名搜索", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "phone", value = "用户手机号搜索", dataType = "String", paramType = "query"),
            @ApiImplicitParam(
                    name = "level",
                    value = "会员等级搜索（当前未实现）",
                    dataType = "String",
                    paramType = "query"),
            @ApiImplicitParam(name = "pageNo", value = "当前页", dataType = "Integer", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", dataType = "Integer", paramType = "query")
    })
    @GetMapping("/userPage")
    public R getUserPage(String username, String phone, String level, PageQuery pageQuery) {
        Page page = (Page) getAppUserList(username, phone, level, pageQuery).getData();
        List<UserVO> users = page.getRecords();
        List<Long> userId =
                users.stream()
                        .map(arg -> Integer.toUnsignedLong(arg.getUserId()))
                        .collect(Collectors.toList());
        List<LinkedHashMap> result = userPointsMShipExtraService.getAllUserExtra(userId).getList();
        List<UserExtra> userExtras = JSON.parseArray(JSON.toJSONString(result), UserExtra.class);
        List<SysUserInfoVO> res = new ArrayList<>();
        users.stream()
                .forEach(
                        arg -> {
                            UserExtra user =
                                    userExtras.stream()
                                            .filter(parm -> Integer.toUnsignedLong(arg.getUserId()) == parm.getUserId())
                                            .findFirst()
                                            .orElse(null);
                            SysUserInfoVO sysUserInfoVO = new SysUserInfoVO();
                            BeanUtil.copyProperties(user, sysUserInfoVO);
                            BeanUtil.copyProperties(arg, sysUserInfoVO);
                            res.add(sysUserInfoVO);
                        });
        Page pageResult = new Page(page.getCurrent(), page.getSize(), page.getTotal());
        pageResult.setRecords(res);
        return R.ok(pageResult);
    }

    /**
     * 分页查询用户
     *
     * @param pageQuery 参数集
     * @param userDTO   查询参数列表
     * @return 用户集合
     */
    @GetMapping("/v1/page")
    public R getUserPage(PageQuery pageQuery, UserDTO userDTO) {
        Page page = new Page();
        page.setCurrent(pageQuery.getPageNo());
        page.setSize(pageQuery.getPageSize());
        return super.getUserPage(page, userDTO);
    }

    @ApiOperation(value = "会员基本信息修改")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "userDTO", value = "参数对象", dataType = "UserDTO", paramType = "body"))
    @PutMapping("/edit/userInfo")
    @Transactional
    public BaseResponse editUserInfo(@Valid @RequestBody UserDTO userDTO) {
        // 参数校验
        BaseResponse response = validUserDto(userDTO);
        if (response.getCode() == 400) {
            return response;
        }
        BaseResponse editUserExtra = null;
        if (userDTO.getMemberShipId() != null) {
            R r = super.updateUserInfo(userDTO);
            if (r.ok() && editUserExtra != null && editUserExtra.getCode() == 200) {
                return BaseResponse.ok(r.getData());
            } else {
                return BaseResponse.error("更新失败", this.getClass().getName());
            }
        }
        R r = super.updateAppUserInfo(userDTO);

        if (r.ok()) {
            return BaseResponse.ok(r.getData());
        }
        return BaseResponse.error("更新失败", this.getClass().getName());
    }

    /**
     * 校验会员基本信息修改
     *
     * @param userDTO
     * @return
     */
    public BaseResponse validUserDto(UserDTO userDTO) {
        //        String phoneReg =
        // "^(13[0-9]|14[579]|15[0-3,5-9]|16[6]|17[0135678]|18[0-9]|19[89])\\d{8}$";

        if (userDTO == null) {
            return BaseResponse.error("修改信息不能为空", this.getClass().getName());
        }
        if (org.apache.commons.lang3.StringUtils.isBlank(userDTO.getUsername())) {
            return BaseResponse.error("昵称不能为空", this.getClass().getName());
        }
        //        if (Pattern.matches(phoneReg, userDTO.getPhone())) {
        //            return BaseResponse.error("手机号码不匹配", this.getClass().getName());
        //        }
        return BaseResponse.ok();
    }

    /**
     * 修改会员的 积分、经验、会员等级信息
     *
     * @param userExreaQuery 积分 等级
     * @return
     */
    @PutMapping("/edit/userInfo/points")
    public BaseResponse editUserExtra(@RequestBody UserExreaQuery userExreaQuery) {
        return userPointsMShipExtraService.editUserExtra(userExreaQuery);
    }

    @PostMapping("/virtualUser")
    @ApiOperation("后台添加虚拟用户")
    public BaseResponse addVirtualUser(@RequestBody VirtualUserQuery virtualUserQuery) {

        SysUser sysUser = new SysUser();
        sysUser.setUsername(virtualUserQuery.getName());
        sysUser.setAvatar(virtualUserQuery.getCover());
        BaseResponse<Integer> response = super.saveVirtualAppUser(sysUser);
        if (response.getCode() == 400) {
            return response;
        }
        int userId = response.getData();
        //获取最低级会话等级
        BaseResponse<Membership> primaryMemberShip = memberShipService.getPrimaryMemberShip();
        Membership membership = primaryMemberShip.getData();
        if (primaryMemberShip.getCode() == 400 || membership == null) {
            return BaseResponse.error("绑定会员失败，注册失败", SysUser.class.getName());
        }
        //新增成功时 给用户关联一个新的   积分、经验、会员等级信息
        UserExreaQuery userExreaQuery = new UserExreaQuery();
        userExreaQuery.setUserId(Integer.toUnsignedLong(userId));
        userExreaQuery.setMembershipId(membership.getId());
        userExreaQuery.setPoints(0d);
        userExreaQuery.setExperience(0d);
        BaseResponse baseResponse = iUserPointsMShipExtraService.addNewUserExtra(userExreaQuery);
        //添加积分信息失败时，删除注册信息
        if (baseResponse.getCode() == 400) {
            super.userDel(userId);
            return BaseResponse.error("注册失败", this.getClass().getName());
        }
        return BaseResponse.ok();
    }
}
