package cn.wise.sc.pay.pass.business.manager.controller.association;

import cn.wise.sc.pay.api.association.service.IArticleCommentService;
import cn.wise.sc.pay.api.association.service.IArticleService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.association.bean.Article;
import cn.wise.sc.pay.domain.association.query.ArticleQuery;
import cn.wise.sc.pay.domain.association.vo.ArticleVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@RestController
@Api(description = "图文/文章")
@RequestMapping("/manage/association/article")
@CrossOrigin
public class ArticleController {

    @Autowired
    IArticleService iArticleService;
    @Autowired
    IArticleCommentService iArticleCommentService;
    @Autowired
    RedissonClient redissonClient;
    /**
     * redis中文章审核开关配置
     */
    String articleAuditSettingKey = "articleAuditSettingKey";


    @GetMapping("getArticleNotVerify")
    @ApiOperation("获取未审核图文列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "authorName", value = "发表人名称", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "timeBucket", value = "搜索时间段", paramType = "path", dataType = "TimeBucket"),
            @ApiImplicitParam(name = "pageQuery", value = "分页", paramType = "path", dataType = "PageQuery")
    })
    public ResponseEntity<Page<Article>> getArticleNotVerify(String authorName, TimeBucket timeBucket, PageQuery pageQuery) {
        Long sTime = 0L;
        Long eTime = 99999999999999L;
        if (timeBucket.getStartTime() != null && timeBucket.getEndTime() != null) {
            sTime = timeBucket.getStartTime();
            eTime = timeBucket.getEndTime();
        }
        BaseResponse baseResponse = iArticleService.getArticleNotVerify(authorName,
                sTime, eTime, pageQuery.getPageNo(), pageQuery.getPageSize());
        return ResponseEntity.ok((Page<Article>) baseResponse.getData());
    }

    @GetMapping("selectById")
    @ApiOperation("根据id获取图文详情")
    @ApiImplicitParam(name = "id", value = "文章id", paramType = "path", dataType = "Long")
    public ResponseEntity<ArticleVO> selectArticleById(Long id) {
        return ResponseEntity.ok(iArticleService.selectArticleById(id).getData());
    }

    @PutMapping("updateStatus")
    @ApiOperation("修改文章/图文审核状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "文章id,多个用逗号分割", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "status", value = "文章状态", paramType = "path", dataType = "String")
    })
    public ResponseEntity updateStatus(String ids, byte status) {

        ResponseEntity response = ResponseEntity.ok(iArticleService.updateStatus(ids, status).getData());
        return response;
    }

    @GetMapping("ponArticle")
    @ApiOperation("上一篇或下一篇图文 pon=0代表上一篇，pon=1代表下一篇")
    public BaseResponse previousOrNextArticle(Long articleId, Integer pon) {
        return iArticleService.previousOrNextArticle(articleId, pon);
    }

    @GetMapping("ponVerifyArticle")
    @ApiOperation("上一篇或下一篇已审核图文 pon=0代表上一篇，pon=1代表下一篇")
    public BaseResponse previousOrNextVerifyArticle(Long articleId, Integer pon) {
        return iArticleService.previousOrNextVerifyArticle(articleId, pon);
    }

    @GetMapping("getArticleVerify")
    @ApiOperation("获取审核通过的图文列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "搜索文章分类", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "order", value = "评论数排序 asc升序 desc降序", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "name", value = "文章内容", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "title", value = "文章标题", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "pageQuery", value = "分页", paramType = "path", dataType = "PageQuery")
    })
    public ResponseEntity<Page<Article>> getArticleVerify(Integer type, String order, String name, String title,
                                                          PageQuery pageQuery) {
        ResponseEntity response = ResponseEntity.ok(
                iArticleService.getArticleVerify(type, order, name, title, pageQuery.getPageNo(), pageQuery.getPageSize()).getData());
        return response;
    }


    @DeleteMapping("deleteArticles")
    @ApiOperation("批量删除图文 或 批量推荐图文")
    public ResponseEntity deleteArticles(String ids, Integer status) {
        ResponseEntity response = ResponseEntity.ok(iArticleService.deleteArticles(ids, status).getData());
        return response;
    }

    @PutMapping("recommend")
    @ApiOperation("推荐")
    @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long")
    public ResponseEntity rcommend(Long id) {
        ResponseEntity response = ResponseEntity.ok(iArticleService.rcommend(id).getData());
        return response;
    }

    @PutMapping("deleteRecommend")
    @ApiOperation("移除推荐")
    @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long")
    public ResponseEntity deleteRecommend(Long id) {
        ResponseEntity response = ResponseEntity.ok(iArticleService.deleteRecommend(id).getData());
        return response;
    }

    @PutMapping("updateViewNum")
    @ApiOperation("修改浏览量")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "文章id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "viewNum", value = "浏览量", paramType = "path", dataType = "Long")
    })
    public ResponseEntity updateViewNum(Long id, Long viewNum) {
        ResponseEntity response = ResponseEntity.ok(iArticleService.updateViewNum(id, viewNum).getData());
        return response;
    }

    @PostMapping("/addArticle")
    @ApiOperation("发表图文分享")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "article", value = "图文/文章", paramType = "body", dataType = "Article"),
            @ApiImplicitParam(
                    name = "files",
                    value = "图文/文章相关资源",
                    paramType = "body",
                    dataType = "MultipartFile")
    })
    public BaseResponse addArticle(@RequestBody ArticleQuery articleQuery) {
        articleQuery.setStatus((byte) 1);
        return iArticleService.addArticle(articleQuery);
    }

    @PostMapping("/uploadPic")
    @ApiOperation("上传图片或视频")
    @ApiImplicitParam(name = "file", value = "图片/视频", paramType = "body", dataType = "MultipartFile")
    public BaseResponse<Map<String, String>> uploadPic(MultipartFile file) {
        return iArticleService.uploadPic(file);
    }


    @GetMapping("articleAuditSetting")
    @ApiOperation("获取审核开关设置")
    public BaseResponse articleAuditSetting() {

        RBucket<String> bucket = redissonClient.getBucket(articleAuditSettingKey);
        if (!bucket.isExists()) {
            bucket.set("close");
        }
        return BaseResponse.ok(bucket.get());
    }

    @PutMapping("articleAuditSetting")
    @ApiParam("开/关审核功能")
    public BaseResponse editArticleAuditSetting() {
        RBucket<String> bucket = redissonClient.getBucket(articleAuditSettingKey);
        if (!bucket.isExists()) {
            bucket.set("close");
            return BaseResponse.okMsg("修改成功!", this.getClass());
        }
        if ("close".equals(bucket.get())) {
            bucket.set("yes");
        } else {
            bucket.set("close");
        }

        return BaseResponse.okMsg("修改成功!", this.getClass());
    }
}

