package cn.wise.sc.pay.pass.business.manager.controller.cashier;

import cn.wise.sc.pay.api.cashier.service.IGoodsService;
import cn.wise.sc.pay.business.goods.model.query.GoodsQuery;
import cn.wise.sc.pay.business.goods.model.query.GoodsTypeQuery;
import cn.wise.sc.pay.business.goods.service.IGoodsTypeService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.pass.business.manager.model.GoodExcelModel;
import cn.wise.sc.pay.pass.business.manager.model.GoodExcelModelListener;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.read.metadata.ReadSheet;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

/**
 * @description: \
 * @author: qh
 * @create: 2020-03-12 16:20
 **/
@CrossOrigin
@RestController
@RequestMapping("/manage/goods")
@Api(description = "商品相关")
public class ManagerGoodsController {

    @Autowired
    IGoodsTypeService iGoodsTypeService;
    final
    IGoodsService iGoodsService;

    public ManagerGoodsController(IGoodsService iGoodsService) {
        this.iGoodsService = iGoodsService;
    }

    @GetMapping("/list/{sellerId}")
    @ApiOperation("所有商品集合(管理端)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "key", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "status", value = "状态 0下架 1上架", paramType = "query", dataType = "int")

    })
    public BaseResponse list(@PathVariable("sellerId") int sellerId, String key, String type, int status) {
        return iGoodsService.list(sellerId, key, type, status);
    }

    @GetMapping("/list/user")
    @ApiOperation("按照类型获取list")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "status", value = "状态 0下架 1上架 -1:全部", paramType = "query", dataType = "int")
    })
    public BaseResponse listUser(String key, String type, int status) {

        return iGoodsService.list(0, "", type, 1);
    }

    @GetMapping("/spc/{goodsId}")
    @ApiOperation("获取商品规格")
    @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int")
    public BaseResponse getGoodsSpc(@PathVariable("goodsId") int goodsId) {
        return iGoodsService.getGoodsSpc(goodsId);
    }

    @GetMapping("/{goodsId}")
    @ApiOperation("获取商品")
    @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int")
    public BaseResponse getGoods(@PathVariable("goodsId") int goodsId) {

        return (iGoodsService.getGoods(goodsId));
    }

    @PostMapping
    @ApiOperation("添加一个商品")
    @ApiImplicitParam(name = "goodsQuery", value = "商品", paramType = "body", dataType = "GoodsQuery")
    public BaseResponse newGoods(@RequestBody GoodsQuery goodsQuery) {
        goodsQuery.setStatus(1);
        return iGoodsService.newGoods(goodsQuery);
    }

    @PutMapping
    @ApiOperation("编辑商品")
    @ApiImplicitParam(name = "goodsQuery", value = "商品", paramType = "body", dataType = "GoodsQuery")
    public BaseResponse edit(@RequestBody GoodsQuery goodsQuery) {
        return iGoodsService.edit(goodsQuery);
    }

    @DeleteMapping("/{goodsId}/{sellerId}")
    @ApiOperation("删除商品")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int")
    })
    public BaseResponse delete(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.delete(goodsId, sellerId);
    }

    @DeleteMapping("/spc/{spcId}/{sellerId}")
    @ApiOperation("删除商品一级规格")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "spcId", value = "一级规格id", paramType = "path", dataType = "int")
    })
    public BaseResponse deleteSpc(@PathVariable("spcId") int spcId, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.deleteSpc(spcId, sellerId);
    }

    @DeleteMapping("/spc/detail/{detailId}/{sellerId}")
    @ApiOperation("删除商品二级规格")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "detailId", value = "二级规格id", paramType = "path", dataType = "int")
    })
    public BaseResponse deleteSpcDetal(@PathVariable("detailId") int detailId, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.deleteSpcDetal(detailId, sellerId);
    }

    @PutMapping("/status/1/{goodsId}/{sellerId}")
    @ApiOperation("上架")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int")
    })
    public BaseResponse shelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.shelveGoods(goodsId, sellerId);
    }

    @PutMapping("/status/0/{goodsId}/{sellerId}")
    @ApiOperation("下架")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int"),
    })
    public BaseResponse unShelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.unShelveGoods(goodsId, sellerId);
    }

    @PutMapping("/manage/status/0/{goodsId}/{sellerId}")
    @ApiOperation("下架(非主观下架,带原因)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "cause", value = "原因", paramType = "query", dataType = "string")
    })
    public BaseResponse unShelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId, String cause) {
        return iGoodsService.unShelveGoods(goodsId, sellerId, cause);
    }

    @PostMapping("/list/waitAudit")
    @ApiOperation("获取待审核的商品集合")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "key", value = "商品名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "shopId", value = "商铺名", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "goodsType", value = "商品类型", paramType = "query", dataType = "string"),
    })
    public BaseResponse getWaitingAuditList(@RequestBody PageQuery pageQuery, String key, Long shopId, String goodsType) {
        return iGoodsService.getWaitingAuditList(pageQuery, key, shopId, goodsType);

    }

    @PutMapping("/audit")
    @ApiOperation("审核商品")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsId", value = "商户id", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "状态  1:通过   3:不通过", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "cause", value = "原因", paramType = "query", dataType = "string"),
    })
    public BaseResponse auditGoods(long goodsId, int status, String cause) {
        return iGoodsService.auditGoods(goodsId, status, cause);
    }

    /**
     * @param pageQuery
     * @param key
     * @param goodsType
     * @return
     */
    @PostMapping("/list")
    @ApiOperation("获取所有的商品集合")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "key", value = "商品名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "shopId", value = "商铺id", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "goodsType", value = "商品类型", paramType = "query", dataType = "string"),
    })
    public BaseResponse list(@RequestBody PageQuery pageQuery, String key, long shopId, String goodsType) {
        return iGoodsService.list(pageQuery, key, shopId, goodsType);
    }

    @PostMapping("/type")
    @ApiOperation("新增一个类型")
    @ApiImplicitParam(name = "typeQuery", value = "商品类型", paramType = "body", dataType = "GoodsTypeQuery")
    public BaseResponse newGoodsType(@RequestBody GoodsTypeQuery typeQuery) {
        return iGoodsService.newGoodsType(typeQuery);
    }

    @GetMapping("/type/list/{sellerId}")
    @ApiOperation("获取类型集合")
    @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int")
    public BaseResponse typeList(@PathVariable("sellerId") int sellerId, short status) {
        return iGoodsService.typeList(sellerId, status);
    }

    @PutMapping("/type")
    @ApiOperation("编辑一个类型")
    @ApiImplicitParam(name = "typeQuery", value = "商品类型", paramType = "body", dataType = "GoodsTypeQuery")
    public BaseResponse editType(@RequestBody GoodsTypeQuery typeQuery) {
        return iGoodsService.editType(typeQuery);
    }

    @DeleteMapping("/type/{id}/{sellerId}")
    @ApiOperation("删除一个类型")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "类型id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int")
    })
    public BaseResponse delType(@PathVariable("id") int id, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.delType(id, sellerId);
    }

    @DeleteMapping("/typeAndUnShelveGoods/{id}/{sellerId}")
    @ApiOperation("删除一个类型以及下架该类别下的所有商品")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "类型id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int")
    })
    public BaseResponse delTypeAndUnShelveGoods(@PathVariable("id") int id, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.delTypeAndUnShelveGoods(id, sellerId);
    }


    @PostMapping("/batch/excel/{seller}")
    @ApiOperation("Excel批量导入商品")
    public BaseResponse<Boolean> excelBatchAdd(@RequestParam(value = "file") MultipartFile file, @PathVariable("seller") String sellerId) {

        ExcelReader excelReader = null;
        try {
            excelReader = EasyExcel.read(file.getInputStream(), GoodExcelModel.class, new GoodExcelModelListener(iGoodsService, sellerId)).build();
            ReadSheet readSheet = EasyExcel.readSheet(0).build();
            excelReader.read(readSheet);
        } catch (Exception e) {
            return BaseResponse.error("读取Excels失败!", false);
        } finally {
            if (excelReader != null) {
                // 这里千万别忘记关闭，读的时候会创建临时文件，到时磁盘会崩的
                excelReader.finish();
            }
        }
        return BaseResponse.ok();
    }


}
