package cn.wise.sc.pay.pass.business.manager.controller.eshop;

import cn.wise.sc.pay.api.eshopping.service.IEshopGoodsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.Min;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@RestController
@Api(description = "商品")
@RequestMapping("/manage/eshop/goods")
@CrossOrigin
@Validated
public class EshopGoodsController {

    @Autowired
    IEshopGoodsService eshopGoodsService;

    @PostMapping
    public BaseResponse addGoods(@RequestBody @Valid EshopGoodsQuery eshopGoodsQuery){
        return eshopGoodsService.addGoods(eshopGoodsQuery);
    }

    @PutMapping
    public BaseResponse updateGoods(@RequestBody @Valid EshopGoodsQuery eshopGoodsQuery){
        return eshopGoodsService.updateGoods(eshopGoodsQuery);
    }

    @GetMapping("goodsPage")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "商品名称", name = "name", paramType = "query", dataType = "String"),
            @ApiImplicitParam(value = "商品状态", name = "marketEnable", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(value = "页码", name = "pageNo", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "每页条数", name = "pageSize", paramType = "query", dataType = "Integer", required = true)
    })
    public BaseResponse goodsPage(@RequestParam(value = "name",required = false) String name,
                                  @RequestParam(value = "marketEnable",required = false) Integer marketEnable,
                                  @RequestParam(value = "pageNo") @DecimalMin("0") Integer pageNo,
                                  @RequestParam(value = "pageSize") Integer pageSize){
        return eshopGoodsService.goodsPage(name, marketEnable, pageNo, pageSize);
    }

    @DeleteMapping
    @ApiImplicitParam(value = "商品id",name = "id",paramType = "query",dataType = "Integer",required = true)
    public BaseResponse deleteGoods(@RequestParam(value = "id", required = true) Integer id){
        return eshopGoodsService.deleteGoods(id);
    }

    @PutMapping("updateStatus")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "商品id", name = "goodsId", paramType = "query", dataType = "String", required = true),
            @ApiImplicitParam(value = "上下架（0下架，1上架）", name = "marketEnable", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "下架原因", name = "underMessage", paramType = "query", dataType = "String")
    })
    public BaseResponse updateGoodsStatus(@RequestParam("goodsId") Integer goodsId,
                                          @RequestParam("marketEnable") Integer marketEnable,
                                          @RequestParam("underMessage") String underMessage){
        return eshopGoodsService.updateGoodsStatus(goodsId,marketEnable,underMessage);
    }

    @GetMapping("/detail/{goodsId}")
    public BaseResponse goodsDetail(@Min(value = 0) @PathVariable("goodsId") Integer goodsId) {
        return eshopGoodsService.goodsDetail(goodsId);
    }
}
