package cn.wise.sc.pay.pass.business.manager.controller.member;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.points.service.IMemberShipService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.point.query.MemberQuery;
import cn.wise.sc.pay.domain.point.vo.MembershipVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 会员相关信息
 */
@CrossOrigin
@RestController
@Api(description = "会员相关  管理端")
@RequestMapping("/manage/member/membership")
public class MemberShipController {
    @Autowired
    IMemberShipService iMemberShipService;

    @GetMapping("/list")
    @ApiOperation("查询所有(条件)")
    @ApiImplicitParam(name = "key", value = "关键字", paramType = "query", dataType = "string")
    public BaseResponse<Page<MembershipVO>> list(PageQuery pageQuery, String key) {

        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        pageWrapperQuery.setPageQuery(pageQuery);

        Map<String,Object> like = MapUtil.newConcurrentHashMap(1);
        if (StrUtil.isNotBlank(key)){
            like.put("name",key);
            pageWrapperQuery.setLike(like);
        }
        return iMemberShipService.page(pageWrapperQuery);
    }
    @GetMapping("/{id}")
    @ApiOperation("查询单个")
    @ApiImplicitParam(name = "id", value = "id", paramType = "path", dataType = "int")
    public BaseResponse getMembership(@PathVariable("id") int id) {

        return iMemberShipService.getMembership(id);
    }

    @PutMapping("/{id}/{status}")
    @ApiOperation("激活/禁用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "状态 1:激活 0:禁用", paramType = "path", dataType = "int")
    })
    public BaseResponse forbiddenOrActive(@PathVariable("id") int id, @PathVariable("status") int status) {

        return iMemberShipService.forbiddenOrActive(id, status);
    }

    @PutMapping
    @ApiOperation("编辑")
    public BaseResponse editMembership(@RequestBody MemberQuery memberQuery) {

        return iMemberShipService.editMembership(memberQuery);
    }

    @DeleteMapping("/{id}")
    @ApiOperation("删除")
    @ApiImplicitParam(name = "id", value = "会员等级id", paramType = "path", dataType = "int")
    public BaseResponse deleteUsers(@PathVariable("id") int id) {

        return iMemberShipService.deleteUsers(id);
    }

    @PostMapping("/new")
    @ApiOperation("添加新的会员等级(折扣传总数如 7.5折=75)")
    public BaseResponse newMembership(@RequestBody MembershipVO membershipVo) {

        return iMemberShipService.newMembership(membershipVo);
    }
}
