package cn.wise.sc.pay.pass.business.manager.controller.merchant;

import cn.wise.sc.pay.api.merchant.service.IStoreService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.Store;
import cn.wise.sc.pay.domain.merchant.query.StoreQuery;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * 商户店铺控制器
 */
@RestController
@CrossOrigin
@RequestMapping("/manage/merchant/store")
@Slf4j

public class StoreController {

    private final IStoreService storeService;

    public StoreController(IStoreService storeService) {
        this.storeService = storeService;
    }

    @ApiOperation(value = "添加店铺信息")
    @PostMapping(path = "/add")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "storeQuery",value = "店铺信息对象",paramType = "body")
            }
    )
    public ResponseEntity addStore(@RequestBody StoreQuery storeQuery){

        BaseResponse responseEntity = storeService.addStore(storeQuery);

        return ResponseEntity.ok(responseEntity.getData());
    }

    @GetMapping(path = "/{id}")
    @ApiOperation(value = "根据店铺id查询店铺信息")
    @ApiImplicitParam(name = "id",value = "店铺id",dataType = "long",paramType = "path")

    public ResponseEntity selectStore(@PathVariable(value = "id") Long id){

       return ResponseEntity.ok(storeService.selectStore(id).getData());

    }

    @PutMapping("/update")
    @ApiOperation(value = "更新店铺店铺信息")
    @ApiImplicitParam(name = "storeQuery",value = "店铺参数传值对象",dataType = "StoreQuery",paramType = "body")
    public ResponseEntity updateStore(@RequestBody StoreQuery storeQuery){

        BaseResponse responseEntity = storeService.updateStore(storeQuery);

        return ResponseEntity.ok(responseEntity.getData());
    }

    @PutMapping("/category/label")
    @ApiOperation(value = "经营类目接口")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "categoryIds",value = "店铺分类id，多个分类用id / 隔开",dataType = "string",paramType = "query"),
                    @ApiImplicitParam(name = "labelIds",value = "店铺标签id，多个标签id用 , 隔开",dataType = "string",paramType = "query"),
                    @ApiImplicitParam(name = "storeId",value = "当前店铺id",dataType = "int",paramType = "query")
            }
    )
    public ResponseEntity updateBusinessCategory(String categoryIds,String labelIds ,Long storeId){


        BaseResponse responseEntity = storeService.updateBusinessCategory(categoryIds, labelIds, storeId);

        return ResponseEntity.ok(responseEntity.getData());
    }

}
