package cn.wise.sc.pay.pass.business.manager.controller.order;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.exhibition.service.IExhibitionOrderService;
import cn.wise.sc.pay.api.merchant.service.IMerchantProfitService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsOrderInfo;
import cn.wise.sc.pay.domain.exhibition.query.GoodsQuery;
import cn.wise.sc.pay.domain.order.OrderStatusUtils;
import cn.wise.sc.pay.domain.order.excel.OrderExcel;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.pass.business.manager.model.OrdersVO;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.metadata.Sheet;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 订单管理
 * @author: qh
 * @create: 2019-12-25 11:11
 **/
@Api(description = "管理端订单管")
@RequestMapping("/manage/orders")
@RestController
@Validated
@CrossOrigin
public class OrdersController {

    @Autowired
    IOrdersService iOrdersService;
    @Autowired
    IExhibitionOrderService iExhibitionOrderService;
    @Autowired
    IMerchantProfitService merchantProfitService;

    @PostMapping("/page")
    @ApiOperation("查询订单分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "consignee", value = "收货人", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "goodsName", value = "商品名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "userName", value = "用户名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "shopId", value = "商铺id", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "orderStatus", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "start", value = "开始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse getPage(@RequestBody PageQuery page, String consignee, String goodsName,
                                String userName, Long shopId, Integer orderStatus, long start, long end) {

        PageWrapperQuery pageWrapperQuery = new PageWrapperQuery();
        pageWrapperQuery.setPageQuery(page);
        //构建等于条件
        Map<String, Object> eq = new HashMap();
        if (shopId != null) {
            eq.put("seller_id", shopId);
        }

        if (orderStatus != null) {
            OrderStatus status = OrderStatusUtils.getStatus(orderStatus);
            if (!OrderStatus.ALL.name().equals(status.name())) {
                eq.put("order_status", OrderStatusUtils.getStatus(orderStatus));
            }
        }
        //构建like条件
        Map<String, Object> like = new HashMap<>();
        if (StrUtil.isNotEmpty(consignee)) {
            like.put("consignee", consignee);
        }
        if (StrUtil.isNotEmpty(goodsName)) {
            like.put("goods", goodsName);
        }
        if (StrUtil.isNotEmpty(userName)) {
            like.put("username", userName);
        }
        Map<String, Object> lt = new HashMap<>();
        Map<String, Object> gt = new HashMap<>();
        if (start != 0 && end != 0) {
            gt.put("create_time", start);
            lt.put("create_time", end);
        }
        pageWrapperQuery.setLike(like);
        pageWrapperQuery.setEq(eq);
        pageWrapperQuery.setGt(gt);
        pageWrapperQuery.setLt(lt);
        BaseResponse baseResponse = iOrdersService.page(pageWrapperQuery);
        if (baseResponse.getCode() == 200) {
            Page page1 = baseResponse.getPage();
            //转换成VO
            List<Orders> list = page1.getRecords();
            List orderVOs = list.stream()
                    .map(arg -> new OrdersVO(arg))
                    .collect(Collectors.toList());
            page1.setRecords(orderVOs);
        }
        return baseResponse;
    }

    @PostMapping("/shop/page")
    @ApiOperation("网上商城查询订单分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "consignee", value = "收货人", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "goodsName", value = "商品名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "userName", value = "用户名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "shopId", value = "商铺id", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "orderStatus", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "start", value = "开始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse getEshopPage(@RequestBody PageQuery page, String consignee, String goodsName,
                                     String userName, Long shopId, Integer orderStatus, long start, long end) {

        PageWrapperQuery pageWrapperQuery = new PageWrapperQuery();
        pageWrapperQuery.setPageQuery(page);
        //构建等于条件
        Map<String, Object> eq = new HashMap();
        eq.put("paas", "ESHOP");
        if (shopId != null) {
            eq.put("seller_id", shopId);
        }
        if (orderStatus != null) {
            OrderStatus status = OrderStatusUtils.getStatus(orderStatus);
            if (!OrderStatus.ALL.name().equals(status.name())) {
                eq.put("order_status", OrderStatusUtils.getStatus(orderStatus));
            }
        }
        //构建like条件
        Map<String, Object> like = new HashMap<>();
        if (StrUtil.isNotEmpty(consignee)) {
            like.put("consignee", consignee);
        }
        if (StrUtil.isNotEmpty(goodsName)) {
            like.put("goods", goodsName);
        }
        if (StrUtil.isNotEmpty(userName)) {
            like.put("username", userName);
        }
        Map<String, Object> lt = new HashMap<>();
        Map<String, Object> gt = new HashMap<>();
        if (start != 0 && end != 0) {
            gt.put("create_time", start);
            lt.put("create_time", end);
        }
        pageWrapperQuery.setLike(like);
        pageWrapperQuery.setEq(eq);
        pageWrapperQuery.setGt(gt);
        pageWrapperQuery.setLt(lt);
        BaseResponse baseResponse = iOrdersService.page(pageWrapperQuery);
        if (baseResponse.getCode() == 200) {
            Page page1 = baseResponse.getPage();
            //转换成VO
            List<Orders> list = page1.getRecords();
            List orderVOs = list.stream().map(arg -> new OrdersVO(arg)).collect(Collectors.toList());
            page1.setRecords(orderVOs);
        }
        return baseResponse;
    }

    @GetMapping
    @ApiOperation("根据订单号查询订单")
    public BaseResponse getOrderByOderNo(String orderNo) {

        Map condition = new HashMap(1);
        condition.put("order_no", orderNo);
        BaseResponse baseResponse = iOrdersService.getOneOrder(JSON.toJSONString(condition));
        if (baseResponse.getCode() == 200) {
            Orders orders = ((OrderResponse) baseResponse).getData();
            OrdersVO ordersVO = new OrdersVO(orders);
            baseResponse.setData(ordersVO);
        }
        return baseResponse;
    }

    @GetMapping("/{id}")
    @ApiOperation("根据id查询订单")
    public BaseResponse getOrderById(@PathVariable("id") long id) {

        Map condition = new HashMap(1);

        if (id != 0) {
            condition.put("id", id);
        } else {
            return BaseResponse.error("id必传", null);
        }
        BaseResponse baseResponse = iOrdersService.getOneOrder(JSON.toJSONString(condition));

        if (baseResponse.getCode() == 200) {
            Orders orders = ((OrderResponse) baseResponse).getData();

            OrdersVO ordersVO = new OrdersVO(orders);

            baseResponse.setData(ordersVO);
        }

        return baseResponse;
    }

    @PostMapping
    @ApiOperation("预下单")
    @ApiImplicitParam(name = "orderQuery", value = "下单", paramType = "body", dataType = "OrderQuery")
    public BaseResponse preOrder(@RequestBody OrderDetailQuery<GoodsQuery> orderQuery) {

        return iExhibitionOrderService.preOrder(orderQuery);
    }

    @GetMapping("/status/{orderNo}")
    @ApiOperation("获取展览订单状态")
    @ApiImplicitParam(name = "orderNo", value = "预下单号", paramType = "path", dataType = "String")
    public BaseResponse getOrderStatus(@PathVariable("orderNo") String orderNo) {
        return iExhibitionOrderService.getOrderStatus(orderNo);
    }

    @GetMapping("/preOderNo")
    @ApiOperation("获取唯一订单号(下单前)")
    public BaseResponse getPreOrderNo() {
        Snowflake snowflake = IdUtil.getSnowflake(8, 8);
        String newIdStr = snowflake.nextIdStr();
        iExhibitionOrderService.getPreOrderNo();
        return BaseResponse.ok(newIdStr);
    }

    @PostMapping("/prePay")
    @ApiOperation("预支付订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse prePayOrder(String orderNo) {
        return iExhibitionOrderService.prePayOrder(orderNo);
    }

    @PostMapping("/pay/success")
    @ApiOperation("支付成功回调")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse payOrderSuccess(String orderNo) {
        BaseResponse response = iExhibitionOrderService.payOrderSuccess(orderNo);
        if (response.code == 200) {
            Map map = new HashMap();
            map.put("order_no", orderNo);
            OrderResponse orderResponse = iOrdersService.getOneOrder(JSON.toJSONString(map));
            Orders orders = orderResponse.getData();
            // 佣金推送
            if (orders != null && orderResponse.getCode() == 200) {
                System.out.println(orderNo + "订单状态改变：" + orderResponse.code + "|" + orders.getOrderStatus());
                if (OrderStatus.WAIT_COMMENT.name().equals(orders.getOrderStatus())) {
                    merchantProfitService.addMoneyData(orders.getSellerId(),
                            orders.getId(), orders.getPaymentMethodName(), orders.getPayPrice(), orders.getCreateTime());
                }
            }
        }
        return response;
    }

    @PutMapping("/pay/cancel")
    @ApiOperation("用户取消支付后续,将订单回退到待支付")
    public BaseResponse payCancel(@NotEmpty(message = "订单号不能为空") String orderNo) {

        OrderDetailQuery orderDetailQuery = new OrderDetailQuery();
        orderDetailQuery.setOrderNo(orderNo);
        orderDetailQuery.setStatus(OrderStatus.WAIT_PAY.name());
        return iOrdersService.editOrder(orderDetailQuery);
    }

    @PostMapping("/pay/fail")
    @ApiOperation("支付失败回调")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse payOrderFail(String orderNo) {
        return iExhibitionOrderService.payOrderFail(orderNo);
    }

    /**
     * 关联物流信息
     *
     * @param logisticsNo 物流号
     * @param orderNo     订单号儿
     * @return Orders
     */
    @PutMapping("/logistics")
    @ApiOperation("订单关联物流")
    public BaseResponse<Orders> addLogistics(@RequestParam("logisticsNo") @NotEmpty(message = "物流号不能为空") String logisticsNo,
                                             @RequestParam("orderNo") @NotEmpty(message = "订单号不为空") String orderNo) {
        OrderDetailQuery<EshopGoodsOrderInfo> orderDetailQuery = new OrderDetailQuery<>();
        orderDetailQuery.setOrderNo(orderNo);
        orderDetailQuery.setLogisticsNo(logisticsNo);
        orderDetailQuery.setStatus(OrderStatus.WAIT_RECEIVE.name());
        return iOrdersService.editOrder(orderDetailQuery);
    }

    /**
     * 统计  根据交易类型统计
     *
     * @return
     */
    @GetMapping("/statistics/status")
    @ApiOperation("统计  根据交易类型统计 ")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "strategy", value = "统计类型  策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "shopName", value = "商品名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse statisticsByStatus(int strategy, String shopName, long start, long end) {

        return iOrdersService.statisticsByStatus(strategy, shopName, start, end);
    }

    /**
     * 统计  根据交易类型统计  (折线图)
     *
     * @return
     */
    @GetMapping("/statistics/chart")
    @ApiOperation("统计  根据交易类型统计  (折线图)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "strategy", value = "统计类型  策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "shopName", value = "商品名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse statisticsChart(int strategy, String shopName, long start, long end) {

        return iOrdersService.statisticsChart(strategy, shopName, start, end);
    }

    /**
     * 统计  根据支付类型 统计金额
     */
    @GetMapping("/statistics/payMethod/Money")
    @ApiOperation("统计  根据支付类型 统计金额")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "strategy", value = "统计类型  策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "shopName", value = "商品名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse statisticsMoneyByPatMethod(int strategy, String shopName, long start, long end) {

        BaseResponse baseResponse = iOrdersService.statisticsMoneyByPatMethod(strategy, shopName, start, end);
        return baseResponse;
    }

    /**
     * 统计  根据支付类型统计笔数
     *
     * @return
     */
    @GetMapping("/statistics/payMethod/Count")
    @ApiOperation("统计  根据支付类型统计笔数")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "strategy", value = "统计类型  策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "shopName", value = "商品名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse statisticsCountByPatMethod(int strategy, String shopName, long start, long end) {

        return iOrdersService.statisticsCountByPatMethod(strategy, shopName, start, end);
    }

    /**
     * 统计  当前统计下的订单
     *
     * @return
     */
    @GetMapping("/statistics/list")
    @ApiOperation("统计 当前统计下的订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "strategy", value = "统计类型  策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "shopName", value = "商品名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "payment", value = "支付方式  0:微信 1:支付宝 2:招行  -1:所有", paramType = "query", dataType = "long")
    })
    public BaseResponse statisticsList(int strategy, String shopName, long start, long end, int payment) {
        return iOrdersService.statisticsList(strategy, shopName, start, end, payment);
    }

    @GetMapping("/excel/orderInfo/{shopId}")
    @ApiOperation("导出代发货订单Excel表(订单物流相关)")
    public void orderLogisticInfoExcel(HttpServletResponse response,
                                       @PathVariable(value = "shopId") Long shopId,
                                       @RequestParam("start") Long start,
                                       @RequestParam("end") Long end) throws IOException {
        ExcelWriter writer = null;
        OutputStream outputStream = response.getOutputStream();
        //添加响应头信息
        response.setHeader("Content-disposition", "attachment; filename=" + "catagory.xls");
        response.setContentType("application/msexcel;charset=UTF-8");//设置类型
        response.setHeader("Pragma", "No-cache");//设置头
        response.setHeader("Cache-Control", "no-cache");//设置头
        response.setDateHeader("Expires", 0);//设置日期头

        //实例化 ExcelWriter
        writer = new ExcelWriter(outputStream, ExcelTypeEnum.XLS, true);

        //实例化表单
        Sheet sheet = new Sheet(1, 0, OrderExcel.class);
        sheet.setSheetName("目录");
        BaseResponse<List<OrderExcel>> baseResponse = iOrdersService.orderInfoExcelByShop(shopId, start, end);
        List<OrderExcel> list = baseResponse.getData();
        //输出
        writer.write(list, sheet);
        writer.finish();
        outputStream.flush();
    }

    public void orderInfoExcelByShop(){

    }

}
