package cn.wise.sc.pay.pass.business.manager.model;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.api.cashier.service.IGoodsService;
import cn.wise.sc.pay.api.cashier.service.impl.GoodsSpecExcel;
import cn.wise.sc.pay.business.goods.model.bean.Goods;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSON;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * @author: qh
 * @create: 2020-07-24 14:31
 **/
public class GoodExcelModelListener extends AnalysisEventListener<GoodExcelModel> {

    private static final Logger LOGGER = LoggerFactory.getLogger(GoodExcelModelListener.class);
    /**
     * 每隔5条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 50   ;
    List<GoodExcelModel> list = new ArrayList<>();
    /**
     * 假设这个是一个DAO，当然有业务逻辑这个也可以是一个service。当然如果不用存储这个对象没用。
     */
    private IGoodsService iGoodsService;
    private String sellerId;

    public GoodExcelModelListener(IGoodsService iGoodsService, String sellerId) {
        // 这里是demo，所以随便new一个。实际使用如果到了spring,请使用下面的有参构造函数
        this.iGoodsService = iGoodsService;
        this.sellerId = sellerId;
    }

    /**
     * 这个每一条数据解析都会来调用
     *
     * @param data    one row value. Is is same as {@link AnalysisContext#readRowHolder()}
     * @param context
     */
    @Override
    public void invoke(GoodExcelModel data, AnalysisContext context) {
        LOGGER.info("解析到一条数据:{}", JSON.toJSONString(data));
        data.setStatus(3);
        data.setSellerId(sellerId);
        list.add(data);
        // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (list.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            list.clear();
        }
    }

    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
        LOGGER.info("所有数据解析完成！");
    }

    /**
     * 加上存储数据库
     */
    private void saveData() {

        LOGGER.info("{}条数据，开始存储数据库！", list.size());

        List<GoodsSpecExcel> batchList = new ArrayList<>(list.size());
        for (GoodExcelModel goodExcelModel : list) {
            GoodsSpecExcel good = new GoodsSpecExcel();
            BeanUtil.copyProperties(goodExcelModel, good);
            batchList.add(good);
        }

        iGoodsService.batchAdd(batchList);
        LOGGER.info("存储数据库成功！");
    }
}
