package cn.wise.sc.pay.pass.business.manager.model;

import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.math.BigDecimal;

/**
 * @description: 管理端订单
 * @author: qh
 * @create: 2019-12-25 19:14
 **/
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class OrdersVO {

    /**
     * 主键
     */
    private Long id;

    /**
     * 订单号
     */
    private String orderNo;

    /**
     * 商品json
     */
    private String goods;

    /**
     * 订单状态
     */
    private String orderStatus;

    /**
     * 支付状态
     */
    private String payStatus;

    /**
     * 用户id
     */
    private Long userId;

    /**
     * 用户电话
     */
    private String userPhone;

    /**
     * 订单总金额
     */
    private BigDecimal totalPrice;

    /**
     * 实际需要支付金额
     */
    private BigDecimal payPrice;

    /**
     * 支付方式
     */
    private String paymentMethodName;

    /**
     * 所属平台
     */
    private String paas;

    /**
     * 备注
     */
    private String remark;

    /**
     * 创建时间
     */
    private Long createTime;

    /**
     * 最后更新时间
     */
    private Long updateTime;

    /**
     * 预留字段
     */
    private String feild1;

    /**
     * 预留字段
     */
    private String feild2;

    /**
     * 预留字段
     */
    private String feild3;

    /**
     * 预留字段
     */
    private String feild4;

    /**
     * 预留字段
     */
    private String feild5;

    /**
     * 收银员
     */
    private String cashier;
    /**
     * 商家id
     */
    private Long sellerId;

    /**
     * 店铺id
     */
    private Long shopId;

    /**
     * 是否为退款
     */
    private boolean isRefund;

    private String address;

    public OrdersVO(Orders orders) {
        this.id = orders.getId();
        this.createTime = orders.getCreateTime();
        this.goods = orders.getGoods();
        this.orderNo = orders.getOrderNo();
        this.orderStatus = getStatus(orders.getOrderStatus());
//        this.paas = orders.getPaas().equals("cashier") ? "线下" : "线上";
        this.paas = orders.getPaymentMethodName();
        this.paymentMethodName = getPaas(orders.getPaymentMethodName());
        this.payPrice = orders.getPayPrice();
        this.payStatus = getPayStatus(orders.getPayStatus());
        this.remark = orders.getRemark();
        this.sellerId = orders.getSellerId();
        this.shopId = orders.getShopId();
        this.totalPrice = orders.getTotalPrice();
        this.userId = orders.getUserId();
        this.userPhone = orders.getUserPhone();
        this.address = "上海虹桥";
        this.updateTime = orders.getUpdateTime();
        this.cashier = orders.getCashier();
        this.isRefund = OrderStatus.REFUNDING.name().equals(orders.getCashier());

    }

    /**
     * 数据库支付状态转中文
     *
     * @param dbStatus
     * @return
     */
    private String getStatus(String dbStatus) {
        if (OrderStatus.WAIT_PAY.name().equals(dbStatus)) {
            return "待支付";
        }
        if (OrderStatus.ENTRY_ORDER.name().equals(dbStatus)) {
            return "挂起";
        }
        if (OrderStatus.CANCELLED.name().equals(dbStatus)) {
            return "已取消";
        }
        if (OrderStatus.COMPLETE.name().equals(dbStatus)) {
            return "已完成";
        }
        if (OrderStatus.WAIT_COMMENT.name().equals(dbStatus)) {
            return "待评价";
        }
        if (OrderStatus.WAIT_DELIVERING.name().equals(dbStatus)) {
            return "待发货";
        }
        if (OrderStatus.WAIT_RECEIVE.name().equals(dbStatus)) {
            return "待收货";
        }
        if (OrderStatus.CONFIRM.name().equals(dbStatus)) {
            return "支付中";
        }
        if (OrderStatus.REFUND.name().equals(dbStatus)) {
            return "已退款";
        }
        return "无效订单";
    }

    private String getPayStatus(String payStatus) {
        if (PayStatus.PAY_YES.name().equals(payStatus)) {
            return "已支付";
        } else {
            return "未支付";
        }
    }

    private String getPaas(String paas) {
        if ("微信".equals(paas) || "支付宝".equals(paas)) {
            return "聚合支付";
        }
        return paas;
    }
}
