package cn.wise.sc.pay.paas.open.api.service;

import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.goods.service.ISpcDetailService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.Do.OrderGoodsDO;
import cn.wise.sc.pay.domain.order.OrderStatusUtils;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.alibaba.fastjson.JSON;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @description: 订单实现类
 * @author: qh
 * @create: 2019-12-04 17:12
 **/
@Service("paasOrderServiceImpl")
public class IPaasOrderServiceImpl implements IPaasOrderService {

    final
    IOrdersService iOrderService;
    final
    ISpcDetailService iSpcDetailService;

    public IPaasOrderServiceImpl(IOrdersService iOrderService, ISpcDetailService iSpcDetailService) {
        this.iOrderService = iOrderService;
        this.iSpcDetailService = iSpcDetailService;
    }

    @Override
    public BaseResponse page(PageWrapperQuery pageWrapperQuery) {
        return iOrderService.page(pageWrapperQuery);
    }

    @Override
    public BaseResponse cancelOrder(String orderNo) {
        return iOrderService.cancelOrder(orderNo);
    }

    @Override
    public BaseResponse editOrder(OrderDetailQuery orderQuery) {
        return iOrderService.editOrder(orderQuery);
    }

    @Override
    public OrderResponse getOneOrder(String condition) {
       return iOrderService.getOneOrder(JSON.toJSONString(condition));
    }

    @Override
    public BaseResponse payOrder(@NonNull String orderNo) {

        if (StringUtils.isEmpty(orderNo)) {
            return BaseResponse.error("订单号不能为空", null);
        }
        Map<String, Object> condition = new HashMap<>();
        condition.put("order_no", orderNo);
        String conditionJson = JSON.toJSONString(condition);

        String goods = iOrderService.getOneOrder(conditionJson).getData().getGoods();

        List<OrderGoodsDO> orderGoodsDOS = JSON.parseArray(goods, OrderGoodsDO.class);

        ThreadFactory threadFactory = r -> new Thread();

        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(orderGoodsDOS.size(),
                orderGoodsDOS.size(),
                2000,
                TimeUnit.MILLISECONDS,
                new LinkedBlockingDeque(),
                threadFactory);

        for (OrderGoodsDO goodsDO : orderGoodsDOS) {
            threadPoolExecutor.execute(() -> {
                boolean flag = true;
                List<Long> spcIds = goodsDO.getSpcDetailId();
                for (Long spcDetailId : spcIds) {
                    while (flag) {
                        flag = iSpcDetailService.subStore(spcDetailId, goodsDO.getNum()).getData();
                        if (flag == true) {
                            flag = false;
                        }
                    }
                }
            });
        }
        threadPoolExecutor.shutdown();
        boolean isSuccess = iOrderService.changeOrderStatus(orderNo, OrderStatus.COMPLETE, null).getCode() == 200;

        if (isSuccess) {
            return BaseResponse.ok("支付成功");
        } else {
            iOrderService.changeOrderStatus(orderNo, OrderStatus.CANCELLED, null);
            return BaseResponse.ok("订单支付失败,请重新下单");
        }
    }

    @Override
    public BaseResponse createOrder(OrderDetailQuery orderDetail) {
        return iOrderService.createOrder(orderDetail);
    }

    @Override
    public BaseResponse getOrdersPage(Map<String, Object> condition, PageQuery pageQuery) {
        PageWrapperQuery query = new PageWrapperQuery();
        query.setPageQuery(new PageQuery());
        query.setEq(condition);
        return iOrderService.page(query);
    }

    @Override
    public BaseResponse getOrders(Map<String, Object> condition) {
        return iOrderService.getOrders(JSON.toJSONString(condition));
    }

    @Override
    public BaseResponse delOrders(String orderNo, Map<String, Object> condition) {
        String conditionJson = JSON.toJSONString(condition);
        return iOrderService.delOrder(orderNo, conditionJson);
    }

    @Override
    public BaseResponse getPreOrderNo() {
        return iOrderService.getOrderNo();
    }

    @Override
    public BaseResponse pageSeller(int seller, int status, long start,
                                   long end, PageQuery pageQuery) {

        PageWrapperQuery pageWrapperQuery = new PageWrapperQuery();
        Map<String, Object> geMap = new HashMap();
        Map<String, Object> leMap = new HashMap();

        Map<String, Object> eqMap = new HashMap();

        if (seller != 0) {
            eqMap.put("seller_id", seller);
        }
        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        if (orderStatus != OrderStatus.ALL) {
            eqMap.put("order_status", orderStatus.name());
            pageWrapperQuery.setEq(eqMap);
        }
        if (start != 0 && end != 0) {
            geMap.put("update_time", start);
            leMap.put("update_time", end);
            pageWrapperQuery.setGe(geMap);
            pageWrapperQuery.setLe(leMap);
        }

        pageWrapperQuery.setPageQuery(pageQuery);
        OrderResponse response = iOrderService.page(pageWrapperQuery);
        return response;

    }

}
