package cn.wise.sc.pay.api.association.service;

import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.*;

/**
 * 文章评论
 */
@FeignClient(value = "saas-v1-association",path = "/association/comment/",fallback = ArticleCommentServiceImpl.class)
public interface IArticleCommentService {

    /**
     * 评论
     * @param comment 评论实体
     * @return
     */
    @PostMapping("/addComment")
    BaseResponse addComment(@RequestBody Comment comment);

    /**
     * 根据ids删除评论 多个用英文逗号隔开
     * @param idStr
     * @return
     */
    @DeleteMapping("/deleteComment")
    BaseResponse deleteComment(@RequestParam("idStr") String idStr);

    /**
     * 评论点赞
     * @param id 评论id
     * @return
     */
    @PutMapping("/updatePraiseNum")
    BaseResponse updateCommentPraise(@RequestParam("id")Long id);

    /**
     * @param pageQuery 分页实体
     * @return
     */
    @GetMapping("/selectComment")
    @ApiOperation("评论列表 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "条数", paramType = "path", dataType = "Integer")
    })
    BaseResponse getCommentList(@SpringQueryMap PageQuery pageQuery);

    /**
     * 根据评论主体 评论列表 分页
     * @param targetId 评论主体id
     * @param pageNo 分页
     * @param pageSize 分页
     * @return
     */
    @GetMapping("/commentPage")
    BaseResponse getCommentsByTargetId(@RequestParam("targetId")Long targetId,
                                       @RequestParam("pageNo") Integer pageNo,
                                       @RequestParam("pageSize") Integer pageSize);
}
