package cn.wise.sc.pay.api.association.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.association.bean.Banner;
import cn.wise.sc.pay.domain.association.query.BannerQuery;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(value = "saas-v1-association", path = "/association/banner", fallback = BannerServiceImpl.class)
public interface IBannerService {


    /**
     * 获取社群分类导航
     *
     * @return
     */
    @GetMapping("/list")
    BaseResponse<List<Banner>> list();

    /**
     * 社群 人文下分类导航
     * @return
     */
    @GetMapping("/rwList")
    BaseResponse rwList();
    /**
     * 下拉选择社群分类，排除“推荐”
     *
     * @return
     */
    @GetMapping("/selectList")
    BaseResponse<List<Banner>> selectList();


    /**
     * 新增社群分类
     *
     * @param bannerQuery 社区请求实体
     * @return
     */
    @PostMapping("/addBanner")
    BaseResponse addBanner(@RequestBody BannerQuery bannerQuery);


    /**
     * 更新社群分类名称
     *
     * @param banner 社区实体
     * @return
     */
    @PutMapping("updateBannerName")
    BaseResponse updateBannerName(@RequestBody Banner banner);


    /**
     * 更新社群分类排序
     *
     * @param tId
     * @param oId
     * @return
     */
    @PutMapping("updateBannerOrder")
    BaseResponse updateBannerOrder(@RequestParam("tId")Long tId,@RequestParam("oId") Long oId);

    /**
     * 删除社群分类
     *
     * @param id 社区id
     * @return
     */
    @DeleteMapping("deleteBanner")

    public BaseResponse deleteBannerById(@RequestParam("id")Long id);
}
