package cn.wise.sc.pay.business.association.controller;


import cn.wise.sc.pay.business.association.service.IArticleBannerService;
import cn.wise.sc.pay.business.association.service.IArticleService;
import cn.wise.sc.pay.business.association.service.ICollectionService;
import cn.wise.sc.pay.business.association.service.IConcernService;
import cn.wise.sc.pay.business.association.service.IPraiseService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.association.bean.Article;
import cn.wise.sc.pay.domain.association.query.ArticleQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@RestController
@Api(description = "图文/文章")
@RequestMapping("/association/article")
@CrossOrigin
public class ArticleController {

    @Autowired
    IArticleService articleService;

    @Autowired
    IArticleBannerService bannerService;

    @Autowired
    ICollectionService collectionService;

    @Autowired
    IPraiseService praiseService;

    @Autowired
    IConcernService concernService;

    @PostMapping("addArticle")
    @ApiOperation("发表图文分享")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "article", value = "图文/文章", paramType = "body", dataType = "Article"),
            @ApiImplicitParam(name = "files", value = "图文/文章相关资源", paramType = "body", dataType = "MultipartFile")
    })
    public BaseResponse addArticle(@RequestBody ArticleQuery articleQuery) {

        return articleService.addArticle(articleQuery);
    }

    @PostMapping("uploadPic")
    @ApiOperation("上传图片或视频")
    @ApiImplicitParam(name = "file", value = "图片/视频", paramType = "body", dataType = "MultipartFile")
    public BaseResponse uploadPic(MultipartFile file) {
        return articleService.uploadPic(file);
    }

    @PutMapping("updateStatus")
    @ApiOperation("修改文章/图文审核状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "文章id,多个用逗号分割", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "status", value = "文章状态", paramType = "path", dataType = "String")
    })
    public BaseResponse updateStatus(String ids, byte status) {
        List<Long> idList = new ArrayList<>();
        if (StringUtils.isNotEmpty(ids)) {
            String[] idStrs = ids.split(",");
            for (String idStr : idStrs) {
                idList.add(Long.parseLong(idStr));
            }
        }
        return articleService.updateArticleStatus(idList, status);
    }

    @DeleteMapping("deleteArticles")
    @ApiOperation("批量删除图文 或 批量推荐图文")
    public BaseResponse deleteArticles(String ids, Integer status) {
        if (status == null || StringUtils.isBlank(ids)) {
            return BaseResponse.error("参数错误", Article.class);
        }
        List<Long> idList = new ArrayList<>();
        if (StringUtils.isNotBlank(ids)) {
            String[] idStrs = ids.split(",");
            for (String idStr : idStrs) {
                idList.add(Long.parseLong(idStr));
            }
        }
        if (status == 1) {
            return articleService.deleteArticlesById(idList);
        } else {
            return bannerService.recommend(idList);
        }
    }

    @PutMapping("updateViewNum")
    @ApiOperation("修改浏览量")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "文章id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "viewNum", value = "浏览量", paramType = "path", dataType = "Long")
    })
    public BaseResponse updateViewNum(Long id, Long viewNum) {
        if (id == null || viewNum == null) {
            return BaseResponse.error("参数错误", Article.class);
        }
        Article article = articleService.getById(id);
        if (article != null) {
            article.setViewNum(viewNum);
            Boolean res = articleService.updateById(article);
            return res ? BaseResponse.ok("修改成功") : BaseResponse.error("修改失败", Article.class);
        }
        return BaseResponse.error("文章不存在", Article.class);

    }

    @PutMapping("viewNumUp")
    @ApiOperation("图文/文章 浏览量+1")
    @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long")
    public BaseResponse updateViewNum(Long id) {
        return articleService.updateArticleViewNum(id);
    }

    @PutMapping("praiseNumUp")
    @ApiOperation("点赞图文/文章")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "articleId", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse updatePraiseNum(Long articleId, Long userId) {
        return praiseService.addPraise(articleId, userId);
    }

    @GetMapping("judgePraise")
    @ApiOperation("判断用户对图文/文章已点赞")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "articleId", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse judgePraise(Long articleId, Long userId) {
        return praiseService.judgePraise(articleId, userId);
    }

    @PutMapping("deletePraise")
    @ApiOperation("取消点赞")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse deletePraise(Long id, Long userId) {
        return praiseService.deletePraise(id, userId);
    }

    @PutMapping("deleteRecommend")
    @ApiOperation("移除推荐")
    @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long")
    public BaseResponse deleteRecommend(Long id) {
        return bannerService.deleteRecommend(id);
    }

    @PutMapping("recommend")
    @ApiOperation("推荐")
    @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long")
    public BaseResponse rcommend(Long id) {
        if (id != null) {
            List<Long> list = new ArrayList<>();
            list.add(id);
            return bannerService.recommend(list);
        }
        return BaseResponse.error("参数错误", Article.class);
    }

    @PostMapping("collection")
    @ApiOperation("收藏")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "articleId", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse collection(Long articleId, Long userId) {
        return collectionService.addCollection(articleId, userId);
    }

    @GetMapping("judgeCollection")
    @ApiOperation("判断用户对图文/文章已收藏")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "articleId", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse judgeCollection(Long articleId, Long userId) {
        return collectionService.judgeCollection(articleId, userId);
    }

    @PutMapping("deleteCollection")
    @ApiOperation("取消收藏")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse deleteCollection(Long id, Long userId) {
        return collectionService.deleteCollection(id, userId);
    }

    @GetMapping("selectById")
    @ApiOperation("根据id获取图文详情")
    @ApiImplicitParam(name = "id", value = "文章id", paramType = "path", dataType = "Long")
    public BaseResponse selectArticleById(Long id) {
        return articleService.selectArticleInfo(id);
    }

    @GetMapping("getArticleNotVerify")
    @ApiOperation("获取未审核图文列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "authorName", value = "发表人名称", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "timeBucket", value = "搜索时间段", paramType = "path", dataType = "TimeBucket"),
            @ApiImplicitParam(name = "pageQuery", value = "分页", paramType = "path", dataType = "PageQuery")
    })
    public BaseResponse<Page<Article>> getArticleNotVerify(String authorName,
                                                           long start, long end, Integer pageNo, Integer pageSize) {
        TimeBucket timeBucket = new TimeBucket();
        timeBucket.setStartTime(start);
        timeBucket.setEndTime(end);
        PageQuery pageQuery = new PageQuery();
        pageQuery.setPageSize(pageSize);
        pageQuery.setPageNo(pageNo);
        return articleService.articlesNotVerify(authorName, timeBucket, pageQuery);
    }

    @GetMapping("getArticleVerify")
    @ApiOperation("获取审核通过的图文列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "搜索文章分类", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "order", value = "评论数排序 asc升序 desc降序", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "name", value = "文章内容", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "title", value = "文章标题", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "pageQuery", value = "分页", paramType = "path", dataType = "PageQuery")
    })
    public BaseResponse<Page<Article>> getArticleVerify(Integer type, String order, String name, String title,
                                                        Integer pageNo, Integer pageSize) {
        PageQuery pageQuery = new PageQuery();
        pageQuery.setPageNo(pageNo);
        pageQuery.setPageSize(pageSize);
        return articleService.articlesVerify(type, order, name, title, pageQuery);
    }

    @GetMapping("userCollections")
    @ApiOperation("用户收藏图文/文章")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse<Page<Article>> getCollectionArticle(Long userId, PageQuery pageQuery) {
        return articleService.getCollectionArticle(userId, pageQuery);
    }

    @GetMapping("userPublishs")
    @ApiOperation("用户发布的图文/文章")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse<Page<Article>> getUserPublishArticle(Long userId, PageQuery pageQuery) {
        return articleService.getUserPublishArticle(userId, pageQuery);
    }

    @GetMapping("userPraises")
    @ApiOperation("用户点赞的图文/文章")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse<Page<Article>> getUserPraiseArticle(Long userId, PageQuery pageQuery) {
        return articleService.getUserPraiseArticle(userId, pageQuery);
    }

    @GetMapping("ponArticle")
    @ApiOperation("上一篇或下一篇图文 pon=0代表上一篇，pon=1代表下一篇")
    public BaseResponse previousOrNextArticle(Long articleId, Integer pon) {
        return articleService.preOrNextArticle(articleId, pon, 0);
    }

    @GetMapping("ponVerifyArticle")
    @ApiOperation("上一篇或下一篇已审核图文 pon=0代表上一篇，pon=1代表下一篇")
    public BaseResponse previousOrNextVerifyArticle(Long articleId, Integer pon) {
        return articleService.preOrNextArticle(articleId, pon, 1);
    }
}

