package cn.wise.sc.pay.business.association.controller;


import cn.wise.sc.pay.business.association.service.IConcernService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.association.bean.Concern;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-25
 */
@Api(description = "关注")
@RestController
@RequestMapping("/association/concern/")
@CrossOrigin
public class ConcernController {

    @Autowired
    IConcernService concernService;

    @PostMapping("addConcern")
    @ApiOperation("关注对方")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "meId", value = "我的id", paramType = "body", dataType = "Long"),
            @ApiImplicitParam(name = "heId", value = "被关注人id", paramType = "body", dataType = "Long"),
            @ApiImplicitParam(name = "heName", value = "被关注人名称", paramType = "body", dataType = "String"),
            @ApiImplicitParam(name = "heImage", value = "被关注人头像", paramType = "body", dataType = "String")
    })
    public ResponseEntity addConcern(@RequestBody Concern concern) {
        return concernService.addConcern(concern);
    }

    @DeleteMapping("deleteConcern")
    @ApiOperation("取消关注")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "meId", value = "我的id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "heId", value = "被关注人id", paramType = "path", dataType = "Long")
    })
    public ResponseEntity deleteConcern(Long meId, Long heId) {
        return concernService.deleteConcern(meId,heId);
    }

    @GetMapping("myConcerns")
    @ApiOperation("我的关注/粉丝列表")
    public ResponseEntity<List<Concern>> getConcernList(Long meId,Integer type) {
        return concernService.getConcernList(meId,type);
    }

    @GetMapping("mainData")
    @ApiOperation("首页数据")
    @ApiImplicitParam(name = "userId", value = "我的id", paramType = "path", dataType = "Long")
    public ResponseEntity<Map<String,Integer>> mainData(Long userId){
        return concernService.getMainData(userId);
    }

    @GetMapping("judgeConcern")
    @ApiOperation("是否关注")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "meId", value = "我的id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "heId", value = "被关注人id", paramType = "path", dataType = "Long")
    })
    public BaseResponse judgeConcern(Long meId, Long heId){
        if(meId != null && heId != null){
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("me_id",meId);
            wrapper.eq("he_id",heId);
            wrapper.last("limit 1");
            Concern concern = concernService.getOne(wrapper);
            return concern != null ? BaseResponse.ok(true) : BaseResponse.error("未关注", false);
        }
        return BaseResponse.error("参数错误",Concern.class);
    }
}

