package cn.wise.sc.pay.business.association.service.impl;

import cn.wise.sc.pay.business.association.mapper.PraiseMapper;
import cn.wise.sc.pay.business.association.service.IArticleService;
import cn.wise.sc.pay.business.association.service.IPraiseService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.association.bean.Article;
import cn.wise.sc.pay.domain.association.bean.Praise;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-25
 */
@Service
public class PraiseServiceImpl extends ServiceImpl<PraiseMapper, Praise> implements IPraiseService {

    @Autowired
    IArticleService articleService;

    @Override
    public BaseResponse addPraise(Long articleId, Long userId) {
        if(articleId == null || userId == null){
            return BaseResponse.error("参数有误",Article.class);
        }
        Praise praise = Praise.builder()
                .articleId(articleId)
                .userId(userId)
                .createTime(System.currentTimeMillis())
                .build();
        Boolean result = false;
        try {
           result = this.save(praise);
        }catch (Exception e){
            return BaseResponse.error("已点过赞",Article.class);
        }
        Article article = articleService.getById(articleId);
        if(article != null){
            article.setPraiseNum(article.getPraiseNum().longValue() + 1);
            articleService.updateById(article);
        }
        return result ? BaseResponse.ok("点赞成功")
                : BaseResponse.error("点赞失败",Article.class);
    }

    @Override
    public BaseResponse judgePraise(Long articleId, Long userId) {
        if(articleId == null || userId == null){
            return BaseResponse.error("参数有误",Article.class);
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("user_id",userId);
        wrapper.eq("article_id",articleId);
        Praise praise = this.getOne(wrapper);
        if(praise != null){
            return BaseResponse.ok(1);
        }
        else {
            return BaseResponse.ok(0);
        }
    }

    @Override
    public BaseResponse deletePraise(Long id, Long userId) {
        if(userId == null || id == null){
            return BaseResponse.error("参数有误",Article.class);
        }
        Article article = articleService.getById(id);
        if(article != null){
            article.setPraiseNum(article.getPraiseNum().longValue() - 1);
            articleService.updateById(article);
        }
        UpdateWrapper<Praise> wrapper = new UpdateWrapper<>();
        wrapper.eq("article_id",id);
        wrapper.eq("user_id",userId);
        boolean result = this.remove(wrapper);
        return result ? BaseResponse.ok("取消点赞成功") : BaseResponse.error("取消点赞失败",Article.class);
    }
}
