package cn.wise.sc.pay.api.cashier.service;

import cn.wise.sc.pay.api.cashier.service.impl.CashierOrdersServiceImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.*;

/**
 * @author qh
 */
@FeignClient(value = "saas-v1-cashier", path = "/order", fallback = CashierOrdersServiceImpl.class)
public interface ICashierOrdersService {

    @PostMapping("/new")
    BaseResponse newOrder(@RequestBody OrderDetailQuery orderQuery);

    @PostMapping("/new/entry")
    BaseResponse newEntryOrder(@RequestBody OrderDetailQuery orderQuery);

    @PutMapping("/entry/")
    BaseResponse editEntryOrder(@RequestBody OrderDetailQuery orderQuery);

    @GetMapping("list/user/{userId}/{status}")
    BaseResponse listUsers(@PathVariable("userId") int userId, @PathVariable("status") int status);

    @GetMapping("/list/seller/{seller}/{status}")
    BaseResponse listSeller(@PathVariable("seller") int seller, @PathVariable("status") int status,
                            @RequestParam("start") long start, @RequestParam("end") long end);

    @PutMapping("{orderNo}")
    BaseResponse payOrder(@PathVariable("orderNo") String orderNo);

    @GetMapping("/orderNo")
    BaseResponse getOrderNo();

    @DeleteMapping("{orderNo}")
    BaseResponse delOrder(@PathVariable("orderNo") String orderNo);

    /**
     * 查询商户订单 分页
     * @param seller 商户id
     * @param status 订单状态
     * @param start 开始
     * @param end 结束
     * @param pageQuery 分页条件
     * @return
     */
    @GetMapping("/page/seller/{seller}/{status}")
    BaseResponse pageSeller(@PathVariable("seller") int seller, @PathVariable("status") int status,
                            @RequestParam("start") long start, @RequestParam("end") long end, @SpringQueryMap PageQuery pageQuery);
}
