package cn.wise.sc.pay.business.cashier.controller;

import cn.wise.sc.pay.business.cashier.model.TrolleyGoodsDetail;
import cn.wise.sc.pay.business.cashier.service.ITrolleyService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import java.util.List;

/**
 * @description: 网上商城购物车
 * @author: qh
 * @create: 2020-03-27 11:00
 **/
@Validated
@CrossOrigin
@RestController
@RequestMapping("/cashier/shoppingTrolley")
public class TrolleyController {

    final
    ITrolleyService iEshopTrolleyService;

    public TrolleyController(ITrolleyService iEshopTrolleyService) {
        this.iEshopTrolleyService = iEshopTrolleyService;
    }

    @PostMapping
    public BaseResponse<List<TrolleyGoodsDetail>> joinTrolley(@RequestBody TrolleyGoodsDetail trolleyGoodsDetail) {
        return iEshopTrolleyService.joinTrolley(trolleyGoodsDetail);
    }


    /**
     * 移除购物车
     *
     * @param goodId 商品id
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @DeleteMapping
    public BaseResponse<List<TrolleyGoodsDetail>> removeTrolley(@RequestParam("goodId") @Min(value = 1, message = "无效的商品号") Long goodId,
                                                                @RequestParam("userId") @Min(value = 1, message = "无效的用户id") Long userId,
                                                                @RequestParam("skuId") Long skuId) {
        return iEshopTrolleyService.removeTrolley(goodId, userId, skuId);
    }

    /**
     * 获取购物车里的所有商品
     *
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @GetMapping
    public BaseResponse<List<TrolleyGoodsDetail>> getGoods(@RequestParam("userId") @Min(value = 1, message = "无效的用户id") Long userId) {
        return iEshopTrolleyService.getGoods(userId);
    }

    /**
     * 编辑购物车
     *
     * @param list   购物车信息
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @PutMapping
    public BaseResponse<List<TrolleyGoodsDetail>> edit(@RequestBody List<TrolleyGoodsDetail> list, @RequestParam("userId") Long userId) {
        return iEshopTrolleyService.edit(list, userId);
    }
}
