package cn.wise.sc.pay.api.eshopping.service;

import cn.wise.sc.pay.api.eshopping.service.hystrix.EshopGoodsServiceImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.query.CalCulate;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsQuery;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.DecimalMin;

/**
 * 文章
 */
@FeignClient(value = "saas-v1-eshopping", path = "/eshop/goods", fallback = EshopGoodsServiceImpl.class)
public interface IEshopGoodsService {

    @PostMapping
    BaseResponse addGoods(@RequestBody @Valid EshopGoodsQuery eshopGoodsQuery);

    @PutMapping
    BaseResponse updateGoods(@RequestBody @Valid EshopGoodsQuery eshopGoodsQuery);

    @GetMapping("goodsPage")
    BaseResponse goodsPage(@RequestParam("name") String name,
                           @RequestParam("marketEnable") Integer marketEnable,
                           @RequestParam(value = "pageNo", required = true) @DecimalMin("0") Integer pageNo,
                           @RequestParam(value = "pageSize", required = true) Integer pageSize);

    @DeleteMapping
    BaseResponse deleteGoods(@RequestParam(value = "id", required = true) Integer id);

    @PutMapping("updateStatus")
    BaseResponse updateGoodsStatus(@RequestParam("goodsId") Integer goodsId,
                                   @RequestParam("marketEnable") Integer marketEnable,
                                   @RequestParam("underMessage") String underMessage);

    @GetMapping("conditionGoodsPage")
    BaseResponse conditionGoodsPage(
            @RequestParam(value = "categoryId", required = false) Integer categoryId,
            @RequestParam(value = "price", required = false) Integer price,
            @RequestParam(value = "createTime", required = false) Integer createTime,
            @RequestParam(value = "multiple", required = false) Integer multiple,
            @RequestParam(value = "pageNo") @DecimalMin("0") Integer pageNo,
            @RequestParam(value = "pageSize") Integer pageSize);

    /**
     * 获取商品详情
     *
     * @param goodsId 商品id
     * @return goodVo
     */
    @GetMapping("/detail/{goodsId}")
    BaseResponse goodsDetail(@PathVariable("goodsId") Integer goodsId);

    /**
     * 计算物流运费
     *
     * @param calCulate
     * @return
     */
    @PostMapping("calculate")
    BaseResponse calculatePrice(@RequestBody CalCulate calCulate);



    @PutMapping("/incrCommentNum/{id}")
     BaseResponse incrCommentNumById(@PathVariable("id") Integer id);
}
