package cn.wise.sc.pay.business.eshopping.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.eshopping.service.IEshopGoodsService;
import cn.wise.sc.pay.business.eshopping.service.IEshopTrolleyService;
import cn.wise.sc.pay.business.eshopping.utils.RedisObjConstant;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.query.TrolleyGoodsDetail;
import com.alibaba.fastjson.JSON;
import org.redisson.api.RList;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @description: 购物车service实现
 * @author: qh
 * @create: 2020-03-27 11:06
 **/
@Service
public class EshopTrolleyServiceImpl implements IEshopTrolleyService {

    /**
     * 过期时间
     */
    private final static Long EXPIRE_TIME = 60 * 60 * 24 * 7L;

    final
    RedissonClient redissonClient;

    final
    IEshopGoodsService iEshopGoodsService;

    public EshopTrolleyServiceImpl(IEshopGoodsService iEshopGoodsService, RedissonClient redissonClient) {
        this.iEshopGoodsService = iEshopGoodsService;
        this.redissonClient = redissonClient;
    }

    @Override
    public BaseResponse<List<TrolleyGoodsDetail>> joinTrolley(TrolleyGoodsDetail trolleyGoodsDetail) {

        //获取购物车
        RList<TrolleyGoodsDetail> rList = redissonClient.getList(RedisObjConstant.SHOPPINGTROLLEY + trolleyGoodsDetail.getUserId());
        boolean flag = true;
        for (TrolleyGoodsDetail detail : rList) {
            if (detail.getGoodId().intValue() == trolleyGoodsDetail.getGoodId()) {
                if (StrUtil.isNotEmpty(detail.getSku()) && detail.getSku().equals(trolleyGoodsDetail.getSku())) {
                    TrolleyGoodsDetail updateTrolly = new TrolleyGoodsDetail();
                    BeanUtils.copyProperties(detail, updateTrolly);
                    updateTrolly.setCount(detail.getCount() + trolleyGoodsDetail.getCount());
                    updateTrolly.setPrice(detail.getPrice());
                    flag = false;
                    int detailIndex = rList.indexOf(detail);
                    rList.fastSet(detailIndex, updateTrolly);
                    break;
                }
            }
        }

        if (flag) {
            TrolleyGoodsDetail trolleyGoodsDetailNew = new TrolleyGoodsDetail();
            BeanUtils.copyProperties(trolleyGoodsDetail, trolleyGoodsDetailNew);
            trolleyGoodsDetailNew.setPrice(trolleyGoodsDetail.getPrice());
            rList.add(trolleyGoodsDetailNew);
        }

        //加入购物车 设置过期时间
        rList.expire(EXPIRE_TIME, TimeUnit.SECONDS);
        List<TrolleyGoodsDetail> result = rList.range(-1);
        return BaseResponse.ok(result);
    }

    @Override
    public BaseResponse<List<TrolleyGoodsDetail>> removeTrolley(Long goodId, Long userId, Long skuId) {

        //获取购物车
        RList<TrolleyGoodsDetail> rList = redissonClient.getList(RedisObjConstant.SHOPPINGTROLLEY + userId);
        if (rList == null || rList.size() == 0) {
            return BaseResponse.error("购物车为空!", List.class.getName());
        } else {
            //根据商品id移除购物车
            rList.removeIf(arg -> {
                //如果skuId有的时候 同时判断skuId
                if (skuId != null && skuId != 0) {
                    return arg.getGoodId().longValue() == goodId &&
                            arg.getSkuId().longValue() == skuId;
                }
                return arg.getGoodId().longValue() == goodId;
            });
            rList.expire(EXPIRE_TIME, TimeUnit.SECONDS);
        }
        //返回购物车
        List<TrolleyGoodsDetail> result = rList.range(-1);
        return BaseResponse.ok(result);
    }

    @Override
    public BaseResponse<List<TrolleyGoodsDetail>> getGoods(Long userId) {
        //获取购物车
        RList<TrolleyGoodsDetail> rList = redissonClient.getList(RedisObjConstant.SHOPPINGTROLLEY + userId);
        if (rList == null || rList.size() == 0) {
            return BaseResponse.error("没有找到您的购物车信息!", List.class.getName());
        } else {
            List<TrolleyGoodsDetail> result = rList.range(-1);
            return BaseResponse.ok(result);
        }
    }

    @Override
    public BaseResponse<List<TrolleyGoodsDetail>> edit(List<TrolleyGoodsDetail> list, Long userId) {

        if (list == null || list.size() == 0) {
            return BaseResponse.error("购物车商品集合为空!", List.class.getName());
        }
        RList<TrolleyGoodsDetail> rList = redissonClient.getList(RedisObjConstant.SHOPPINGTROLLEY + userId);
        rList.clear();
        rList.addAll(list);
        List<TrolleyGoodsDetail> result = rList.range(-1);
        return BaseResponse.ok(result);
    }

    public static void main(String[] args) {
        List<TrolleyGoodsDetail> list = new ArrayList<>();
        TrolleyGoodsDetail trolleyGoodsDetail = new TrolleyGoodsDetail();
        trolleyGoodsDetail.setGoodId(1);
        trolleyGoodsDetail.setSku("asdasd");
        trolleyGoodsDetail.setTitle("wqeqwe");
        trolleyGoodsDetail.setPrice(new BigDecimal(80));
        trolleyGoodsDetail.setCount(1);
        trolleyGoodsDetail.setImg("asdasd");
        list.add(trolleyGoodsDetail);
        System.out.println(JSON.toJSONString(list));
    }
}
