package cn.wise.sc.pay.business.eshopping.service.impl;

import cn.wise.sc.pay.business.eshopping.configure.FastDFSUtils;
import cn.wise.sc.pay.business.eshopping.mapper.EshopTypeMapper;
import cn.wise.sc.pay.business.eshopping.service.IEshopTypeService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopType;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopTypeQuery;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopTypeTree;
import cn.wise.sc.pay.domain.eshopping.model.query.TreeUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@Service
public class EshopTypeServiceImpl extends ServiceImpl<EshopTypeMapper, EshopType> implements IEshopTypeService {

    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    public BaseResponse getGoodsTypeTree() {
        List<EshopType> types = this.list();
        if(types != null && types.size() > 0){
            types.stream().forEach(arg->{
                arg.setIcon(fastDFSUtils.fullUrl(arg.getIcon()));
            });
        }
        List<EshopTypeTree> typeTrees = types.stream().map(EshopTypeTree::new)
                .sorted(Comparator.comparingInt(EshopTypeTree::getSortNo)).collect(Collectors.toList());

        List<EshopTypeTree> tree = TreeUtil.buildByLoop(typeTrees,0);
        return BaseResponse.ok(tree);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse addGoodsType(EshopTypeQuery eshopType) {
        EshopType eType = EshopType.builder()
                .parentId(eshopType.getParentId())
                .typeName(eshopType.getTypeName())
                .sortNo(eshopType.getSortNo())
                .icon(eshopType.getIcon())
                .createTime(System.currentTimeMillis())
                .modifyTime(System.currentTimeMillis())
                .build();
        this.save(eType);
        return BaseResponse.ok(eType);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse updateGoodsType(EshopTypeQuery eshopType) {
        EshopType eType = EshopType.builder()
                .typeName(eshopType.getTypeName())
                .typeId(eshopType.getTypeId())
                .parentId(eshopType.getParentId())
                .sortNo(eshopType.getSortNo())
                .icon(eshopType.getIcon())
                .modifyTime(System.currentTimeMillis())
                .build();
        return BaseResponse.ok(this.updateById(eType));
    }
}
