package cn.wise.sc.pay.business.eshopping.service.impl;

import cn.wise.sc.pay.business.eshopping.mapper.SpecificationMapper;
import cn.wise.sc.pay.business.eshopping.service.ISpecificationService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.Specification;
import cn.wise.sc.pay.domain.eshopping.model.query.SpecificationQuery;
import cn.wise.sc.pay.domain.eshopping.model.vo.SpecificationVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 商品规格 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-12-03
 */
@Service
public class SpecificationServiceImpl extends ServiceImpl<SpecificationMapper, Specification> implements ISpecificationService {


    @Autowired
    SpecificationMapper specificationMapper;

    @Override
    @Transactional
    public BaseResponse newSpecification(SpecificationQuery specificationQuery) {

        Specification newSpecification = Specification.builder()
                .goodsId(specificationQuery.getGoodsId())
                .name(specificationQuery.getName())
                .status(specificationQuery.getStatus())
                .descr(specificationQuery.getDescr())
                .price(specificationQuery.getPrice())
                .store(specificationQuery.getStore())
                .weight(specificationQuery.getWeight())
                .build();
        boolean isSalve = this.save(newSpecification);
        return isSalve ? BaseResponse.ok("添加成功") : BaseResponse.error("添加失败",Specification.class);
    }

    @Override
    public BaseResponse editSpecification(SpecificationQuery specificationQuery) {
        if (specificationQuery.getId() == 0) {
            return BaseResponse.error("参数错误",Specification.class);
        }
        Specification updateSpec = Specification.builder()
                .name(specificationQuery.getName())
                .status(specificationQuery.getStatus())
                .descr(specificationQuery.getDescr())
                .price(specificationQuery.getPrice())
                .store(specificationQuery.getStore())
                .weight(specificationQuery.getWeight())
                .build();
        UpdateWrapper uw = new UpdateWrapper();
        uw.eq("id", specificationQuery.getId());
        boolean isUpdate = this.update(updateSpec, uw);

        return isUpdate ? BaseResponse.ok("更新成功") : BaseResponse.error("更新失败",Specification.class);
    }

    @Override
    public BaseResponse delSpecification(long id) {

        if (id == 0) {
            return BaseResponse.error("参数错误",Specification.class);
        }
        boolean isRemove = this.removeById(id);

        return isRemove ? BaseResponse.ok("删除成功") : BaseResponse.error("删除失败",Specification.class);
    }

    @Override
    public BaseResponse getList(long goodsId) {

        if (goodsId == 0) {
            return BaseResponse.error("参数错误",Specification.class);
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("goods_id", goodsId);
        try {
            List<Specification> specs = this.list(qw);
            BaseResponse baseResponse = BaseResponse.ok("查询成功");
            baseResponse.setList(specs);
            return baseResponse;
        } catch (Exception ex) {
            return BaseResponse.error("查询失败",Specification.class);
        }
    }


    @Override
    public BaseResponse getGoodsSpc(int goodsId) {

        if (goodsId == 0) {
            return BaseResponse.error("参数有误",Specification.class);
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("goods_id",goodsId);
        List<Specification> specifications = this.list(wrapper);
        List<SpecificationVO> voList = new ArrayList<>();
        specifications.stream().forEach(arg->{
            voList.add(SpecificationVO.toVO(arg));
        });
        return BaseResponse.ok(voList);

    }
}
