package cn.wise.sc.pay.business.exhibition.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.exhibition.configure.FastDFSUtils;
import cn.wise.sc.pay.business.exhibition.mapper.ExhibitionMapper;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionGroundService;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionService;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionStoreService;
import cn.wise.sc.pay.business.exhibition.service.ITicketNoticeService;
import cn.wise.sc.pay.business.exhibition.service.ITicketService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.common.core.util.DateTimeUtils;
import cn.wise.sc.pay.domain.exhibition.bean.Exhibition;
import cn.wise.sc.pay.domain.exhibition.bean.ExhibitionGround;
import cn.wise.sc.pay.domain.exhibition.bean.ExhibitionStore;
import cn.wise.sc.pay.domain.exhibition.bean.Ticket;
import cn.wise.sc.pay.domain.exhibition.bean.TicketNotice;
import cn.wise.sc.pay.domain.exhibition.query.ExhibitionGroundQuery;
import cn.wise.sc.pay.domain.exhibition.query.ExhibitionQuery;
import cn.wise.sc.pay.domain.exhibition.query.TicketNoticeQuery;
import cn.wise.sc.pay.domain.exhibition.query.TicketQuery;
import cn.wise.sc.pay.domain.exhibition.vo.ExhibitionAPPVO;
import cn.wise.sc.pay.domain.exhibition.vo.ExhibitionManagerVO;
import cn.wise.sc.pay.domain.exhibition.vo.TicketVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 服务实现类
 *
 * @author qh
 * @since 2019-11-18
 */
@Service
@SuppressWarnings("all")
public class ExhibitionServiceImpl extends ServiceImpl<ExhibitionMapper, Exhibition>
        implements IExhibitionService {

    @Autowired
    ExhibitionMapper exhibitionMapper;
    @Autowired
    IExhibitionGroundService iExhibitionGroundService;
    @Autowired
    ITicketNoticeService iTicketNoticeService;
    @Autowired
    IExhibitionStoreService iExhibitionStoreService;
    @Autowired
    ITicketService iTicketService;
    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    @Transactional
    public BaseResponse addExhibition(ExhibitionQuery exhibitionQuery) {

        // 判断票的规格是否有效
        if (!vaildTickeQuery(exhibitionQuery.getTicketQueries())) {
            return BaseResponse.error("规格不符合", new Exhibition());
        }

        // 添加展览基本信息
        Exhibition exhibition =
                Exhibition.builder()
                        .type(exhibitionQuery.getType())
                        .address(exhibitionQuery.getAddress())
                        .advertisingType(exhibitionQuery.getAdvertisingType())
                        .totalComment(0)
                        .describe(exhibitionQuery.getDescribe())
                        .discountPrice(exhibitionQuery.getDiscounPrice())
                        .singleType(exhibitionQuery.getSingleType())
                        .title(exhibitionQuery.getTitle())
                        .tag(exhibitionQuery.getTag())
                        .wantWatch(0)
                        .refundRate(exhibitionQuery.getRefundRate())
                        .coverUrl(exhibitionQuery.getCover())
                        .advertisingUrl(exhibitionQuery.getAdvertisingCover())
                        .price(exhibitionQuery.getPrice())
                        .delFlag(0)
                        .createTime(System.currentTimeMillis())
                        .showType(0)
                        .build();
        List<ExhibitionGroundQuery> exhibitionGroundQueries =
                exhibitionQuery.getExhibitionGroundQueries();
        if (exhibitionGroundQueries == null || exhibitionGroundQueries.size() == 0) {
            return BaseResponse.error("请添加场次信息", new Exhibition());
        }
        ArrayList<Long> groundSet = new ArrayList<>();
        List<ExhibitionGround> exhibitionGroundsInsert =
                Lists.newArrayListWithCapacity(exhibitionGroundQueries.size());
        StringBuilder activeTime = new StringBuilder();
        for (ExhibitionGroundQuery exGround : exhibitionGroundQueries) {
            activeTime
                    .append(exGround.getStartTime())
                    .append("-")
                    .append(exGround.getEndTime())
                    .append("|");
            groundSet.add(exGround.getStartTime());
            groundSet.add(exGround.getEndTime());
        }
        // 将展览开始时间加入
        if (activeTime.toString().endsWith("|")) {
            activeTime.deleteCharAt(activeTime.length() - 1);
        }
        Collections.sort(groundSet);
        // 根据时间初始化 展览状态
        exhibition.setStatus(1);
        if (groundSet.size() < 2) {
            return BaseResponse.error("添加失败,场次消息有误!", Exhibition.class);
        }
        if (groundSet.get(0) > System.currentTimeMillis()) {
            exhibition.setStatus(2);
        }
        if (groundSet.get(groundSet.size() - 1) < System.currentTimeMillis()) {
            exhibition.setStatus(0);
        }

        //判断是否重复
        exhibition.setGround(activeTime.toString());
        QueryWrapper qw = new QueryWrapper();
        qw.eq("title", exhibition.getTitle());
        qw.eq("ground", exhibition.getGround());
        qw.eq("del_flag", 1);
        int count = this.count(qw);
        if (count > 0) {
            return BaseResponse.error("当前名字和场次的展览已存在!", this.getClass().getName());
        }

        this.save(exhibition);

        // 关联票和每张票的库存
        List<TicketQuery> ticketQueries = exhibitionQuery.getTicketQueries();
        List<ExhibitionStore> exhibitionStoresInsert =
                Lists.newArrayListWithCapacity(ticketQueries.size());
        if (ticketQueries == null || ticketQueries.size() == 0) {
            return BaseResponse.error("票务类型不能为空", Exhibition.class);
        }
        for (TicketQuery ticketQuery : ticketQueries) {
            Ticket ticket =
                    Ticket.builder() // /
                            .price(ticketQuery.getPrice())
                            .discountPrice(ticketQuery.getDiscountPrice())
                            .name(ticketQuery.getName())
                            .associationId(exhibition.getId()) // 关联展览
                            .type(ticketQuery.getType())
                            .activityType("EXHIBITION")
                            .build();
            iTicketService.save(ticket);
            // 为每个票型创建 库存
            ExhibitionStore exhibitionStore =
                    ExhibitionStore.builder()
                            .exhibitionId(exhibition.getId())
                            .ticketId(ticket.getId())
                            .remaining(ticketQuery.getTotalPublish())
                            .singleSold(ticketQuery.getSingleSold())
                            .totalPublish(ticketQuery.getTotalPublish())
                            .totalSold(0)
                            .chargeOff(0)
                            .version(0)
                            .build();
            exhibitionStoresInsert.add(exhibitionStore);
        }
        iExhibitionStoreService.saveBatch(exhibitionStoresInsert);

        // 构建票须知
        List<TicketNoticeQuery> tnQuerys = exhibitionQuery.getTicketNoticesQuerys();
        List<TicketNotice> ticketNotices = new ArrayList<>(tnQuerys.size());
        if (tnQuerys != null && tnQuerys.size() != 0) {
            for (TicketNoticeQuery tnQuery : tnQuerys) {
                TicketNotice ticketNotice =
                        TicketNotice.builder()
                                .exhibitionId(exhibition.getId())
                                .title(tnQuery.getTitle())
                                .describe(tnQuery.getDescribe())
                                .build();
                ticketNotices.add(ticketNotice);
            }
            iTicketNoticeService.saveBatch(ticketNotices);
        }

        if (StrUtil.isNotEmpty(exhibition.getAdvertisingUrl())) {
            exhibition.setAdvertisingUrl(fastDFSUtils.fullUrl(exhibition.getAdvertisingUrl()));
        }
        if (StrUtil.isNotEmpty(exhibition.getCoverUrl())) {
            exhibition.setCoverUrl(fastDFSUtils.fullUrl(exhibition.getCoverUrl()));
        }
        return BaseResponse.ok("添加展览成功", exhibition);
    }

    /**
     * 判断规格是否有效
     *
     * @param ticketQueries
     * @return
     */
    private boolean vaildTickeQuery(List<TicketQuery> ticketQueries) {

        int size = ticketQueries.size();
        if (size == 0) {
            return false;
        }

        // 判断名字是否重复
        int vaildName =
                ticketQueries.stream().map(arg -> arg.getName()).collect(Collectors.toSet()).size();
        if (vaildName != size) {
            return false;
        }

        return true;
    }

    @Override
    public BaseResponse editShowType(Long id, Integer type) {

        if (id.longValue() == 0 || type == null || type < 0 || type > 1) {
            return BaseResponse.error("参数错误", new Exhibition());
        }

        Exhibition exhibition = new Exhibition();
        exhibition.setId(id);
        exhibition.setShowType(type);
        boolean isSuccess = this.updateById(exhibition);
        if (isSuccess) {
            return BaseResponse.ok("编辑成功!", new Exhibition());
        } else {
            return BaseResponse.error("编辑失败", new Exhibition());
        }
    }

    @Override
    public BaseResponse getExhibitionList(String type, Integer showType, TimeBucket timeBucket) {

        QueryWrapper ew = new QueryWrapper();
        ew.eq("del_flag", 0);
        ew.ne("status", 0);

        if (StringUtils.isNotEmpty(type)) {
            ew.like("type", type);
        }
        if (showType != null) {
            ew.like("show_type", showType + "");
        }

        List<Exhibition> exhibitions = this.list(ew);
        List<ExhibitionAPPVO> exhibitionAPPVOS = new ArrayList<>(exhibitions.size());
        //
        if (CollectionUtils.isEmpty(exhibitions)) {
            BaseResponse baseResponse = BaseResponse.ok("查询成功", new ExhibitionAPPVO());
            baseResponse.setList(exhibitionAPPVOS);
            return baseResponse;
        }

        // 找票型
        List<Long> exIds = exhibitions.stream().map(arg -> arg.getId()).collect(Collectors.toList());
        QueryWrapper<Ticket> ticketQw = new QueryWrapper<>();
        ticketQw.in("association_id", exIds);
        List<Ticket> tickets = iTicketService.list(ticketQw);

        for (Exhibition exhibition : exhibitions) {
            if (StrUtil.isNotEmpty(exhibition.getAdvertisingUrl())) {
                exhibition.setAdvertisingUrl(fastDFSUtils.fullUrl(exhibition.getAdvertisingUrl()));
            }
            if (StrUtil.isNotEmpty(exhibition.getCoverUrl())) {
                exhibition.setCoverUrl(fastDFSUtils.fullUrl(exhibition.getCoverUrl()));
            }
            ExhibitionAPPVO exhibitionAppVO =
                    ExhibitionAPPVO.getExDetailAppVO(exhibition, tickets, new ArrayList<>());
            String ground = exhibition.getGround();
            List<Long> grounds = getStart(ground);
            long start = grounds.get(0);
            long end = grounds.get(grounds.size() - 1);
            // 预售
            if (start > System.currentTimeMillis()) {
                TicketVO ticketVO =
                        exhibitionAppVO.getTicketVOS().stream()
                                .filter(arg -> "预售单次票".equals(arg.getName()))
                                .findFirst()
                                .orElse(null);
                if (ticketVO == null) {
                    exhibitionAppVO.setPrice(
                            exhibitionAppVO.getTicketVOS().stream()
                                    .filter(arg -> "正式单次票".equals(arg.getName()))
                                    .findFirst()
                                    .get()
                                    .getPrice());
                } else {
                    exhibition.setPrice(ticketVO.getPrice());
                }
            }
            // 进行中
            if (start < System.currentTimeMillis() && System.currentTimeMillis() < end) {
                exhibitionAppVO.setStatus("进行中");
                exhibition.setPrice(
                        exhibitionAppVO.getTicketVOS().stream()
                                .filter(arg -> "正式单次票".equals(arg.getName()))
                                .findFirst()
                                .orElse(TicketVO.builder().price(-1D).build())
                                .getPrice());
                if (exhibition.getStatus() != 1) {
                    exhibition.setStatus(1);
                    this.updateById(exhibition);
                }
            }
            // 已结束时
            if (end < System.currentTimeMillis()) {
                exhibitionAppVO.setStatus("已结束");
                if (exhibition.getStatus() != 0) {
                    exhibition.setStatus(0);
                    this.updateById(exhibition);
                }
                continue;
            }

            //判断是否有时间条件
            if (DateTimeUtils.validTimeBucket(timeBucket)) {
                //开展时间大于条件时间
                if (start > timeBucket.getStartTime()) {
                    exhibitionAPPVOS.add(exhibitionAppVO);
                }
            } else {
                exhibitionAPPVOS.add(exhibitionAppVO);
            }
        }
        BaseResponse baseResponse = BaseResponse.ok("查询成功", new ExhibitionAPPVO());
        baseResponse.setList(exhibitionAPPVOS);
        return baseResponse;
    }

    @Override
    public BaseResponse getExhibitionPage(PageWrapperQuery pageWrapperQuery) {

        QueryWrapper ew = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);
        ew.orderByDesc("create_time");
        //
        ew.eq("del_flag", 0);
        Page page =
                new Page(
                        pageWrapperQuery.getPageQuery().getPageNo(),
                        pageWrapperQuery.getPageQuery().getPageSize());

        page = (Page) this.page(page, ew);

        List<Exhibition> exhibitions = page.getRecords();
        if (exhibitions.size() == 0) {
            return BaseResponse.okMsg("没找到相关数据", Exhibition.class);
        }

        // 找票型
        List<Long> exIds = exhibitions.stream().map(arg -> arg.getId()).collect(Collectors.toList());
        QueryWrapper<Ticket> ticketQw = new QueryWrapper<>();
        ticketQw.in("association_id", exIds);
        List<Ticket> tickets = iTicketService.list(ticketQw);

        // 找库存
        List<Long> ticketIds = tickets.stream().map(arg -> arg.getId()).collect(Collectors.toList());
        QueryWrapper<ExhibitionStore> storeQw = new QueryWrapper<>();
        storeQw.in("ticket_id", ticketIds);
        List<ExhibitionStore> storeList = iExhibitionStoreService.list(storeQw);

        List<ExhibitionManagerVO> exhibitionManagerVOS =
                Lists.newArrayListWithCapacity(exhibitions.size());
        for (Exhibition exhibition : exhibitions) {
            ExhibitionManagerVO exhibitionManagerVO =
                    new ExhibitionManagerVO(exhibition, tickets, storeList);
            if (StrUtil.isNotEmpty(exhibitionManagerVO.getCoverUrl())) {
                exhibitionManagerVO.setCoverUrl(fastDFSUtils.fullUrl(exhibition.getCoverUrl()));
            }
            exhibitionManagerVOS.add(exhibitionManagerVO);
        }

        page.setRecords(exhibitionManagerVOS);
        BaseResponse baseResponse = BaseResponse.ok("查询成功", new ExhibitionManagerVO());
        baseResponse.setPage(page);
        return baseResponse;
    }

    @Override
    public BaseResponse getExhibitionDetail(long id) {

        if (id == 0) {
            return BaseResponse.error("参数错误", new Exhibition());
        }

        Exhibition exhibition = this.getById(id);

        // 关联购票须知
        QueryWrapper ewTickNotice = new QueryWrapper();
        ewTickNotice.eq("exhibition_id", id);
        List<TicketNotice> ticketNotices = iTicketNoticeService.list(ewTickNotice);

        // 关联票
        QueryWrapper ewTicket = new QueryWrapper();
        ewTicket.eq("association_id", exhibition.getId());
        List<Ticket> tickets = iTicketService.list(ewTicket);

        if (StrUtil.isNotEmpty(exhibition.getAdvertisingUrl())) {
            exhibition.setAdvertisingUrl(fastDFSUtils.fullUrl(exhibition.getAdvertisingUrl()));
        }
        if (StrUtil.isNotEmpty(exhibition.getCoverUrl())) {
            exhibition.setCoverUrl(fastDFSUtils.fullUrl(exhibition.getCoverUrl()));
        }

        ExhibitionAPPVO exhibitionAppVO =
                ExhibitionAPPVO.getExDetailAppVO(exhibition, tickets, ticketNotices);
        String ground = exhibition.getGround();
        List<Long> grounds = getStart(ground);
        long start = grounds.get(0);
        long end = grounds.get(grounds.size() - 1);
        // 预售
        if (start > System.currentTimeMillis()) {
            exhibitionAppVO.setTicketVOS(
                    exhibitionAppVO.getTicketVOS().stream()
                            .filter(arg -> arg.getName().contains("预售"))
                            .collect(Collectors.toList()));
        }
        // 进行中
        if (start < System.currentTimeMillis() && System.currentTimeMillis() < end) {
            exhibitionAppVO.setTicketVOS(
                    exhibitionAppVO.getTicketVOS().stream()
                            .filter(arg -> arg.getName().contains("正式"))
                            .collect(Collectors.toList()));
        }

        //关联库存
        List<TicketVO> ticketVOS = exhibitionAppVO.getTicketVOS();
        for (TicketVO ticketVO : ticketVOS) {
            ExhibitionStore storeByTicketId = iExhibitionStoreService.getStoreByTicketId(ticketVO.getId());
            ticketVO.setRemaining(storeByTicketId.getRemaining());
        }

        return BaseResponse.ok(exhibitionAppVO);
    }

    @Override
    public BaseResponse getAdvertising() {
        QueryWrapper ew = new QueryWrapper();
        ew.eq("advertising_type", "1");
        ew.eq("del_flag", 0);
        try {

            List<Exhibition> list = this.list(ew);
            BaseResponse baseResponse = BaseResponse.ok(new Exhibition());

            list.stream()
                    .forEach(
                            ex -> {
                                if (StrUtil.isNotEmpty(ex.getAdvertisingUrl())) {
                                    ex.setAdvertisingUrl(fastDFSUtils.fullUrl(ex.getAdvertisingUrl()));
                                }
                                if (StrUtil.isNotEmpty(ex.getCoverUrl())) {
                                    ex.setCoverUrl(fastDFSUtils.fullUrl(ex.getCoverUrl()));
                                }
                            });

            baseResponse.setList(list);
            return baseResponse;

        } catch (Exception ex) {
            return BaseResponse.error("获取广告位失败", new Exhibition());
        }
    }

    @Override
    public BaseResponse delById(Integer id) {

        if (id == null) {
            return BaseResponse.error("id为空", "");
        }
        UpdateWrapper uw = new UpdateWrapper<>().eq("id", id).set("del_flag", 1);
        try {
            boolean update = this.update(uw);
            return update
                    ? BaseResponse.ok("删除成功")
                    : BaseResponse.error("删除失败", this.getClass().getName());
        } catch (Exception ex) {
            return BaseResponse.error("更新失败", "");
        }
    }

    @Override
    @Transactional
    public BaseResponse updateExhibition(ExhibitionQuery exhibitionQuery) {

        // 添加展览基本信息
        Exhibition exhibition =
                Exhibition.builder()
                        .id(exhibitionQuery.getId())
                        .type(exhibitionQuery.getType())
                        .address(exhibitionQuery.getAddress())
                        .advertisingType(exhibitionQuery.getAdvertisingType())
                        .totalComment(0)
                        .describe(exhibitionQuery.getDescribe())
                        .discountPrice(exhibitionQuery.getDiscounPrice())
                        .singleType(exhibitionQuery.getSingleType())
                        .title(exhibitionQuery.getTitle())
                        .tag(exhibitionQuery.getTag())
                        .wantWatch(0)
                        .refundRate(exhibitionQuery.getRefundRate())
                        .coverUrl(exhibitionQuery.getCover())
                        .advertisingUrl(exhibitionQuery.getAdvertisingCover())
                        .price(exhibitionQuery.getPrice())
                        .build();

        boolean isSuccess = this.updateById(exhibition);

        // 关联场次信息
        List<ExhibitionGroundQuery> exhibitionGroundQueries =
                exhibitionQuery.getExhibitionGroundQueries();
        if (exhibitionGroundQueries.size() == 0) {
            return BaseResponse.error("请天假场次信息", new Exhibition());
        }
        List<ExhibitionGround> exhibitionGroundsInsert =
                Lists.newArrayListWithCapacity(exhibitionGroundQueries.size());
        StringBuilder activeTime = new StringBuilder();
        for (ExhibitionGroundQuery exGround : exhibitionGroundQueries) {
            activeTime
                    .append(exGround.getStartTime())
                    .append("-")
                    .append(exGround.getEndTime())
                    .append("|");
        }
        // 将展览开始时间加入
        if (activeTime.toString().endsWith("|")) {
            activeTime.deleteCharAt(activeTime.length() - 1);
        }
        exhibition.setGround(activeTime.toString());

        // 关联购票须知
        List<TicketNoticeQuery> tnQuerys = exhibitionQuery.getTicketNoticesQuerys();
        List<TicketNotice> ticketNotices = Lists.newArrayListWithCapacity(tnQuerys.size());
        List<TicketNotice> insertNotices = Lists.newArrayListWithCapacity(tnQuerys.size());
        for (TicketNoticeQuery tnQuery : tnQuerys) {
            TicketNotice ticketNotice =
                    TicketNotice.builder()
                            .id(tnQuery.getId())
                            .exhibitionId(exhibition.getId())
                            .title(tnQuery.getTitle())
                            .describe(tnQuery.getDescribe())
                            .build();
            if (ticketNotice.getId() != null && ticketNotice.getId() != 0L) {
                ticketNotices.add(ticketNotice);
            } else {
                insertNotices.add(ticketNotice);
            }
        }
        if (ticketNotices.size() != 0) {
            iTicketNoticeService.updateBatchById(ticketNotices);
        }
        if (insertNotices.size() != 0) {
            iTicketNoticeService.saveBatch(insertNotices);
        }

        // 更新票据基本信息
        List<TicketQuery> ticketQueries = exhibitionQuery.getTicketQueries();
        List<Ticket> tickets = Lists.newArrayListWithCapacity(ticketQueries.size());
        for (TicketQuery ticketQuery : ticketQueries) {
            Ticket ticket =
                    Ticket.builder()
                            .id(ticketQuery.getId())
                            .name(ticketQuery.getName())
                            .price(ticketQuery.getPrice())
                            .build();
            tickets.add(ticket);
        }
        iTicketService.updateBatchById(tickets);
        if (StrUtil.isNotEmpty(exhibition.getAdvertisingUrl())) {
            exhibition.setAdvertisingUrl(fastDFSUtils.fullUrl(exhibition.getAdvertisingUrl()));
        }
        if (StrUtil.isNotEmpty(exhibition.getCoverUrl())) {
            exhibition.setCoverUrl(fastDFSUtils.fullUrl(exhibition.getCoverUrl()));
        }
        return BaseResponse.ok("添加展览编辑成功", exhibition);
    }

    @Override
    public BaseResponse getExhibitionStore(Long id) {

        if (id == null || id == 0) {
            return BaseResponse.error("id必传!", Exhibition.class);
        }

        QueryWrapper qw = new QueryWrapper();
        qw.eq("association_id", id);

        List<Ticket> list = iTicketService.list(qw);
        List<TicketVO> ticketVOS = new ArrayList<>(list.size());
        for (Ticket ticket : list) {
            // 先同步吧
            iExhibitionStoreService.saveDB(ticket.getId());
            // 拿库存
            ExhibitionStore exhibitionStore = iExhibitionStoreService.getStoreByTicketId(ticket.getId());
            QueryWrapper sw = new QueryWrapper();
            sw.eq("ticket_id", ticket.getId());
            // 拿基础字段
            ExhibitionStore dbStore = iExhibitionStoreService.getOne(sw);
            TicketVO ticketVO = new TicketVO();
            ticketVO.setName(ticket.getName());
            ticketVO.setPrice(ticket.getPrice());
            ticketVO.setId(ticket.getId());
            ticketVO.setSingleSold(dbStore.getSingleSold());
            ticketVO.setTotalSold(dbStore.getTotalPublish() - exhibitionStore.getRemaining());
            ticketVO.setTotalPublish(dbStore.getTotalPublish());
            ticketVO.setRemaining(exhibitionStore.getRemaining());
            ticketVOS.add(ticketVO);
        }
        return BaseResponse.ok(ticketVOS);
    }

    @Override
    @Transactional
    public BaseResponse updateStore(List<TicketVO> ticketVOS) {

        List<Long> ticketIds = ticketVOS.stream().map(arg -> arg.getId()).collect(Collectors.toList());
        QueryWrapper qw = new QueryWrapper();
        if (ticketIds.size() != ticketVOS.size()) {
            return BaseResponse.error("更新失败!id为必传参数!", Exhibition.class);
        }
        qw.in("ticket_id", ticketIds);
        // 更新基本字段
        List<ExhibitionStore> list = iExhibitionStoreService.list(qw);
        for (ExhibitionStore store : list) {
            boolean isAdd = false;
            TicketVO ticketVO =
                    ticketVOS.stream()
                            .filter(arg -> arg.getId().longValue() == store.getTicketId())
                            .findFirst()
                            .orElse(null);
            if (ticketVO != null) {
                store.setSingleSold(ticketVO.getSingleSold());
                ExhibitionStore redisStore = iExhibitionStoreService.getStoreByTicketId(ticketVO.getId());
                // 计算库存增量
                BaseResponse response = response = BaseResponse.ok();
                // 更新redis库存
                if (ticketVO.getTotalPublish() > store.getTotalPublish()) {
                    isAdd = true;
                    response =
                            iExhibitionStoreService.addStore(
                                    store.getTicketId(), ticketVO.getTotalPublish() - store.getTotalPublish());
                    iExhibitionStoreService.saveDB(ticketVO.getId());
                }
                if (ticketVO.getTotalPublish() < store.getTotalPublish()) {
                    response =
                            iExhibitionStoreService.decreaseStore(
                                    store.getTicketId(), store.getTotalPublish() - ticketVO.getTotalPublish());
                    iExhibitionStoreService.saveDB(ticketVO.getId());
                }
                store.setTotalPublish(ticketVO.getTotalPublish());
                // redis成功
                if (response.getCode() == 200) {
                    boolean success = iExhibitionStoreService.updateById(store);
                    // 数据库no成功 回退redis
                    if (!success) {
                        if (isAdd) {
                            iExhibitionStoreService.decreaseStore(
                                    store.getTicketId(), store.getTotalPublish() - ticketVO.getTotalPublish());
                        } else {
                            iExhibitionStoreService.addStore(
                                    store.getTicketId(), ticketVO.getTotalPublish() - store.getTotalPublish());
                        }
                        return BaseResponse.error("库存更新失败!", ExhibitionStore.class);
                    }
                }
            }
        }

        return BaseResponse.ok();
    }

    /**
     * 先看人数加+1 数据允许容错，不需要严格并发
     *
     * @param id
     * @return
     */
    public void putWantWatch(int id) {
        if (id == 0) {
            return;
        }
        exhibitionMapper.putWantWatch(id);
    }

    /**
     * 获取展开开放时间段
     *
     * @param ground
     * @return
     */
    private static List<Long> getStart(String ground) {
        String[] str = ground.split("-");
        List<Long> sort = new ArrayList<>();
        for (int i = 0; i < str.length; i++) {
            String tmp = str[i];
            String[] tmp2 = tmp.split("\\|");
            if (tmp2.length > 1) {
                for (int j = 0; j < tmp2.length; j++) {
                    if (StrUtil.isNotEmpty(tmp2[j])) {
                        sort.add(Long.parseLong(tmp2[j]));
                    }
                }
            } else {
                sort.add(Long.parseLong(tmp));
            }
        }
        Collections.sort(sort);
        return sort;
    }
}
