package cn.wise.sc.pay.api.merchant.service;

import cn.wise.sc.pay.api.merchant.service.hystrix.MerchantCommentImpl;
import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * @description: 店铺点评
 * @author: qh
 * @create: 2020-06-15 14:51
 **/
@FeignClient(value = "saas-v1-merchant",path = "/merchant/comment",fallback = MerchantCommentImpl.class)
public interface IMerchantCommentService {
    /**
     //     * 添加评论
     //     * @param comment
     //     * @return
     //     */
    @PostMapping("/addComment")
    BaseResponse addComment(@RequestBody Comment comment);

    /**
     * 删除评论，多个评论id用英文逗号 ','隔开
     * @param idStr
     * @return
     */
    @DeleteMapping("/deleteComment")
    BaseResponse deleteComment(@RequestBody String idStr);

    /**
     * 更新评论点赞数
     * @param id
     * @return
     */
    @PutMapping("/updatePraiseNum/{id}")
    BaseResponse updateCommentPraise(@PathVariable("id") Long id);

    /**
     * 评论列表分页查询
     * @param pageQuery
     * @return
     */
    @GetMapping("/selectComment")
    BaseResponse getCommentList(@SpringQueryMap PageQuery pageQuery);

    /**
     * 根据评论目标id分页查询
     * @param targetId
     * @param pageQuery
     * @return
     */
    @GetMapping("/commentPage")
    BaseResponse<Page<Comment>> getCommentsByTargetId(@RequestParam("targetId") Long targetId,
                                                      @SpringQueryMap PageQuery pageQuery);

}
