package cn.wise.sc.pay.business.merchant.controller;

import cn.wise.sc.pay.business.merchant.configure.FastDFSUtils;
import cn.wise.sc.pay.business.merchant.service.IBannerMenuService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.merchant.bean.BannerMenu;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author lqk
 * @since 2019/10/08 09:09
 */
@RestController
@CrossOrigin
@RequestMapping("merchant/menu/")
@Slf4j
public class BannerMenuController {

    @Autowired
    IBannerMenuService bannerMenuService;

    @Autowired
    FastDFSUtils fastDFSUtils;

    @PostMapping("addOrUpdateMenu")
    @ApiOperation("添加或更新导航栏")
    public ResponseEntity<BannerMenu> uploadPic(@RequestBody BannerMenu menu) {
        if(menu == null || StringUtils.isBlank(menu.getName())){
            return ResponseEntity.badRequest().body(menu);
        }
        return bannerMenuService.addOrUpdateBannerMenu(menu);
    }

    @PutMapping("orderMenu")
    @ApiOperation("排序")
    @ApiImplicitParams({
        @ApiImplicitParam( name = "type", value = "上移或下移(0上移，1下移)", paramType = "path", dataType = "Integer"),
        @ApiImplicitParam( name = "id", value = "当前菜单id", paramType = "path", dataType = "Integer"),
    })
    public BaseResponse orderMenu(Integer type, Integer id) {
        return bannerMenuService.orderMenu(type,id);
    }

    @GetMapping("getMenus")
    public BaseResponse getMenus() {
        return BaseResponse.ok(bannerMenuService.getBannerMenus());
    }

    @GetMapping("getPageMenus")
    public BaseResponse getPageMenus(PageQuery pageQuery) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("del_flag", 0);
        wrapper.orderByAsc("order_num");
        Page page = new Page<>(pageQuery.getPageNo(),pageQuery.getPageSize());
        page = (Page) bannerMenuService.page(page,wrapper);
        List<BannerMenu> list = page.getRecords();
        list.stream().forEach(arg->{
            arg.setMenuIcon(fastDFSUtils.fullUrl(arg.getMenuIcon()));
        });
        return BaseResponse.ok(page);
    }

    @DeleteMapping("deleteBannerImage")
    public BaseResponse deletePic(Integer id){
        return bannerMenuService.deleteBannerMenu(id);
    }
}
