package cn.wise.sc.pay.business.merchant.controller;

import cn.wise.sc.pay.business.merchant.service.IDeviceService;
import cn.wise.sc.pay.business.merchant.service.ITypeService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.merchant.bean.Device;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author lqk
 * @since 2019/10/08 09:09
 */
@RestController
@CrossOrigin
@RequestMapping("/merchant/device/")
@Slf4j
public class DeviceController {

    @Autowired
    IDeviceService deviceService;

    @Autowired
    ITypeService typeService;

    @GetMapping("typeName")
    public BaseResponse getTypeList(String typeName) {
        QueryWrapper wrapper = new QueryWrapper();
        if(StringUtils.isNotBlank(typeName)){
            wrapper.like("name", typeName);
        }
        return BaseResponse.ok(typeService.list(wrapper));
    }

    @PostMapping("addOrUpdateDevice")
    public BaseResponse addDevice(@RequestBody Device device) {
        if (StringUtils.isBlank(device.getDeviceCode()) || StringUtils.isBlank(device.getMerchName())
                || StringUtils.isBlank(device.getTypeName())) {
            return BaseResponse.error("店铺名或设备编号为空",Device.class);
        }
        return deviceService.insertDevice(device);
    }

    @GetMapping("{merchId}")
    public BaseResponse getMerchDevices(@PathVariable("merchId") Long merchId) {
        if (merchId == null) {
            return BaseResponse.error("merchId为空", Merchant.class.getName());
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("merch_id", merchId);
        List<Device> result = deviceService.list(wrapper);
        return BaseResponse.ok(result);
    }

    @DeleteMapping("delete/{id}")
    public BaseResponse deleteDevice(@PathVariable("id") Long id) {
        if (id == null) {
            return BaseResponse.error("请求失败，参数为空",Device.class);
        }
        boolean result = deviceService.removeById(id);
        return result ? BaseResponse.ok(true) : BaseResponse.error("请求失败",Device.class);
    }

    @DeleteMapping("page")
    public BaseResponse devicePage(Long merchId, Integer typeId, PageQuery pageQuery) {
        if (pageQuery == null) {
            return BaseResponse.error("参数错误",Device.class);
        }
        Page<Device> page = new Page();
        page.setCurrent(pageQuery.getPageNo());
        page.setSize(pageQuery.getPageSize());
        QueryWrapper wrapper = new QueryWrapper();
        if (merchId != null) {
            wrapper.eq("merch_id", merchId);
        }
        if (typeId != null) {
            wrapper.eq("type_id", typeId);
        }

        page = (Page<Device>) deviceService.page(page, wrapper);
        return BaseResponse.ok(page);
    }
}
