package cn.wise.sc.pay.business.merchant.service.impl;

import cn.wise.sc.pay.business.merchant.configure.FastDFSUtils;
import cn.wise.sc.pay.business.merchant.mapper.BannerImageMapper;
import cn.wise.sc.pay.business.merchant.service.IBannerImageService;
import cn.wise.sc.pay.business.merchant.service.IMerchantService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.BannerImage;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.merchant.query.BannerImageQuery;
import cn.wise.sc.pay.domain.merchant.vo.MerchantBannersVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @ClassName BankPermitServiceImpl
 * @Author Administrator
 * @Date 2019/12/18
 * @Version V1.0
 **/
@Service
public class BannerImageServiceImpl extends ServiceImpl<BannerImageMapper, BannerImage> implements IBannerImageService {

    @Autowired
    FastDFSUtils fastDFSUtils;

    @Autowired
    IMerchantService merchantService;

    @Override
    public BaseResponse addBannerImage(MultipartFile file, Integer type) {
        String path = "";
        if (file != null && StringUtils.isNotEmpty(file.getOriginalFilename())) {
            try {
                path = fastDFSUtils.uploadFile(file).getPath();
            } catch (IOException e) {
                return BaseResponse.error("图片上传失败", BannerImage.class);
            }
        }
        String field = type + "";
        BannerImage bImage = BannerImage.builder().imgUrl(path)
                .field(field)
                .createTime(System.currentTimeMillis())
                .updateTime(System.currentTimeMillis())
                .build();
        this.save(bImage);
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("img_url", path);
        wrapper.eq("field", field);
        wrapper.last("limit 1");
        BannerImage bannerImage = this.getOne(wrapper);
        bannerImage.setImgUrl(fastDFSUtils.fullUrl(bannerImage.getImgUrl()));
        return BaseResponse.ok(bannerImage);
    }

    @Override
    public List<BannerImage> getBannerImages(String field) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("field", field);
        List<BannerImage> list = this.list(wrapper);
        list.stream().forEach(arg -> {
            arg.setImgUrl(fastDFSUtils.fullUrl(arg.getImgUrl()));
        });
        return list;
    }

    @Override
    public BaseResponse deleteBannerImage(Integer id) {
        BannerImage byId = this.getById(id);
        boolean b = this.removeById(id);
        return b ? BaseResponse.ok("删除成功") : BaseResponse.error("删除失败", BannerImage.class);
    }

    @Override
    /**
     * 添加收银端广告图
     */
    public BaseResponse<BannerImage> addCashierAdvert(BannerImageQuery bannerImageQuery, Integer type) {
        if (StringUtils.isBlank(bannerImageQuery.getImagePath())) {
            return BaseResponse.error("图片路径不能为空", this.getClass().getName());
        }
        if (bannerImageQuery.getMerchantId() == null || bannerImageQuery.getMerchantId() <= 0) {
            return BaseResponse.error("商户id号不能为空", this.getClass().getName());
        }
        String field = type + "";
        BannerImage bImage = BannerImage.builder().imgUrl(bannerImageQuery.getImagePath())
                .field(field)
                .createTime(System.currentTimeMillis())
                .updateTime(System.currentTimeMillis())
                .merchantId(bannerImageQuery.getMerchantId())
                .build();
        this.save(bImage);
        //
        bImage.setImgUrl(fastDFSUtils.fullUrl(bImage.getImgUrl()));
        return BaseResponse.ok(bImage);
    }

    @Override
    /**
     * 收银端根据商家id获取商家广告图
     */
    public BaseResponse getCashierAdvert(Long merchantId, Integer type) {
        if (merchantId == null || merchantId <= 0) {
            return BaseResponse.error("商户id号不能为空", this.getClass().getName());
        }
        QueryWrapper<BannerImage> wrapper = new QueryWrapper();
        if (type > -1) {
            String field = type + "";
            wrapper.eq("field", field);
        }
        wrapper.eq("merchant_id", merchantId);
        List<BannerImage> list = this.list(wrapper);
        list.stream().forEach(arg -> {
            arg.setImgUrl(fastDFSUtils.fullUrl(arg.getImgUrl()));
        });
        return BaseResponse.ok(list, this.getClass());
    }

    @Override
    public BaseResponse listBannerByMerchants(Integer pageNo, Integer pageSize, String name) {
        if (pageNo == null || pageSize == null || pageNo <= 0 || pageSize <= 0) {
            return BaseResponse.error("参数不正确", this.getClass().getName());
        }
        BaseResponse<IPage<Merchant>> listMerchant = merchantService.listMerchant(pageNo, pageSize, name);
        if (listMerchant.getCode() == 400) {
            return BaseResponse.error("分页店铺失败", this.getClass().getName());
        }
        //分页店铺对象
        IPage<Merchant> merchantIPage = listMerchant.getData();
        //店铺列表
        List<Merchant> merchants = merchantIPage.getRecords();
//店铺对应的广告图列表
        List<BannerImage> imageList = null;
        if (!CollectionUtils.isEmpty(merchants)) {
            Set<Long> merchantIds = merchants.stream().map(ms -> ms.getId()).collect(Collectors.toSet());
            QueryWrapper<BannerImage> queryWrapper = new QueryWrapper<>();
            //收银端广告位
            queryWrapper.eq("field", "1");
            queryWrapper.in("merchant_id", merchantIds);
            imageList = this.list(queryWrapper);
        }
        List<MerchantBannersVo> merchantBannersVos = new ArrayList<>();

        List<BannerImage> finalImageList = imageList;
        merchants.stream().forEach(ms -> {
            MerchantBannersVo bannersVo = MerchantBannersVo.builder()
                    .id(ms.getId())
                    .manageStatus(ms.getManageStatus())
                    .merchName(ms.getMerchName())
                    .build();
            //广告图
            List<BannerImage> bannerImages = new ArrayList<>();
            if (!CollectionUtils.isEmpty(finalImageList)) {
                finalImageList.stream().forEach(il -> {
                    if (ms.getId().equals(il.getMerchantId())) {
                        il.setImgUrl(fastDFSUtils.fullUrl(il.getImgUrl()));
                        bannerImages.add(il);
                    }
                });
            }
            bannersVo.setBannerImages(bannerImages);
            merchantBannersVos.add(bannersVo);
        });
        IPage<MerchantBannersVo> voIPage = new Page<>();
        voIPage.setTotal(merchantIPage.getTotal());
        voIPage.setSize(merchantIPage.getSize());
        voIPage.setPages(merchantIPage.getPages());
        voIPage.setCurrent(merchantIPage.getCurrent());
        voIPage.setRecords(merchantBannersVos);
        return BaseResponse.ok(voIPage, this.getClass());
    }
}
