package cn.wise.sc.pay.business.merchant.service.impl;

import cn.wise.sc.pay.business.merchant.mapper.CategoryMapper;
import cn.wise.sc.pay.business.merchant.service.ICategoryService;
import cn.wise.sc.pay.business.merchant.service.IMerchCategoryService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.Category;
import cn.wise.sc.pay.domain.merchant.query.CategoryQuery;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName BankPermitServiceImpl
 * @Author Administrator
 * @Date 2019/12/18
 * @Version V1.0
 **/
@Service
public class CategoryServiceImpl extends ServiceImpl<CategoryMapper, Category> implements ICategoryService {

    @Autowired
    IMerchCategoryService merchCategoryService;

    @Override
    public BaseResponse insertCategory(CategoryQuery categoryQuery) {
        Category category = Category.builder()
                .name(categoryQuery.getName())
                .parentId(categoryQuery.getParentId())
                .rank(categoryQuery.getRank())
                .createTime(System.currentTimeMillis())
                .updateTime(System.currentTimeMillis())
                .build();
        boolean result = this.save(category);
        return result ? BaseResponse.ok("添加分类成功") : BaseResponse.error("添加分类失败",Category.class);
    }

    @Override
    public BaseResponse updateCategory(Category category) {
        Category resCategory = this.getById(category.getId());
        category.setCreateTime(resCategory.getCreateTime());
        category.setUpdateTime(System.currentTimeMillis());
        boolean result = this.updateById(category);
        return result ? BaseResponse.ok("修改分类成功") : BaseResponse.error("修改分类失败",Category.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse deleteById(Long id) {
        boolean result = this.removeById(id);
        if(result){
            UpdateWrapper wrapper = new UpdateWrapper();
            wrapper.eq("category_id",id);
            merchCategoryService.remove(wrapper);
        }
        return result ? BaseResponse.ok("删除分类成功") : BaseResponse.error("删除分类失败",Category.class);
    }

    @Override
    public BaseResponse getCategorys() {
        List<Category> result = this.list();
        List<Category> res = new ArrayList<>();
        for (Category c : result) {
            for (Category category : result ){
                if(c.getId().longValue() == category.getParentId().longValue()){
                    if(c.getCategories() == null){
                        c.setCategories(new ArrayList<>());
                    }
                    c.getCategories().add(category);
                }
            }
        }
        for (Category category : result) {
            if(category.getParentId() == 0){
                res.add(category);
            }
        }
        return BaseResponse.ok(res);
    }
}
