package cn.wise.sc.pay.business.merchant.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.merchant.mapper.MerchDiscountMapper;
import cn.wise.sc.pay.business.merchant.service.IMerchDiscountService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.MerchDiscount;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import java.util.List;
/**
 * @description:
 * @author: qh
 * @create: 2020-09-07 09:41
 **/
@Service
public class MerchDiscountServiceImpl extends ServiceImpl<MerchDiscountMapper, MerchDiscount>
        implements IMerchDiscountService {

    @Override
    public BaseResponse<MerchDiscount> getBySellerId(Long seller) {

        if (seller == null || seller == 0L) {
            return BaseResponse.error("seller不能为空!", MerchDiscount.class.getName());
        }

        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("seller_id", seller);

        return BaseResponse.ok(this.getOne(queryWrapper));
    }

    @Override
    public BaseResponse<Boolean> updateMerchDiscountBySeller(MerchDiscount merchDiscount) {

        //判断折扣信息是否有效
        String discountStr = merchDiscount.getDiscountStr();
        if (StrUtil.isEmpty(discountStr)) {
            return BaseResponse.error("折扣信息不能为空!", MerchDiscount.class.getName());
        }
        if (merchDiscount.getId() == null || merchDiscount.getId() <= 0 ||
                merchDiscount.getSellerId() == null || merchDiscount.getSellerId() == 0) {
            return BaseResponse.error("折扣信息不能为空!", MerchDiscount.class.getName());
        }
        try {
            List<MerchDiscount.DiscountSetting> discountSettings =
                    JSON.parseArray(discountStr, MerchDiscount.DiscountSetting.class);
        } catch (Exception ex) {
            return BaseResponse.error("折扣信息配置格式不正确!",
                    MerchDiscount.class.getName());
        }

        boolean flag = this.updateById(merchDiscount);

        if (flag) {
            return BaseResponse.ok(true);
        } else {
            return BaseResponse.error("更新失败", Merchant.class.getName());
        }
    }

    @Override
    public BaseResponse<MerchDiscount> addNewMerchDiscount(MerchDiscount merchDiscount) {

        //判断折扣信息是否有效
        String discountStr = merchDiscount.getDiscountStr();
        if (StrUtil.isEmpty(discountStr)) {
            return BaseResponse.error("折扣信息不能为空!", MerchDiscount.class.getName());
        }
        if (merchDiscount.getSellerId() == null || merchDiscount.getSellerId() == 0) {
            return BaseResponse.error("折扣信息不能为空!", MerchDiscount.class.getName());
        }
        try {
            List<MerchDiscount.DiscountSetting> discountSettings =
                    JSON.parseArray(discountStr, MerchDiscount.DiscountSetting.class);
        } catch (Exception ex) {
            return BaseResponse.error("折扣信息配置格式不正确!",
                    MerchDiscount.class.getName());
        }
        //添加新的折扣信息
        boolean save = this.save(merchDiscount);
        if (save){
            return BaseResponse.ok(merchDiscount);
        }else {
            return BaseResponse.error("添加失败!",MerchDiscount.class.getName());
        }

    }
}
