package cn.wise.sc.pay.business.merchant.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.merchant.mapper.StoreMapper;
import cn.wise.sc.pay.business.merchant.service.IMerchCategoryService;
import cn.wise.sc.pay.business.merchant.service.IMerchLabelService;
import cn.wise.sc.pay.business.merchant.service.IStoreExtraService;
import cn.wise.sc.pay.business.merchant.service.IStoreService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.MerchCategory;
import cn.wise.sc.pay.domain.merchant.bean.MerchLabel;
import cn.wise.sc.pay.domain.merchant.bean.Store;
import cn.wise.sc.pay.domain.merchant.bean.StoreExtra;
import cn.wise.sc.pay.domain.merchant.query.StoreQuery;
import cn.wise.sc.pay.domain.merchant.vo.StoreVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 店铺业务类
 */
@Service
public class StoreServiceImpl extends ServiceImpl<StoreMapper, Store> implements IStoreService {

    @Autowired
    private IMerchLabelService merchLabelService;
    @Autowired
    private IMerchCategoryService merchCategoryService;
    @Autowired
    private IStoreExtraService iStoreExtraService;
    @Autowired
    private StoreMapper storeMapper;

    @Override
    @Transactional
    public BaseResponse addStore(StoreQuery storeQuery) {

        Store store = Store.builder()
                .name(storeQuery.getName())
                .address(storeQuery.getAddress())
                .merchId(storeQuery.getMerchId())
                .area(storeQuery.getArea())
                .commissionRate(storeQuery.getCommissionRate())
                .longitude(storeQuery.getLongitude())
                .latitude(storeQuery.getLatitude())
                .details(storeQuery.getDescribe())
                .build();

        if (storeQuery.getManageStatus() == null) {
            store.setManageStatus(0);
        } else {
            store.setManageStatus(storeQuery.getManageStatus());
        }

        boolean save = this.save(store);



        return save ? BaseResponse.ok("店铺添加成功") : BaseResponse.error("店铺添加失败", Store.class);
    }


    @Override
    public BaseResponse pageStore(Integer pageNo, Integer pageSize) {

        Page<Store> storePage = new Page<>(pageNo, pageSize);

        IPage<Store> storeIPage = this.baseMapper.selectPage(storePage, null);

        return storeIPage == null ? BaseResponse.error("查询失败", Store.class) : BaseResponse.ok(storeIPage);
    }

    @Transactional
    @Override
    public BaseResponse updateStore(StoreQuery storeQuery) {

        Store store = Store.builder()
                .name(storeQuery.getName())
                .area(storeQuery.getArea())
                .address(storeQuery.getAddress())
                .commissionRate(storeQuery.getCommissionRate())
                .longitude(storeQuery.getLongitude())
                .latitude(storeQuery.getLatitude())
                .details(storeQuery.getDescribe())
                .build();

        boolean update = this.update(store, null);

        return update ? BaseResponse.ok("更新店铺成功") : BaseResponse.error("更新店铺失败", Store.class);
    }

    @Override
    @Transactional
    public BaseResponse businessCategory(String category, String label, Long storeId) {
        List<Long> categoryList = new ArrayList<>();
        List<Long> labelList = new ArrayList<>();

        if (category.contains("/")) {
            String[] categories = category.split("/");
            for (String s : categories) {
                categoryList.add(Long.parseLong(s));
            }
        } else {
            categoryList.add(Long.parseLong(category));
        }

        if (label.contains(",")) {
            String[] labels = label.split(",");
            for (String s : labels) {
                labelList.add(Long.parseLong(s));
            }
        } else {
            labelList.add(Long.parseLong(label));
        }

        List<MerchLabel> merchLabels = new ArrayList<>();
        List<MerchCategory> MerchCategorys = new ArrayList<>();

        for (Long aLong : labelList) {
            MerchLabel merchLabel = new MerchLabel();
            merchLabel.setMerchId(storeId);
            merchLabel.setCreateTime(System.currentTimeMillis());
            merchLabel.setLabelId(aLong);
            merchLabels.add(merchLabel);
        }
        for (Long aLong : categoryList) {
            MerchCategory merchCategory = new MerchCategory();
            merchCategory.setMerchId(storeId);
            merchCategory.setCategoryId(aLong);
            merchCategory.setCreateTime(System.currentTimeMillis());
            MerchCategorys.add(merchCategory);
        }

        boolean updateBatch = merchLabelService.saveOrUpdateBatch(merchLabels);
        boolean updateBatch1 = merchCategoryService.saveOrUpdateBatch(MerchCategorys);
        if (updateBatch && updateBatch1) {
            return BaseResponse.ok("标签分类设置成功");
        }
        return BaseResponse.error("设置分类或标签失败", MerchLabel.class);
    }

    @Override
    public BaseResponse<StoreVo> getStoreInfo(Long id) {

        if (id == 0) {
            return BaseResponse.error("店铺id是必须的!", StoreVo.class.getName());
        }

        QueryWrapper qw = new QueryWrapper();
        qw.eq("id", id);
        Store store = this.getOne(qw);
        assert store != null;
        QueryWrapper qw2 = new QueryWrapper();
        qw2.eq("store_id", id);
        StoreExtra storeExtra = iStoreExtraService.getOne(qw2);
        assert storeExtra != null;
        StoreVo storeVo = new StoreVo();
        BeanUtil.copyProperties(store, storeVo);
        BeanUtil.copyProperties(storeExtra, storeVo);

        return BaseResponse.ok(storeVo);
    }

}
