package cn.wise.sc.pay.business.order.controller;

import cn.wise.sc.pay.business.order.ali.AliPayApiConfig;
import cn.wise.sc.pay.business.order.ali.AliPayApiConfigKit;
import cn.wise.sc.pay.business.order.model.AliPayBean;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.parser.Feature;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.internal.util.AlipayEncrypt;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

@RestController
@CrossOrigin
@RequestMapping("/alipay")
public class AliController extends AbstractAliPayApiController{

    private static final Logger log = LoggerFactory.getLogger(AliController.class);


    @Autowired
    private AliPayBean aliPayBean;

    @Override
    public AliPayApiConfig getApiConfig() {
        AliPayApiConfig aliPayApiConfig;
        try {
            AliPayApiConfigKit.setThreadLocalAppId(aliPayBean.getAppId());
            aliPayApiConfig = AliPayApiConfigKit.getApiConfig(aliPayBean.getAppId());
        } catch (Exception e) {
            aliPayApiConfig = AliPayApiConfig.builder()
                    .setAppId(aliPayBean.getAppId())
                    .setAliPayPublicKey(aliPayBean.getPublicKey())
                    .setCharset("UTF-8")
                    .setPrivateKey(aliPayBean.getPrivateKey())
                    .setServiceUrl("https://openapi.alipay.com/gateway.do?")
                    .setSignType("RSA2")
                    .build();
            AliPayApiConfigKit.setThreadLocalAppId(aliPayBean.getAppId()); // 2.1.2 之后的版本，可以不用单独设置
            AliPayApiConfigKit.setThreadLocalAliPayApiConfig(aliPayApiConfig);
        }
        return aliPayApiConfig;
    }

    /**
     * 应用授权回调
     */
    @RequestMapping(value = "/getBuyerId")
    @ResponseBody
    public BaseResponse getBuyerId(@RequestParam("auth_code") String authCode) {
        try {
            AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
            request.setCode(authCode);
            request.setGrantType("authorization_code");
            AliPayApiConfig config = getApiConfig();
            AlipayClient client = new DefaultAlipayClient("https://openapi.alipay.com/gateway.do",
                    config.getAppId(),
                    config.getPrivateKey(),
                    "json",
                    "utf-8",
                    config.getAliPayPublicKey(),
                    "RSA2");
            AlipaySystemOauthTokenResponse oauthTokenResponse = client.execute(request);
            return BaseResponse.ok(oauthTokenResponse.getUserId());
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return BaseResponse.error("服务异常",AliController.class);
    }

    @PostMapping("/checkRsa")
    public BaseResponse rsaCheck(String param){
        //1. 获取验签和解密所需要的参数
        Map<String, String> openapiResult = JSON.parseObject(param,
                new TypeReference<Map<String, String>>() {
                }, Feature.OrderedField);
        String signType = "RSA2";
        String charset = "UTF-8";
        String encryptType = "AES";
        String sign = openapiResult.get("sign");
        String content = openapiResult.get("response");

        //如果密文的
        boolean isDataEncrypted = !content.startsWith("{");
        boolean signCheckPass = false;

        //2. 验签
        String signContent = content;
        AliPayApiConfig config = getApiConfig();
        String signVeriKey = config.getAliPayPublicKey();
        String decryptKey = config.getPrivateKey();
        //如果是加密的报文则需要在密文的前后添加双引号
        if (isDataEncrypted) {
            signContent = "\"" + signContent + "\"";
        }
        try {
            signCheckPass = AlipaySignature.rsaCheck(signContent, sign, signVeriKey, charset, signType);
        } catch (AlipayApiException e) {
            log.error("验签失败",AliPayApiConfig.class);
        }
        if(!signCheckPass) {
            return BaseResponse.error("验签失败",AliPayApiConfig.class.getName());
        }
        String plainData = null;
        if (isDataEncrypted) {
            try {
                AlipayEncrypt.decryptContent(content, encryptType, decryptKey, charset);
            } catch (AlipayApiException e) {
                //解密异常, 记录日志
                return BaseResponse.error("解密异常",AliPayApiConfig.class.getName());
            }
        } else {
            plainData = content;
        }
        return BaseResponse.ok(plainData);
    }

}
