package cn.wise.sc.pay.business.order.controller;

import cn.wise.sc.pay.api.eshopping.service.IEshopGoodsService;
import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.business.comment.controller.CommonCommentController;
import cn.wise.sc.pay.business.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 订单-评论控制器
 */
@RestController
@RequestMapping(value = "/order/comment")
public class OrderCommentController extends CommonCommentController {
    @Autowired
    IOrdersService ordersService;
    @Autowired
    IEshopGoodsService eshopGoodsService;

    @PostMapping("/addComment")
    @ApiOperation("评论")
    @ApiImplicitParam(name = "comment", value = "评论实体", paramType = "body", dataType = "Comment")
    @Override
    public BaseResponse addComment(@RequestBody Comment comment) {
        BaseResponse response = super.addComment(comment);
        if (response.getCode() == 400) {
            return response;
        }
        //添加商品评论数量
        BaseResponse baseResponse = eshopGoodsService.incrCommentNumById(comment.getTargetId().intValue());
        if (baseResponse.getCode() == 400) {
            this.deleteComment(String.valueOf(comment.getTargetId()));
            return baseResponse;
        }
        return BaseResponse.ok();
    }
    @GetMapping("/commentPage")
    @ApiOperation("根据评论主体 评论列表 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "targetId", value = "评论主体id", paramType = "query", dataType = "Long"),
            @ApiImplicitParam(name = "pageQuery", value = "分页查询对象", paramType = "query", dataType = "PageQuery")
    })
    @Override
    public BaseResponse<Page<Comment>> getCommentsByTargetId(Long targetId, PageQuery pageQuery) {
        return super.getCommentsByTargetId(targetId, pageQuery);
    }
    @DeleteMapping("/deleteComment")
    @ApiOperation("根据ids删除评论 多个用英文逗号隔开")
    @ApiImplicitParam(name = "idStr", value = "id 多个用逗号分割", paramType = "body", dataType = "String")
    @Override
    public BaseResponse deleteComment(@RequestBody String idStr) {
        return super.deleteComment(idStr);
    }

    @PutMapping("/updatePraiseNum/{id}")
    @ApiOperation("评论点赞")
    @ApiImplicitParam(name = "id", value = "评论id", paramType = "path", dataType = "Long")
    @Override
    public BaseResponse updateCommentPraise(@PathVariable Long id) {
        return super.updateCommentPraise(id);
    }

    @GetMapping("/selectComment")
    @ApiOperation("评论列表 分页")
    @Override
    public BaseResponse getCommentList(PageQuery pageQuery) {
        return super.getCommentList(pageQuery);
    }


}
