package cn.wise.sc.pay.business.order.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.order.service.IOrdersService;
import cn.wise.sc.pay.business.order.service.Producer;
import cn.wise.sc.pay.business.order.service.strategy.OrderStatisticsStrategyUtil;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.excel.OrderExcel;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 订单服务
 * @author: qh
 * @create: 2019-12-14 11:43
 **/
@RestController
@RequestMapping("/order")
@CrossOrigin
public class OrderController {

    final
    Producer producer;
    final
    IOrdersService ordersService;
    final
    RabbitTemplate rabbitTemplate;

    public OrderController(Producer producer, IOrdersService ordersService, RabbitTemplate rabbitTemplate) {
        this.producer = producer;
        this.ordersService = ordersService;
        this.rabbitTemplate = rabbitTemplate;
    }

    @PostMapping("bill")
    public OrderResponse importAddrlist(MultipartFile file) {
        return null;
    }

    /**
     * 创建订单
     * @param orderDetailQuery
     * @return
     */
    @PostMapping
    public OrderResponse createOrder(@RequestBody OrderDetailQuery orderDetailQuery) {
        return ordersService.createOrder(orderDetailQuery);
    }

    /**
     * 取消订单
     * @param orderNo
     * @return
     */
    @PutMapping("/{orderNo}")
    public OrderResponse cancelOrder(@PathVariable("orderNo") String orderNo) {
        return ordersService.cancelOrder(orderNo);
    }

    @GetMapping("/excel/orderInfo/{shopId}")
    public BaseResponse<List<OrderExcel>> orderInfoExcelByShop(@PathVariable(value = "shopId") Long shopId,
                                                               @RequestParam("start") Long start,
                                                               @RequestParam("end") Long end) {
        //获取数据
        QueryWrapper qw = new QueryWrapper();
        qw.eq("order_status", "WAIT_DELIVERING");
        if (shopId != 0) {
            qw.eq("seller_id", shopId);
        }
        if (start != 0 && end != 0) {
            qw.between("update_time", start, end);
        }
        List<Orders> catagoryList = ordersService.list(qw);
        List<OrderExcel> list = new ArrayList<>(catagoryList.size());
        for (Orders orders : catagoryList) {
            OrderExcel orderExcel = new OrderExcel();
            BeanUtil.copyProperties(orders, orderExcel, "createTime", "updateTime", "payTime");
            DateTime date = DateUtil.date(orders.getCreateTime());
            orderExcel.setCreateTime(DateUtil.formatDateTime(date));
            DateTime date2 = DateUtil.date(orders.getUpdateTime());
            orderExcel.setUpdateTime(DateUtil.formatDateTime(date2));

            list.add(orderExcel);
        }

        return BaseResponse.ok(list);
    }

    /**
     * 删除订单
     *
     * @param orderNo
     * @param conditionJson
     * @return
     */
    @DeleteMapping
    public OrderResponse delOrder(@RequestParam("orderNo") String orderNo, @RequestParam("condition") String
            conditionJson) {

        Map condition = json2Map(conditionJson);
        return ordersService.delOrders(orderNo, condition);
    }

    /**
     * 修改订单状态
     *
     * @param orderNo
     * @param orderStatus
     * @param conditionJson
     * @return
     */
    @PutMapping("/status")
    public OrderResponse changeOrderStatus(@RequestParam("orderNo") String orderNo,
                                           @RequestBody OrderStatus orderStatus,
                                           @RequestParam("conditionJson") String conditionJson) {
        Map condition = new HashMap();
        if (StrUtil.isNotEmpty(conditionJson)) {
            condition = json2Map(conditionJson);
        }
        return ordersService.changeOrderStatus(orderNo, orderStatus, condition);
    }


    /**
     * 修改订单状态 同时更新订单id
     *
     * @param orderNo
     * @param userId
     * @param orderStatus
     * @param conditionJson
     * @return
     */
    @PutMapping("/status/{userId}")
    OrderResponse changeOrderStatusByUserId(@RequestParam("orderNo") String orderNo,
                                            @PathVariable("userId") String userId,
                                            @RequestBody OrderStatus orderStatus,
                                            @RequestParam("condition") String conditionJson) {
        Map condition = json2Map(conditionJson);
        return ordersService.changeOrderStatus(orderNo, orderStatus, condition);
    }

    /**
     * 修改订单
     *
     * @param orderDetailQuery
     * @return
     */
    @PutMapping
    public OrderResponse editOrder(@RequestBody OrderDetailQuery orderDetailQuery) {
        return ordersService.editOrder(orderDetailQuery);
    }

    /**
     * 获取预下单号
     *
     * @return
     */
    @GetMapping("/preOrderNo")
    public BaseResponse<String> getOrderNo() {

        String orderNo = ordersService.getPreOrderNo();
        BaseResponse<String> eventResult = BaseResponse.ok("获取预下单号儿成功", orderNo);
        return eventResult;
    }

    /**
     * 获取订单列表
     *
     * @param conditionJson
     * @return
     */
    @GetMapping("/list")
    public OrderResponse getOrders(@RequestParam("condition") String conditionJson) {

        Map condition = json2Map(conditionJson);
        return ordersService.getOrders(condition);
    }

    /**
     * 获取订单集合 复杂查询
     *
     * @param pageWrapperQuery
     * @return
     */
    @PostMapping("/list/complex")
    public OrderResponse getOrders(@RequestBody PageWrapperQuery pageWrapperQuery) {
        return ordersService.getListcomplex(pageWrapperQuery);
    }

    /**
     * 获取订单列表 根据用户id
     *
     * @param userId 用户id
     * @param status 订单状态
     * @return
     */
    @PostMapping("/list")
    public OrderResponse getOrdersByUserId(@RequestParam("condition") String userId, @RequestBody OrderStatus
            status) {

        return ordersService.getOrdersByUserId(userId, status);
    }

    @GetMapping
    public OrderResponse getOneOrder(@RequestParam("condition") String conditionJson) {

        Map condition = json2Map(conditionJson);
        return ordersService.getOneOrder(condition);
    }

    /**
     * 满足条件的条数
     *
     * @param start    起始时间
     * @param end      结束时间
     * @param status   订单状态
     * @param sellerId 商户id
     * @return
     */
    @GetMapping("/count")
    public BaseResponse<Long> getCount(@RequestParam("start") Long start, @RequestParam("end") Long end,
                                       @RequestBody OrderStatus status, @RequestParam("sellerId") String sellerId) {
        return ordersService.countOrdersByDate(start, end, status, sellerId);
    }

    /**
     * 分页查询
     *
     * @param pageWrapperQuery
     * @return
     */
    @PostMapping("/page")
    public OrderResponse page(@RequestBody PageWrapperQuery pageWrapperQuery) {
        return ordersService.page(pageWrapperQuery);
    }

    /**
     * 批量更新 根据id
     *
     * @param ordersCancels
     * @return
     */
    @PutMapping("/batch")
    public BaseResponse<Boolean> updateBatch(@RequestBody List<OrderDetailQuery> ordersCancels) {

        boolean isSuccess = ordersService.updateBatch(ordersCancels);
        if (!isSuccess) {
            BaseResponse<Boolean> booleanBaseResponse = BaseResponse.error("更新失败", isSuccess);
            return booleanBaseResponse;
        }
        BaseResponse<Boolean> booleanBaseResponse = BaseResponse.ok("更新成功", isSuccess);
        return booleanBaseResponse;
    }


    /**
     * 统计  根据交易类型统计
     *
     * @return
     */
    @GetMapping("/statistics/status")
    public BaseResponse statisticsByStatus(int strategy, String shopname, long start, long end) {

        return ordersService.statisticsTop(OrderStatisticsStrategyUtil.getStrategy(strategy, ordersService), shopname, start, end);
    }

    /**
     * 统计  根据交易类型统计  (折线图)
     *
     * @return
     */
    @GetMapping("/statistics/chart")
    public BaseResponse statisticsChart(int strategy, String shopname, long start, long end) {

        return ordersService.statisticsByChart(OrderStatisticsStrategyUtil.getStrategy(strategy, ordersService), shopname, start, end);
    }

    /**
     * 统计  根据支付类型 统计金额
     *
     * @return
     */
    @GetMapping("/statistics/payMethod/Money")
    public BaseResponse statisticsMoneyByPatMethod(int strategy, String shopname, long start, long end) {

        return ordersService.statisticsMoneyPayMethod(OrderStatisticsStrategyUtil.getStrategy(strategy, ordersService), shopname, start, end);
    }

    /**
     * 统计  根据支付类型统计笔数
     *
     * @return
     */
    @GetMapping("/statistics/payMethod/Count")
    public BaseResponse statisticsCountByPatMethod(int strategy, String shopname, long start, long end) {
        return ordersService.statisticsCountPayMethod(
                OrderStatisticsStrategyUtil.getStrategy(strategy, ordersService), shopname, start, end);
    }

    /**
     * 统计  当前统计下的所有订单
     *
     * @return
     */
    @GetMapping("/statistics/list")
    public BaseResponse statisticsList(int strategy, String shopname, long start,
                                       long end, int payment) {
        return ordersService.statisticsList(OrderStatisticsStrategyUtil.getStrategy(strategy, ordersService), shopname, start, end, payment);
    }

    /**
     * 获取时间端的聚合支付金额&现金支付的总数
     *
     * @param pageWrapperQuery 查询条件构造 {@link PageWrapperQuery}
     * @return
     */
    @PostMapping("/statistics/aggregate")
   public BaseResponse statisticsAggregate(@RequestBody PageWrapperQuery pageWrapperQuery) {
        return ordersService.statisticsAggregate(pageWrapperQuery);
    }

    /**
     * 统计商品售卖的金额  通过卖家id
     *
     * @param pageWrapperQuery 条件
     * @return
     */
    @PostMapping("/statistics/goods/market")
    public BaseResponse statisticsGoodsMarketMoneyGroupSellerGood(@RequestBody PageWrapperQuery pageWrapperQuery) {
        return ordersService.statisticsGoodsMarketMoneyGroupSellerGood(pageWrapperQuery);
    }

    /**
     * JSON转MAP
     *
     * @param codition
     * @return
     */
    private Map json2Map(String codition) {
        Map condition;
        try {
            condition = JSON.parseObject(codition, HashMap.class);

        } catch (Exception ex) {
            condition = new HashMap();
        }

        return condition;
    }
}
