package cn.wise.sc.payment.cmb.service;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.payment.cmb.model.MerchantPayPro;
import com.alibaba.fastjson.JSON;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;

/**
 * @description: 入驻商户支付信息服务
 * @author: qh
 * @create: 2020-09-16 10:23
 **/
@Component
public class MerchantPayService {

    final
    RedissonClient redissonClient;

    /**
     * redis前缀
     */
    private final String MERCHANT_PAY_INFO_PRE = "MERCHANT:PAY:INFO:";

    public MerchantPayService(RedissonClient redissonClient) {
        this.redissonClient = redissonClient;
    }


    /**
     * 获取商户配置信息
     *
     * @param sellerId 商户id
     * @return MerchantPayPro
     */
    public MerchantPayPro getBySellerId(String sellerId) {
        if (StrUtil.isEmpty(sellerId)) {
            //适配不存sellerId的情况
            //这里适配的大富贵
            MerchantPayPro merchantPayPro = new MerchantPayPro();
            merchantPayPro.setMerId("30899915812002G");
            merchantPayPro.setCashier("");
            merchantPayPro.setCmbAppId("");
            merchantPayPro.setSecretKey("");
            merchantPayPro.setMerchantPrivateKey("");
            return merchantPayPro;
        }
        String key = MERCHANT_PAY_INFO_PRE + sellerId;
        RBucket<String> bucket = redissonClient.getBucket(key);
        //判断是否存在key
        if (bucket.isExists()) {
            String merchantPayStr = bucket.get();
            return JSON.parseObject(merchantPayStr, MerchantPayPro.class);
        } else {
            //不存在时
            //todo mysql读取
            bucket.set("");
            return null;
        }
    }

}
