package cn.wise.sc.payment.common;

import cn.hutool.cache.Cache;
import cn.hutool.core.convert.Convert;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;

/**
 * @author neo.shu
 * @since 2019/11/6 15:19
 */

public class CacheUtil {

    private final ReentrantReadWriteLock cacheLock = new ReentrantReadWriteLock();
    private final ReadLock readLock;
    private final WriteLock writeLock;
    public static Cache<String, Map<String, AtomicInteger>> hutoolCache4Int = cn.hutool.cache.CacheUtil.newFIFOCache(100);

    public CacheUtil() {
        this.readLock = this.cacheLock.readLock();
        this.writeLock = this.cacheLock.writeLock();
    }

    public static String covert2str(Object o) {
        return Convert.toStr(o);
    }

    public void cacheInt(String key, String mapKey) {
        this.writeLock.lock();
        try {
            Map<String, AtomicInteger> map = hutoolCache4Int.get(key);
            if (map == null) {
                map = new ConcurrentHashMap<>();
                map.put(mapKey, new AtomicInteger(1));
            } else {
                AtomicInteger a = map.get(mapKey)==null?new AtomicInteger(0):map.get(mapKey);
                a.incrementAndGet();
                map.put(mapKey, a);
            }
            hutoolCache4Int.put(key, map);
        } finally {
            this.writeLock.unlock();
        }

    }

    public Integer getInt(String key, String mapKey) {
        this.readLock.lock();

        int val;
        try {
            val = hutoolCache4Int.get(key).get(mapKey).get();
        } finally {
            this.readLock.unlock();
        }

        return val;
    }

    public static void main(String[] args) {
        CacheUtil cacheUtil = new CacheUtil();

        new Thread(() -> {
            cacheUtil.cacheInt("article", "id13");
            System.out.println(cacheUtil.getInt("article", "id13"));
        }).start();

        new Thread(() -> {
            cacheUtil.cacheInt("article", "id12");
            System.out.println(cacheUtil.getInt("article", "id12"));
        }).start();

        new Thread(() -> {
            cacheUtil.cacheInt("article", "id12");
            System.out.println(cacheUtil.getInt("article", "id12"));
        }).start();

        new Thread(() -> {
            cacheUtil.cacheInt("article", "id12");
            System.out.println(cacheUtil.getInt("article", "id12"));
        }).start();


    }
}
