package cn.wise.sc.pay.api.points.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.CashPointRecord;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.math.BigDecimal;

/**
 * @description: 现金返回积分a'pi
 * @author: qh
 * @create: 2020-05-11 16:31
 **/
@FeignClient(value = "saas-v1-points", path = "cashPointRecord", fallback = CashPointRecordServiceImpl.class)
public interface ICashPointRecordService {

    /**
     * 根据积分和会员等级获取扫码积分
     *
     * @param orderNo 订单号  必填
     * @param money   现金消费金额 必填
     * @param userId  用户id 非必填(考虑用户未注册)
     * @return CashPointRecord
     */
    @PostMapping
    BaseResponse<CashPointRecord> getPoints(@RequestParam("orderNo") String orderNo,
                                            @RequestParam("money") BigDecimal money,
                                            @RequestParam("userId")Long userId);

    /**
     * 扫码获取积分
     *
     * @param id     扫码积分兑换主键
     * @param userId 用户id  必填
     * @return String
     */
    @PutMapping
    BaseResponse<String> usePoint(@RequestParam("id")Long id,@RequestParam("userId") Long userId);
}
