package cn.wise.sc.pay.api.points.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import cn.wise.sc.pay.domain.point.vo.PointDetailVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiImplicitParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@FeignClient(value = "saas-v1-points", path = "/pointDetail", fallback = PointDetailImpl.class)
public interface IPointDetailService {

    @PutMapping("/add")
    ResponseEntity add(@RequestBody PointDetailQuery pointDetailQuery);

    /**
     * 查询用户积分消费记录
     *
     * @param pageNo   起始页
     * @param pageSize 每页大小
     * @param userId   用户id
     * @return
     */
    @GetMapping("/consume/{userId}")
    BaseResponse listByConsume(@RequestParam("pageNo") Integer pageNo,
                               @RequestParam("pageSize") Integer pageSize,
                               @PathVariable("userId") Long userId);

    /**
     * 查询用户积分增加记录
     *
     * @param pageNo   起始页
     * @param pageSize 每页大小
     * @param userId   用户id
     * @return
     */
    @GetMapping("/increase/{userId}")
    BaseResponse listByIncrease(@RequestParam("pageNo") Integer pageNo,
                                @RequestParam("pageSize") Integer pageSize,
                                @PathVariable("userId") Long userId);

    /**
     * 根据用户id查询用户积分记录
     *
     * @param
     * @param userId
     * @return
     */
    @GetMapping("/{userId}")
    @ApiImplicitParam(name = "userId", value = "userId", paramType = "path", dataType = "int")
    BaseResponse<Page<PointDetailVO>> list(@RequestParam("pageNo") Integer pageNo,
                                           @RequestParam("pageSize") Integer pageSize,
                                           @PathVariable("userId") int userId);
}
