package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.IGiftGoodsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.query.GiftGoodsQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 *实物商品控制器
 * @author jiawei
 */
@RestController
@RequestMapping("/giftGoods")
@Api(tags = "实物商品相关接口")
@CrossOrigin
public class GiftGoodsController {
    @Autowired
    IGiftGoodsService giftGoodsService;

    @ApiOperation(value = "添加一条积分商城  实物商品记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "giftGoodsQuery", value = "实物商品类型参数对象", dataType = "GiftGoodsQuery", paramType = "body")
            }
    )
    @PostMapping
    public BaseResponse add(@Valid @RequestBody GiftGoodsQuery giftGoodsQuery) {
        BaseResponse response = giftGoodsService.add(giftGoodsQuery);
        return response;
    }

    @GetMapping("/page")
    @ApiOperation(value = "根据商品名称条件分页")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "name", value = "商品名称", dataType = "String", paramType = "query")
            }
    )
    public BaseResponse pageOnCondition(String name, PageQuery pageQuery) {
        BaseResponse response = giftGoodsService.pageOnCondition(name, pageQuery);
        return response;
    }

    @DeleteMapping("/{id}")
    @ApiOperation(value = "根据id值删除记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "path")
            }
    )
    public BaseResponse remove(@PathVariable Long id) {

        BaseResponse response = giftGoodsService.discard(id);
        return response;
    }

    @PutMapping
    @ApiOperation(value = "修改数据信息，传入数据时请传入id值")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "couponTypeQuery", value = "参数封装对象", dataType = "CouponTypeQuery", paramType = "body")
            }
    )
    public BaseResponse edit(@RequestBody GiftGoodsQuery giftGoodsQuery) {
        BaseResponse response = giftGoodsService.edit(giftGoodsQuery);
        return response;
    }

    @GetMapping("{id}")
    @ApiOperation(value = "根据id值查询记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "long", paramType = "path")
            }
    )
    public BaseResponse infoById(@PathVariable Long id) {
        BaseResponse response = giftGoodsService.infoById(id);
        return response;
    }

}
