package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.IGoodRecordService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.query.DeliveryInfoQuery;
import cn.wise.sc.pay.domain.point.query.GoodExchangeQuery;
import cn.wise.sc.pay.domain.point.query.GoodRecordAddressQuery;
import cn.wise.sc.pay.domain.point.query.GoodRecordQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;

/**
 * 商品兑换记录控制器
 *
 * @author jiawei
 */
@RestController
@RequestMapping("/goodsRecord")
@Api(tags = "商品兑换记录相关接口")
@CrossOrigin
public class GoodRecordController {

    @Autowired
    IGoodRecordService goodRecordService;

    @ApiOperation(value = "添加一条商品兑换记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "giftGoodsQuery", value = "实物商品类型参数对象", dataType = "GiftGoodsQuery", paramType = "body")
            }
    )
    @PostMapping("/record")
    @ApiIgnore
    public BaseResponse addRecord(@RequestBody @Valid GoodRecordQuery goodRecordQuery) {
        BaseResponse response = goodRecordService.add(goodRecordQuery);
        return response;
    }

    @ApiOperation(value = "添加一条发货信息")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "deliveryInfoQuery", value = "发货信息参数对象", dataType = "DeliveryInfoQuery", paramType = "body")
            }
    )
    @PostMapping("/deliveryInfo")
    public BaseResponse addDeliveryInfo(@Valid @RequestBody DeliveryInfoQuery deliveryInfoQuery) {
        BaseResponse response = goodRecordService.addDeliveryInfo(deliveryInfoQuery);
        return response;
    }


    @GetMapping("/virtualGood/page")
    @ApiOperation(value = "虚拟商品条件分页")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "name", value = "券名称", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "status", value = "状态", dataType = "int", paramType = "query")
            }
    )
    public BaseResponse pageOnVirtualGoods(String name, PageQuery pageQuery, Integer status) {
        BaseResponse response = goodRecordService.pageOnVirtualGoods(name, pageQuery, status);
        return response;
    }

    @GetMapping("/giftGood/page")
    @ApiOperation(value = "实物商品条件分页")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "name", value = "商品名称", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "status", value = "状态", dataType = "int", paramType = "query")
            }
    )
    public BaseResponse pageOnGiftGoods(String name, PageQuery pageQuery, Integer status) {
        BaseResponse response = goodRecordService.pageOnGiftGoods(name, pageQuery, status);
        return response;
    }

    @ApiOperation(value = "获取一条信息记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "发货信息参数对象", dataType = "id", paramType = "path")
            }
    )
    @GetMapping("/{id}")
    public BaseResponse getRecord(@PathVariable Long id) {
        BaseResponse response = goodRecordService.getRecord(id);
        return response;
    }

    //
    @ApiOperation(value = "分页显示积分可兑换商品列表")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "pageNo", value = "分页参数", dataType = "int", paramType = "query"),
                    @ApiImplicitParam(name = "pageSize", value = "分页参数", dataType = "int", paramType = "query")
            }
    )
    @GetMapping("/exchangeRecord")
    public BaseResponse pageExchange(PageQuery pageQuery) {
        BaseResponse response = goodRecordService.pageExchange(pageQuery);
        return response;
    }

    @ApiOperation(value = "商品兑换")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "goodExchangeQuery", value = "商品兑换参数封装", dataType = "GoodExchangeQuery", paramType = "body")
            }
    )
    @PutMapping("/exchangeGoods")
    public BaseResponse exchangeGoods(@Valid @RequestBody GoodExchangeQuery goodExchangeQuery) {
        BaseResponse response = goodRecordService.exchangeGoods(goodExchangeQuery);
        return response;
    }


    @ApiOperation(value = "根据用户id分页查询兑换记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "userId", value = "用户id", dataType = "long", paramType = "path")
            }
    )
    @GetMapping("/page/{userId}")
    public BaseResponse pageByUserId(@NotNull(message = "用户id不能为空") @PathVariable Long userId, PageQuery pageQuery) {
        BaseResponse response = goodRecordService.pageByUserId(userId, pageQuery);
        return response;
    }

    @ApiOperation(value = "获取最新的兑换记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "size", value = "最新记录的条数", dataType = "int", paramType = "path")
            }
    )
    @GetMapping("/latestRecords/{size}")
    public BaseResponse getLatestRecords(@NotNull(message = "展示条数不能为空") @PathVariable Integer size) {
        BaseResponse response = goodRecordService.getLatestRecords(size);
        return response;
    }

    @ApiOperation(value = "添加实物商品兑换记录的收货地址和收货人")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "goodRecordAddressQuery", value = "参数封装对象", dataType = "GoodRecordAddressQuery", paramType = "body")
            }
    )
    @PutMapping("/add/address")
    public BaseResponse addAddressById(@RequestBody GoodRecordAddressQuery goodRecordAddressQuery) {
        BaseResponse response = goodRecordService.addAddressById(goodRecordAddressQuery);
        return response;
    }
}
