package cn.wise.sc.pay.business.points.controller;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.service.IMembershipService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.query.MemberQuery;
import cn.wise.sc.pay.domain.point.vo.MembershipVO;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 会员等级控制器
 * @author: qh
 * @create: 2019-11-11 11:37
 **/
@CrossOrigin
@Api(description = "会员等级相关api")
@RequestMapping("/membership")
@RestController
public class MembershipController {

    @Autowired
    IMembershipService iMembershipService;

    @Autowired
    FastDFSUtils fastDFSUtils;

    /**
     * 查询所有(条件)
     *
     * @param pageWrapperQuery 关键字
     * @return
     */
    @PostMapping("/page")
    BaseResponse<Page<MembershipVO>> page(@RequestBody PageWrapperQuery pageWrapperQuery) {
        return iMembershipService.list(pageWrapperQuery);
    }

    @GetMapping("/list")
    @ApiOperation("查询所有(条件)")
    @ApiImplicitParam(name = "key", value = "关键字", paramType = "query", dataType = "string")
    public BaseResponse<Page<MembershipVO>> page(PageQuery pageQuery, String key) {

        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        pageWrapperQuery.setPageQuery(pageQuery);
        if (StrUtil.isNotEmpty(key)) {
            Map<String, Object> like = MapUtil.newConcurrentHashMap(1);
            like.put("name", key);
            pageWrapperQuery.setLike(like);
        }
        return iMembershipService.list(pageWrapperQuery);
    }

    @GetMapping("/user/{userId}")
    @ApiOperation("根据用户id获取他的会员等级")
    public BaseResponse getShipByUserId(@PathVariable("userId") long userId) {
        return iMembershipService.getShipByUserId(userId);
    }


    @GetMapping("/{id}")
    @ApiOperation("查询单个")
    @ApiImplicitParam(name = "id", value = "id", paramType = "path", dataType = "int")
    public BaseResponse getMembership(@PathVariable("id") int id) {

        return iMembershipService.getMembexrship(id);
    }

    @PutMapping("/{id}/{status}")
    @ApiOperation("激活/禁用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "状态 1:激活 0:禁用", paramType = "path", dataType = "int")
    })
    public BaseResponse forbiddenOrActive(@PathVariable("id") int id, @PathVariable("status") int status) {

        return iMembershipService.forbiddenOrActive(id, status);
    }

    @PutMapping
    @ApiOperation("编辑")
    public BaseResponse editMembership(@RequestBody MemberQuery memberQuery) {

        return iMembershipService.editMembership(memberQuery);
    }

    @DeleteMapping("/{id}")
    @ApiOperation("删除")
    @ApiImplicitParam(name = "id", value = "会员等级id", paramType = "path", dataType = "int")
    public BaseResponse deleteUsers(@PathVariable("id") int id) {

        return iMembershipService.deleteMembership(id);
    }


    @PostMapping("/new")
    @ApiOperation("添加新的会员等级(折扣传总数如 7.5折=75)")
    public BaseResponse newMembership(@RequestBody MembershipVO membershipVo) {
        return iMembershipService.newMembership(membershipVo);
    }

    @GetMapping("/getMemberInfo")
    @ApiOperation("根据会员id获取会员信息以及会员权益信息")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "memberId", value = "会员等级id", dataType = "long", paramType = "query")
            }
    )
    public BaseResponse<UserExtraVOApp> getMemberInfo(Long memberId) {
        BaseResponse<UserExtraVOApp> response = iMembershipService.getMemberInfo(memberId);
        return response;
    }

    @PostMapping("/upload")
    @ApiOperation("上传图片")
    public BaseResponse upload(MultipartFile file) {

        Map<String, String> map = new HashMap<>();
        String path = "";
        if (file != null && StringUtils.isNotEmpty(file.getOriginalFilename())) {
            try {
                path = fastDFSUtils.uploadFile(file).getPath();
                map.put("host", fastDFSUtils.fullUrl(path));
            } catch (IOException e) {
                return BaseResponse.error("图片上传失败", map);
            }
        }
        map.put("uri", path);
        return BaseResponse.ok(map);
    }

    /**
     * 根据id集合查询
     *
     * @param memberShipIds
     * @return
     */
    @GetMapping("/list/ids")
    public BaseResponse<List<Membership>> listByIds(@RequestParam("memberShipIds") List<Long> memberShipIds) {
        BaseResponse<List<Membership>> listByIds = iMembershipService.getListByIds(memberShipIds);
        return listByIds;

    }

    @ApiIgnore
    @ApiOperation(value = "获取最初级的会员等级对象")
    @GetMapping("/primary")
    public BaseResponse<Membership> getPrimaryMemberShip() {
        BaseResponse<Membership> primaryMemberShip = iMembershipService.getPrimaryMemberShip();
        return primaryMemberShip;
    }
}
