package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.IPointDetailService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import cn.wise.sc.pay.domain.point.vo.PointDetailVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 订单明细
 * </p>
 *
 * @author qh
 * @since 2019-11-13
 */
@Api(tags = "积分明细")
@CrossOrigin
@RestController
@RequestMapping("/pointDetail")
public class PointDetailController {

    @Autowired
    IPointDetailService iPointDetailService;

    @GetMapping("/{userId}")
    @ApiOperation("更具条件查询所有")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "long"),
                    @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "query", dataType = "int"),
                    @ApiImplicitParam(name = "pageSize", value = "每页显示数量", paramType = "query", dataType = "int")
            }
    )
    public BaseResponse<Page<PointDetailVO>> list(PageQuery pageQuery, @PathVariable("userId") int userId) {
        return iPointDetailService.listByUserId(userId, pageQuery);
    }

    @ApiOperation("查询用户积分消费记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "long"),
                    @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "query", dataType = "int"),
                    @ApiImplicitParam(name = "pageSize", value = "每页显示数量", paramType = "query", dataType = "int")
            }
    )
    @GetMapping("/consume/{userId}")
    public BaseResponse listByConsume(Integer pageNo, Integer pageSize, @PathVariable Long userId) {
        BaseResponse responseEntity = iPointDetailService.listByCOrI(pageNo, pageSize, userId, 0);
        return responseEntity;
    }

    @ApiOperation("查询用户积分增加记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "long"),
                    @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "query", dataType = "int"),
                    @ApiImplicitParam(name = "pageSize", value = "每页显示数量", paramType = "query", dataType = "int")
            }
    )
    @GetMapping("/increase/{userId}")
    public BaseResponse listByIncrease(Integer pageNo, Integer pageSize, @PathVariable Long userId) {
        BaseResponse responseEntity = iPointDetailService.listByCOrI(pageNo, pageSize, userId, 1);
        return responseEntity;
    }


    @PutMapping("/add")
    @ApiOperation("新增积分详情")
    @ApiImplicitParam(name = "pointDetailQuery", value = "积分详情对象", paramType = "body", dataType = "PointDetailQuery")
    public BaseResponse add(@RequestBody PointDetailQuery pointDetailQuery) {

        BaseResponse responseEntity = iPointDetailService.addPointDetail(pointDetailQuery);
        return responseEntity;
    }

}

