package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.IPointSourceV1Service;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.query.PointSourceV1Query;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.ArrayList;
import java.util.List;

/**
 * 积分来源相关接口
 */
@RestController
@RequestMapping("/points/pointSource")
@CrossOrigin
@Api(tags = "积分来源相关接口")
public class PointSourceV1Controller {
    @Autowired
    IPointSourceV1Service pointSourceV1Service;

    @ApiIgnore  //暂不用
    @ApiOperation(value = "新增一条积分来源记录", notes = "")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "pointSourceV1Query", value = "参数封装对象", dataType = "PointSourceV1Query", paramType = "body")
            }
    )
    @PostMapping
    public BaseResponse save(@RequestBody PointSourceV1Query pointSourceV1Query) {
        BaseResponse response = pointSourceV1Service.save(pointSourceV1Query);
        return response;
    }

    @GetMapping(value = "/list/condition")
    @ApiOperation(value = "根据途径查询和数据列表查询", notes = "发布文章,  线上消费,  签到奖励,  优质点评")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "way", value = "途径标识符", dataType = "String", paramType = "query")
            }
    )
    public BaseResponse listOnCondition(String way) {
        BaseResponse response = pointSourceV1Service.listOnCondition(way);
        return response;
    }

    @ApiIgnore
    @PutMapping("/status/{id}")
    @ApiOperation(value = "根据id禁用")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "id主键", dataType = "int", paramType = "path")
    )
    public BaseResponse changeStatusById(@PathVariable Integer id) {
        List<Integer> integers = new ArrayList<>();
        integers.add(id);
        BaseResponse response = pointSourceV1Service.changeStatus(integers);
        return response;
    }

    //修改后
    @PutMapping("/status/way/{way}")
    @ApiOperation(value = "根据途径禁用")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "way", value = "途径类型", dataType = "int", paramType = "path")
    )
    public BaseResponse changeStatusByWay(@PathVariable Integer way) {
        List<Integer> integers = new ArrayList<>();
        integers.add(way);
        BaseResponse response = pointSourceV1Service.changeStatusByWay(integers);
        return response;
    }

    //修改后
    @PutMapping("/enable/way/{way}")
    @ApiOperation(value = "根据途径启用用")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "way", value = "途径类型", dataType = "int", paramType = "path")
    )
    public BaseResponse enableStatusByWay(@PathVariable Integer way) {
        List<Integer> integers = new ArrayList<>();
        integers.add(way);
        BaseResponse response = pointSourceV1Service.enableStatusByWays(integers);
        return response;
    }

    //修改后
    @ApiIgnore
    @PutMapping("/enable/way/ways")
    @ApiOperation(value = "根据途径批量启用")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "way", value = "途径类型", dataType = "int", paramType = "path")
    )
    public BaseResponse enableStatusByWays(List<Integer> ways) {

        BaseResponse response = pointSourceV1Service.enableStatusByWays(ways);
        return response;
    }

    @ApiIgnore
    @PutMapping("/status/")
    @ApiOperation(value = "根据id批量禁用")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "ids", value = "id主键", dataType = "list", paramType = "body")
    )
    public BaseResponse changeStatusById(List<Integer> ids) {
        BaseResponse response = pointSourceV1Service.changeStatus(ids);
        return response;
    }

    //根据途径批量禁用  修改后
    @ApiIgnore
    @PutMapping("/status/ways")
    @ApiOperation(value = "根据途径批量禁用")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "ways", value = "途径类型集合", dataType = "list", paramType = "body")
    )
    public BaseResponse changeStatusByWays(List<Integer> ways) {
        BaseResponse response = pointSourceV1Service.changeStatusByWay(ways);
        return response;
    }

    @ApiIgnore
    @PutMapping("/update")
    @ApiOperation(value = "根据id修改数据")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "pointSourceV1Query", value = "参数封装对象", dataType = "PointSourceV1Query", paramType = "body")
    )
    public BaseResponse updateById(@RequestBody PointSourceV1Query pointSourceV1Query) {
        BaseResponse response = pointSourceV1Service.updateDataById(pointSourceV1Query);
        return response;
    }

    //根据途径修改
    @PutMapping("/update/way")
    @ApiOperation(value = "根据途径修改数据")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "pointSourceV1Query", value = "参数封装对象", dataType = "PointSourceV1Query", paramType = "body")
    )
    public BaseResponse updateByWay(@RequestBody PointSourceV1Query pointSourceV1Query) {
        BaseResponse response = pointSourceV1Service.updateDataByWay(pointSourceV1Query);
        return response;
    }

    @GetMapping("/score")
    @ApiOperation(value = "根据会员id与途径获取积分值", notes = "正值为增加，负值为消费")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "key", value = "关键字", dataType = "long", paramType = "query"),
                    @ApiImplicitParam(name = "pathway", value = "积分来源途径", dataType = "String", paramType = "query")
            }
    )
    public Double getScore(Long key, String pathway) {
        Double score = pointSourceV1Service.getScore(key, pathway);
        return score;
    }
}
