package cn.wise.sc.pay.business.points.service.impl;

import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.mapper.GiftGoodsMapper;
import cn.wise.sc.pay.business.points.service.IGiftGoodsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.GiftGoods;
import cn.wise.sc.pay.domain.point.query.GiftGoodsQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

/**
 * 积分商城  实物商品业务实现类
 *
 * @author jiawei
 */
@Service
public class GiftGoodsServiceImpl extends ServiceImpl<GiftGoodsMapper, GiftGoods> implements IGiftGoodsService {

    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    /**
     * 添加一条积分商城  实物商品记录
     */
    public BaseResponse add(GiftGoodsQuery giftGoodsQuery) {
        GiftGoods giftGoods = GiftGoodsQuery.query2Entity(giftGoodsQuery);
        //创建时间
        giftGoods.setCreateTime(System.currentTimeMillis());
        //库存数量
        giftGoods.setRemainNum(giftGoods.getAmount());
        boolean save = this.save(giftGoods);
        return save ? BaseResponse.ok(giftGoods) : BaseResponse.error("添加数据失败", GiftGoodsServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据商品名称条件分页
     */
    public BaseResponse pageOnCondition(String name, PageQuery pageQuery) {
        Page<GiftGoods> giftGoodsPage = new Page<>();
        giftGoodsPage.setCurrent(pageQuery.getPageNo());
        giftGoodsPage.setSize(pageQuery.getPageSize());

        QueryWrapper<GiftGoods> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("create_time");
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like("name", name);
        }
        IPage<GiftGoods> page = this.page(giftGoodsPage, queryWrapper);
        if (page != null && !CollectionUtils.isEmpty(page.getRecords())) {
            //拼接图片
            page.getRecords().stream().forEach(re -> re.setImage(fastDFSUtils.fullUrl(re.getImage())));
        }
        return BaseResponse.ok(page, GiftGoodsServiceImpl.class);
    }

    @Override
    /**
     * 根据id值删除记录
     */
    public BaseResponse discard(Long id) {
        if (id == null) {
            return BaseResponse.error("请传入id值", CouponTypeServiceImpl.class.getName());
        }
        boolean remove = this.removeById(id);
        return remove ? BaseResponse.ok() : BaseResponse.error("删除信息失败", CouponTypeServiceImpl.class.getName());
    }

    @Override
    public BaseResponse edit(GiftGoodsQuery giftGoodsQuery) {
        if (giftGoodsQuery.getId() == null) {
            return BaseResponse.error("请传入id值", CouponTypeServiceImpl.class.getName());
        }
        GiftGoods giftGoods = GiftGoodsQuery.query2Entity(giftGoodsQuery);
        giftGoods.setUpdateTime(System.currentTimeMillis());

        boolean update = this.updateById(giftGoods);
        return update ? BaseResponse.ok(giftGoods) : BaseResponse.error("删除信息失败", CouponTypeServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据id值查询记录
     */
    public BaseResponse infoById(Long id) {
        if (id == null) {
            return BaseResponse.error("请传入id值", CouponsServiceImpl.class.getName());
        }
        GiftGoods info = this.getById(id);
        if (info != null) {
            //拼接图片
            info.setImage(fastDFSUtils.fullUrl(info.getImage()));
        }
        return BaseResponse.ok(info, CouponsServiceImpl.class);
    }
}
