package cn.wise.sc.pay.business.points.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.mapper.GoodRecordMapper;
import cn.wise.sc.pay.business.points.service.*;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.*;
import cn.wise.sc.pay.domain.point.cnst.PointTypeConsts;
import cn.wise.sc.pay.domain.point.query.*;
import cn.wise.sc.pay.domain.point.vo.ExchangeVo;
import cn.wise.sc.pay.domain.point.vo.GoodRecordVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 商品兑换记录业务实现类
 */
@Service
public class GoodRecordServiceImpl extends ServiceImpl<GoodRecordMapper, GoodRecord>
        implements IGoodRecordService {
    @Autowired
    GoodRecordMapper goodRecordMapper;
    @Autowired
    ICouponsService couponsService;
    @Autowired
    ICouponTypeService couponTypeService;
    @Autowired
    IGiftGoodsService giftGoodsService;
    @Autowired
    IVirtualGoodsService virtualGoodsService;
    @Autowired
    IUserExtraService iUserExtraService;
    @Autowired
    IPointDetailService pointDetailService;

    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    /** 添加一条商品兑换记录 */
    public BaseResponse add(GoodRecordQuery goodRecordQuery) {
        if (goodRecordQuery.getType() == null) {
            return BaseResponse.error("请传入记录所属类型", GoodRecordServiceImpl.class.getName());
        }
        GoodRecord goodRecord = GoodRecordQuery.query2Entity(goodRecordQuery);
        goodRecord.setStatus(0);
        goodRecord.setCreateTime(System.currentTimeMillis());
        goodRecord.setTradeTime(System.currentTimeMillis());
        boolean save = this.save(goodRecord);
        return save
                ? BaseResponse.ok(goodRecord)
                : BaseResponse.error("添加数据失败", GoodRecordServiceImpl.class.getName());
    }

    @Override
    /** 添加一条发货信息 */
    public BaseResponse addDeliveryInfo(DeliveryInfoQuery deliveryInfoQuery) {
        if (deliveryInfoQuery.getId() == null) {
            return BaseResponse.error("请传入记录所属类型", GoodRecordServiceImpl.class.getName());
        }
        GoodRecord goodRecord = new GoodRecord();
        goodRecord.setCompany(deliveryInfoQuery.getCompany());
        goodRecord.setDeliveryNo(deliveryInfoQuery.getDeliveryNo());
        goodRecord.setId(deliveryInfoQuery.getId());
        goodRecord.setDeliveryTime(System.currentTimeMillis());
        goodRecord.setStatus(1);
        boolean update = this.updateById(goodRecord);
        return update
                ? BaseResponse.ok(goodRecord)
                : BaseResponse.error("添加数据失败", GoodRecordServiceImpl.class.getName());
    }

    @Override
    /** 虚拟商品条件分页 */
    public BaseResponse pageOnVirtualGoods(String name, PageQuery pageQuery, Integer status) {
        // 分页对象
        Page<GoodRecordVo> goodRecordPage = new Page<>();
        goodRecordPage.setSize(pageQuery.getPageSize());
        goodRecordPage.setCurrent(pageQuery.getPageNo());
        // 条件构造
        QueryWrapper queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("gr.create_time");
        // 逻辑删除条件
        queryWrapper.eq("gr.del_flag", 0);
        queryWrapper.eq("vg.del_flag", 0);
        queryWrapper.eq("c.del_flag", 0);
        // 虚拟商品类型标记
        queryWrapper.eq("gr.type", 1);
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like("c.name", name);
        }
        if (status != null) {
            queryWrapper.eq("gr.status", status);
        }
        IPage<GoodRecordVo> goodRecordVoIPage =
                goodRecordMapper.pageVirtualGoodRecord(goodRecordPage, queryWrapper);
        List<GoodRecordVo> records = goodRecordVoIPage.getRecords();
        if (records == null || records.size() == 0) {
            return BaseResponse.ok(goodRecordVoIPage, GoodRecordServiceImpl.class);
        }
        // 优惠券类型id集合
        Set<Integer> couponTypeIds =
                records.stream().map(r -> r.getTypeId()).collect(Collectors.toSet());

        Collection<CouponType> couponTypes = null;
        if (couponTypeIds != null && couponTypeIds.size() > 0) {
            couponTypes = couponTypeService.listByIds(couponTypeIds);
        }
        Collection<CouponType> finalCouponTypes = couponTypes;
        records.stream()
                .forEach(
                        r -> {
                            if (finalCouponTypes != null && finalCouponTypes.size() > 0) {
                                finalCouponTypes.stream()
                                        .forEach(
                                                ct -> {
                                                    if (ct.getId().equals(r.getTypeId())) {
                                                        r.setCouponType(ct.getName());
                                                        // 拼接图片
                                                        r.setImage(fastDFSUtils.fullUrl(ct.getImage()));
                                                    }
                                                });
                            }
                        });
        goodRecordVoIPage.setRecords(records);
        return BaseResponse.ok(goodRecordVoIPage, GoodRecordServiceImpl.class);
    }

    @Override
    /** 实物商品条件分页 */
    public BaseResponse pageOnGiftGoods(String name, PageQuery pageQuery, Integer status) {
        // 分页对象
        Page<GoodRecordVo> goodRecordPage = new Page<>();
        goodRecordPage.setSize(pageQuery.getPageSize());
        goodRecordPage.setCurrent(pageQuery.getPageNo());
        // 条件构造
        QueryWrapper queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("gg.del_flag", 0);
        queryWrapper.eq("gr.del_flag", 0);
        queryWrapper.orderByDesc("gr.create_time");
        // 商品类别标记
        queryWrapper.eq("gr.type", 0);
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like("gg.name", name);
        }
        if (status != null) {
            queryWrapper.eq("gr.status", status);
        }
        IPage<GoodRecordVo> goodRecordVoIPage =
                goodRecordMapper.pageGiftGoodRecord(goodRecordPage, queryWrapper);
        if (goodRecordVoIPage != null || !CollectionUtils.isEmpty(goodRecordVoIPage.getRecords())) {
            // 拼接图片
            goodRecordVoIPage.getRecords().stream()
                    .forEach(gv -> gv.setImage(fastDFSUtils.fullUrl(gv.getImage())));
        }
        return BaseResponse.ok(goodRecordVoIPage);
    }

    @Override
    public BaseResponse getRecord(Long id) {
        if (id == null) {
            return BaseResponse.error("请传入主键id", GoodRecordServiceImpl.class.getName());
        }
        GoodRecord goodRecord = this.getById(id);
        return BaseResponse.ok(goodRecord, GoodRecordServiceImpl.class);
    }

    @Override
    /** 分页显示积分兑换商品列表 */
    public BaseResponse pageExchange(PageQuery pageQuery) {

        // 分页对象
        Page<ExchangeVo> exchangeVoPage = new Page<>();
        exchangeVoPage.setCurrent(pageQuery.getPageNo());
        exchangeVoPage.setSize(pageQuery.getPageSize());
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.orderByDesc("point");
        IPage<ExchangeVo> page = goodRecordMapper.exchangeVoPage(exchangeVoPage, wrapper);

        // 分页的值对象
        List<ExchangeVo> records = page.getRecords();
        if (page == null || records == null || records.size() < 1) {
            return BaseResponse.ok(page, GoodRecordServiceImpl.class);
        }

        // 优惠券id集合
        Set<Long> couponIds1 = records.stream().map(re -> re.getCouponId()).collect(Collectors.toSet());
        Set<Long> couponIds = couponIds1.stream().filter(c -> c != 0).collect(Collectors.toSet());
        // 优惠券数据集合
        Collection<Coupons> coupons = null;
        if (!CollectionUtils.isEmpty(couponIds)) {
            // 优惠券数据集合
            coupons = couponsService.listByIds(couponIds);
        }
        // 优惠券类型id集合
        Set<Integer> typeIds = null;
        if (!CollectionUtils.isEmpty(coupons)) {
            // 优惠券类型id集合
            typeIds = coupons.stream().map(co -> co.getTypeId()).collect(Collectors.toSet());
        }

        // 优惠券类型数据集合
        Collection<CouponType> couponTypes = null;
        if (!CollectionUtils.isEmpty(typeIds)) {
            // 优惠券类型数据集合
            couponTypes = couponTypeService.listByIds(typeIds);
        }

        // 值对象数据重组
        Collection<Coupons> finalCoupons = coupons;
        Collection<CouponType> finalCouponTypes = couponTypes;

        records.stream()
                .forEach(
                        re -> {
                            // 虚拟商品
                            if (re.getFlag() == 1 && finalCoupons != null && finalCoupons.size() > 0) {
                                finalCoupons.stream()
                                        .forEach(
                                                coupon -> {
                                                    if (coupon.getId().longValue() == re.getCouponId())
                                                    // 虚拟商品 （优惠券名字）
                                                    {
                                                        re.setName(coupon.getName());
                                                    }

                                                    if (re.getCouponId().equals(coupon.getId())
                                                            && finalCouponTypes != null
                                                            && finalCouponTypes.size() > 0) {
                                                        finalCouponTypes.stream()
                                                                .forEach(
                                                                        ct -> {
                                                                            if (coupon.getTypeId().equals(ct.getId())) {
                                                                                // 虚拟商品（优惠券图片地址）
                                                                                re.setImage(fastDFSUtils.fullUrl(ct.getImage()));
                                                                            }
                                                                        });
                                                    }
                                                });
                            } else {
                                // 实物商品 拼接图片
                                    re.setImage(fastDFSUtils.fullUrl(re.getImage()));
                            }
                        });

        return BaseResponse.ok(page, GoodRecordServiceImpl.class);
    }

    @Override
    public BaseResponse increasePersonNum(Long id, Integer flag) {
        if (id == null) {
            return BaseResponse.error("请传入主键id值", GoodRecordServiceImpl.class.getName());
        }
        if (flag != 0 && flag != 1) {
            return BaseResponse.error("flag参数错误", GoodRecordServiceImpl.class.getName());
        }
        boolean update = false;
        // 实物商品类型
        if (flag == 0) {
            GiftGoods byId = giftGoodsService.getById(id);
            if (byId.getPersonNum() == null) {
                byId.setPersonNum(1);
            } else {
                byId.setPersonNum(byId.getPersonNum() + 1);
            }
            update = giftGoodsService.updateById(byId);
        } else {
            VirtualGoods byId = virtualGoodsService.getById(id);

            if (byId.getPersonNum() == null) {
                byId.setPersonNum(1);
            } else {
                byId.setPersonNum(byId.getPersonNum() + 1);
            }
            update = virtualGoodsService.updateById(byId);
        }
        return update
                ? BaseResponse.ok()
                : BaseResponse.error("更新信息失败", GoodRecordServiceImpl.class.getName());
    }

    /**
     * 判断商品是否剩余,是否达到没人限额
     *
     * @param id
     * @param flag  1虚拟商品 0实物
     * @param count 已兑换张数
     * @return
     */
    public BaseResponse<GoodsExchangeDto> isRemainNum(Long id, Integer flag, int count) {
        //
        GoodsExchangeDto exchangeDto = GoodsExchangeDto.builder().build();
        //
        // 商品标识   实物商品
        if (flag == 0) {
            GiftGoods giftGood = giftGoodsService.getById(id);
            if (giftGood == null) {
                return BaseResponse.error("该商品已经不存在", GoodRecordServiceImpl.class.getName());
            }
            Integer remainNum = giftGood.getRemainNum();
            Integer personMax = giftGood.getPersonMax();
            Integer point = giftGood.getPoint();
            Integer model = giftGood.getModel();
            Long aLong = giftGood.getId();
            Integer personNum = giftGood.getPersonNum();
            if (remainNum <= 0) {

                return BaseResponse.error("商品数量不足", GoodRecordServiceImpl.class.getName());
            }
            if (personMax <= count) {
                return BaseResponse.error("达到领取上限", GoodRecordServiceImpl.class.getName());
            }
            exchangeDto.setPoint(point);
            exchangeDto.setModel(model);
            exchangeDto.setGoodsId(aLong);
            exchangeDto.setRemainNum(remainNum);
            exchangeDto.setPersonNum(personNum);
            return BaseResponse.ok(exchangeDto);
        }
        if (flag == 1) {
            VirtualGoods virtualGood = virtualGoodsService.getById(id);
            if (virtualGood == null) {
                return BaseResponse.error("该商品已经不存在", GoodRecordServiceImpl.class.getName());
            }
            Integer remainNum = virtualGood.getRemainNum();
            Integer personMax = virtualGood.getPersonMax();
            Integer point = virtualGood.getPoint();
            Integer model = virtualGood.getModel();
            Long aLong = virtualGood.getId();
            Integer personNum = virtualGood.getPersonNum();

            if (remainNum <= 0) {
                return BaseResponse.error("商品数量不足", GoodRecordServiceImpl.class.getName());
            }
            if (personMax <= count) {
                return BaseResponse.error("达到领取上限", GoodRecordServiceImpl.class.getName());
            }
            exchangeDto.setPoint(point);
            exchangeDto.setModel(model);
            exchangeDto.setGoodsId(aLong);
            exchangeDto.setRemainNum(remainNum);
            exchangeDto.setPersonNum(personNum);

            // 扣减虚拟商品的数量
            virtualGood.setRemainNum(virtualGood.getRemainNum() - 1);
            virtualGood.setPersonNum(personNum + 1);
            virtualGoodsService.updateById(virtualGood);

            return BaseResponse.ok(exchangeDto);
        }
        return BaseResponse.error("参数标识错误", GoodRecordServiceImpl.class.getName());
    }

    /**
     * 抽奖兑换
     *
     * @param
     * @return
     */
    public boolean isExchangeCondition() {
        Random random = new Random();
        double v = random.nextDouble();
        double i = 0.5;
        if (v >= i) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    @Transactional
    /** 商品兑换 */
    public BaseResponse exchangeGoods(GoodExchangeQuery goodExchangeQuery) {

        QueryWrapper<GoodRecord> goodRecordQueryWrapper = new QueryWrapper<>();
        goodRecordQueryWrapper.eq("type", goodExchangeQuery.getFlag());
        goodRecordQueryWrapper.eq("user_id", goodExchangeQuery.getUserId());
        goodRecordQueryWrapper.eq("goods_id", goodExchangeQuery.getId());
        // 用户已经兑换记录数
        int count = this.count(goodRecordQueryWrapper);
        // 判断用户是否领取上限,商品数量是否剩余
        BaseResponse<GoodsExchangeDto> baseResponse =
                this.isRemainNum(goodExchangeQuery.getId(), goodExchangeQuery.getFlag(), count);
        GoodsExchangeDto exchangeDto = baseResponse.getData();
        //
        if (baseResponse.getCode() == 400) {
            return baseResponse;
        }
        // 根据用户id获取用户积分信息
        UserExtra userExtra = iUserExtraService.getUserExtraByUserId(goodExchangeQuery.getUserId());
        // 用户当前的积分值
        Double userExtraPoints = userExtra.getPoints();
        // 商品兑换或者抽奖所需积分
        Integer point = exchangeDto.getPoint();

        if (userExtraPoints < point) {
            return BaseResponse.error("剩余积分不足", GoodRecordServiceImpl.class.getName());
        }
        double setPoint;
        setPoint = userExtraPoints - point;
        // 修改用户积分信息(减少用户积分)
        UserExreaQuery userExreaQuery =
                UserExreaQuery.builder()
                        .userId(goodExchangeQuery.getUserId())
                        .points(Double.valueOf(setPoint))
                        .type(1)
                        .build();
        // 商品是抽奖时
        if (exchangeDto.getModel() == 0) {
            // 判断积分兑换是否成功
            boolean condition = this.isExchangeCondition();
            if (!condition) {
                // 扣减积分
                BaseResponse response = iUserExtraService.editUserExtra(userExreaQuery);

                // 增加参与人数
                BaseResponse response1 =
                        this.increasePersonNum(goodExchangeQuery.getId(), goodExchangeQuery.getFlag());

                if (response.getCode() == 400 || response1.getCode() == 400) {
                    return response;
                }
                // 添加积分详情表(添加记录)
                PointDetailQuery pointDetailBuild =
                        PointDetailQuery.builder()
                                .userId(goodExchangeQuery.getUserId())
                                .score(exchangeDto.getPoint())
                                .inOrOut(0)
                                .pathWay(
                                        exchangeDto.getModel() == 0
                                                ? PointTypeConsts.POINT_DRAW
                                                : PointTypeConsts.POINT_EXCHANGE)
                                .build();
                BaseResponse baseResponse1 = pointDetailService.insertPointDetailInfo(pointDetailBuild);
                if (baseResponse1.getCode() == 400) {
                    return baseResponse1;
                }
                return BaseResponse.error("运气不足", GoodRecordServiceImpl.class.getName());
            }
        }
        // 扣减积分
        BaseResponse response = iUserExtraService.editUserExtra(userExreaQuery);
        if (response.getCode() == 400) {
            return response;
        }
        // 添加兑换记录
        GoodRecordQuery goodRecordBuild =
                GoodRecordQuery.builder()
                        .goodsId(exchangeDto.getGoodsId())
                        //                .address(goodExchangeQuery.getAddress())
                        .userName(goodExchangeQuery.getUserName())
                        .point(exchangeDto.getPoint())
                        .status(0)
                        .type(goodExchangeQuery.getFlag())
                        .userId(goodExchangeQuery.getUserId())
                        .build();
        BaseResponse add = this.add(goodRecordBuild);
        if (add.getCode() == 400) {
            return add;
        }
        // 添加积分详情表(添加记录)
        PointDetailQuery pointDetailBuild =
                PointDetailQuery.builder()
                        .userId(goodExchangeQuery.getUserId())
                        .score(exchangeDto.getPoint())
                        .inOrOut(0)
                        .pathWay(exchangeDto.getModel() == 0 ? "积分参与抽奖" : "兑换商品")
                        .build();
        BaseResponse baseResponse1 = pointDetailService.insertPointDetailInfo(pointDetailBuild);
        if (baseResponse1.getCode() == 400) {
            return baseResponse1;
        }
        // 减少商品剩余数量,添加参与人数
        Integer remainNum = exchangeDto.getRemainNum() - 1;
        boolean success;
        // 实物商品
        if (goodExchangeQuery.getFlag() == 0) {
            GiftGoods giftGood =
                    GiftGoods.builder()
                            .id(goodExchangeQuery.getId())
                            .remainNum(remainNum)
                            .personNum(exchangeDto.getPersonNum() == null ? 1 : exchangeDto.getPersonNum() + 1)
                            .build();
            boolean update = giftGoodsService.updateById(giftGood);
            success = update;
        } else {
            VirtualGoods virtualGood =
                    VirtualGoods.builder()
                            .id(goodExchangeQuery.getId())
                            .personNum(exchangeDto.getPersonNum() == null ? 1 : exchangeDto.getPersonNum() + 1)
                            .remainNum(remainNum)
                            .build();
            boolean update = virtualGoodsService.updateById(virtualGood);
            success = update;
        }
        return success
                ? BaseResponse.ok(goodRecordBuild, this.getClass())
                : BaseResponse.error("兑换失败", GoodRecordServiceImpl.class);
    }

    @Override
    /** 根据用户id分页显示兑换记录 */
    public BaseResponse pageByUserId(Long userId, PageQuery pageQuery) {
        // 条件
        QueryWrapper<GoodRecordVo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId);
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("gr.del_flag", 0);
        // 分页
        Page<GoodRecordVo> goodRecordVoPage = new Page<>();
        goodRecordVoPage.setCurrent(pageQuery.getPageNo());
        goodRecordVoPage.setSize(pageQuery.getPageSize());
        // 分页数据
        IPage<GoodRecordVo> goodRecordVoIPage =
                goodRecordMapper.pageGoodRecordDetail(goodRecordVoPage, queryWrapper);
        List<GoodRecordVo> records = goodRecordVoIPage.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return BaseResponse.ok(goodRecordVoIPage, GoodRecordServiceImpl.class);
        }
        // 虚拟商品id
        Set<Long> couponIds =
                records.stream()
                        .filter(re -> re.getType() == 1)
                        .map(re -> re.getCouponId())
                        .collect(Collectors.toSet());
        // 优惠券数据
        List<Coupons> couponsList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(couponIds)) {
            Collection<Coupons> virtualGoods1 = couponsService.listByIds(couponIds);
            couponsList.addAll(virtualGoods1);
        }
        List<CouponType> couponTypes = new ArrayList<>();

        if (CollectionUtil.isNotEmpty(couponsList)) {
            Set<Integer> couponTypeIds =
                    couponsList.stream().map(cl -> cl.getTypeId()).collect(Collectors.toSet());
            Collection<CouponType> couponType = couponTypeService.listByIds(couponTypeIds);
            couponTypes.addAll(couponType);
        }
        // 组装数据
        records.stream()
                .forEach(
                        re -> {
                            // 虚拟商品
                            if (re.getType() == 1 && CollectionUtil.isNotEmpty(couponsList)) {
                                couponsList.stream()
                                        .forEach(
                                                cl -> {
                                                    if (re.getCouponId().equals(cl.getId())) {
                                                        // 优惠券名字
                                                        re.setName(cl.getName());
                                                        if (CollectionUtil.isNotEmpty(couponTypes)) {
                                                            couponTypes.stream()
                                                                    .forEach(
                                                                            ct -> {
                                                                                if (cl.getTypeId().equals(ct.getId())) {
                                                                                    // 优惠券类型数据
                                                                                    re.setCouponType(ct.getName());
                                                                                    re.setImage(fastDFSUtils.fullUrl(ct.getImage()));
                                                                                }
                                                                            });
                                                        }
                                                    }
                                                });
                            } else {
                                // 实物商品 图片拼接
                                re.setImage(fastDFSUtils.fullUrl(re.getImage()));
                            }
                        });
        goodRecordVoIPage.setRecords(records);
        return BaseResponse.ok(goodRecordVoIPage, GoodRecordServiceImpl.class);
    }

    @Override
    /** 获取最新的兑换记录 */
    public BaseResponse getLatestRecords(Integer size) {
        QueryWrapper<GoodRecordVo> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("gr.create_time");
        queryWrapper.last("limit " + size);
        queryWrapper.eq("gr.del_flag", 0);
        //
        List<GoodRecordVo> records = goodRecordMapper.getLatestRecords(queryWrapper);
        if (CollectionUtil.isEmpty(records)) {
            return BaseResponse.ok(records, this.getClass());
        }
        //
        Set<Long> couponIds = null;
        List<GoodRecordVo> goodRecordVos =
                records.stream().filter(re -> re.getType() != null && re.getType() == 1).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(goodRecordVos)) {
            couponIds = goodRecordVos.stream().map(re -> re.getCouponId()).collect(Collectors.toSet());
        }
        Collection<Coupons> coupons = null;
        if (CollectionUtil.isNotEmpty(couponIds)) {
            coupons = couponsService.listByIds(couponIds);
        }
        Collection<Coupons> finalCoupons = coupons;
        records.stream()
                .forEach(
                        re -> {
                            if (re.getType() == 1 && CollectionUtil.isNotEmpty(finalCoupons)) {
                                finalCoupons.stream()
                                        .forEach(
                                                c -> {
                                                    if (re.getCouponId().equals(re.getId())) {
                                                        re.setName(c.getName());
                                                    }
                                                });
                            }
                        });
        BaseResponse<List<GoodRecordVo>> baseResponse = BaseResponse.ok(records);
        return baseResponse;
    }

    @Override
    /** 添加实物商品兑换记录的收货地址和收货人 */
    public BaseResponse addAddressById(GoodRecordAddressQuery goodRecordAddressQuery) {
        // 信息
        GoodRecord goodRecord =
                GoodRecord.builder()
                        .address(goodRecordAddressQuery.getAddress())
                        .personPhone(goodRecordAddressQuery.getPersonPhone())
                        .personName(goodRecordAddressQuery.getPersonName())
                        .id(goodRecordAddressQuery.getId())
                        .updateTime(System.currentTimeMillis())
                        .build();
        // 更新
        boolean update = this.updateById(goodRecord);
        return update
                ? BaseResponse.ok(goodRecord, GoodRecordServiceImpl.class)
                : BaseResponse.error("添加收货地址信息失败", GoodRecordServiceImpl.class.getName());
    }
}
