package cn.wise.sc.pay.business.points.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.points.mapper.PointSourceV1Mapper;
import cn.wise.sc.pay.business.points.service.IMembershipService;
import cn.wise.sc.pay.business.points.service.IPointSourceV1Service;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.bean.PointSourceV1;
import cn.wise.sc.pay.domain.point.query.CumulativeScoreQuery;
import cn.wise.sc.pay.domain.point.query.MemberScoreQuery;
import cn.wise.sc.pay.domain.point.query.PointSourceV1Query;
import cn.wise.sc.pay.domain.point.query.WordComment;
import cn.wise.sc.pay.domain.point.util.PointSourceV1Util;
import cn.wise.sc.pay.domain.point.vo.CumulativeScoreVo;
import cn.wise.sc.pay.domain.point.vo.MemberScoreVo;
import cn.wise.sc.pay.domain.point.vo.PointSourceV1Vo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.util.concurrent.AtomicDouble;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 积分来源设置业务层接口实现类
 */
@Service
public class PointSourceV1ServiceImpl extends ServiceImpl<PointSourceV1Mapper, PointSourceV1> implements IPointSourceV1Service {

    @Autowired
    IMembershipService membershipService;

    @Override
    public BaseResponse save(PointSourceV1Query pointSourceV1Query) {
        PointSourceV1 pointSourceV1 = PointSourceV1Util.parseQueryParameter(pointSourceV1Query);
        if (pointSourceV1 == null) {
            return BaseResponse.error("参数解析错误", PointSourceV1ServiceImpl.class);
        }
        if (StringUtils.isEmpty(pointSourceV1.getWay())) {
            return BaseResponse.error("请传入途径类型", PointSourceV1ServiceImpl.class);
        }
        if (pointSourceV1.getSetType() == null) {
            return BaseResponse.error("请传入设置类型", PointSourceV1ServiceImpl.class);
        }
        pointSourceV1.setCreateTime(System.currentTimeMillis());
        pointSourceV1.setStatus(0);
        boolean save = this.save(pointSourceV1);
        return save ? BaseResponse.ok(pointSourceV1) : BaseResponse.error("添加失败", pointSourceV1Query);
    }

    @Override
    //根据id批量禁用
    public BaseResponse changeStatus(List<Integer> ids) {
        if (ids == null && ids.size() == 0) {
            return BaseResponse.error("请传入id值", PointSourceV1ServiceImpl.class.getName());
        }
        UpdateWrapper<PointSourceV1> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", ids);
        updateWrapper.set("status", 1);
        boolean update = this.update(updateWrapper);
        return update ? BaseResponse.ok() : BaseResponse.error("禁用失败", PointSourceV1ServiceImpl.class.getName());
    }

    @Override
    public BaseResponse updateDataById(PointSourceV1Query pointSourceV1Query) {
        PointSourceV1 pointSourceV1 = PointSourceV1Util.parseQueryParameter(pointSourceV1Query);
        if (pointSourceV1 == null) {
            return BaseResponse.error("参数解析错误", PointSourceV1ServiceImpl.class);
        }
        if (StringUtils.isEmpty(pointSourceV1.getWay())) {
            return BaseResponse.error("请传入途径类型", PointSourceV1ServiceImpl.class);
        }
        if (pointSourceV1.getSetType() == null) {
            return BaseResponse.error("请传入设置类型", PointSourceV1ServiceImpl.class);
        }
        if (pointSourceV1.getId() == null) {
            return BaseResponse.error("请传入主键id", PointSourceV1ServiceImpl.class);
        }
        pointSourceV1.setUpdateTime(System.currentTimeMillis());
        boolean update = this.updateById(pointSourceV1);
        return update ? BaseResponse.ok(pointSourceV1) : BaseResponse.error("更新信息失败", pointSourceV1Query);
    }


    @Override
    public BaseResponse listOnCondition(String way) {
        QueryWrapper<PointSourceV1> queryWrapper = null;
        if (!StringUtils.isEmpty(way)) {
            queryWrapper = new QueryWrapper<>();
            queryWrapper.like("way", way);
        }
        //列表记录
        List<PointSourceV1> pointSourceV1List = this.list(queryWrapper);
        if (pointSourceV1List == null) {
            return BaseResponse.ok(pointSourceV1List, PointDetailServiceImpl.class);
        }
        //返回的值对象
        List<PointSourceV1Vo> pointSourceV1Vos = new ArrayList<>();
        pointSourceV1List.stream().forEach(ps -> {
            PointSourceV1Vo pointSourceV1Vo = PointSourceV1Util.parseBean2Vo(ps);
            //不安天累计设置  设置默认值
            if (ps.getSetType() != 3) {
                pointSourceV1Vo.setCumulativeScores(PointSourceV1Util.defaultCumulativeScore());
                pointSourceV1Vo.setStringCumulativeScores(PointSourceV1Util.defaultCumulativeScoreVo());
            }
            pointSourceV1Vos.add(pointSourceV1Vo);
        });
        //过滤是会员设置类型
        List<PointSourceV1Vo> pointSourceMember = pointSourceV1Vos.stream().filter(pv -> pv.getSetType() == 1)
                .collect(Collectors.toList());
        //会员等级id集合
        Set<Long> memberShipIds = new HashSet<>();
        //所关联的会员等级对象
        List<Membership> memberships = new ArrayList();

        if (pointSourceMember != null && pointSourceMember.size() != 0) {
            List<MemberScoreQuery> memberScores = null;
            for (PointSourceV1Vo co : pointSourceMember) {
                memberScores = co.getMemberScores();
                memberShipIds = memberScores.stream().map(me -> me.getMemberShipId()).collect(Collectors.toSet());
            }
        }
        if (memberShipIds != null && memberShipIds.size() > 0) {
            //会员对象
            Collection<Membership> membership = membershipService.listByIds(memberShipIds);
            memberships.addAll(membership);
        }
//组装返回对象
        pointSourceV1Vos.stream().forEach(pvos -> {
            pvos.setWayFlag(PointSourceV1Util.getWayInt(pvos.getWay()));
//按天累计签到类型
            if (pvos.getSetType() == 3) {
                //封装按天累计签到值对象
                List<CumulativeScoreVo> cumulativeScoreVos = new ArrayList<>();
                List<CumulativeScoreQuery> cumulativeScores = pvos.getCumulativeScores();
                cumulativeScores.stream().forEach(ct -> {
                    CumulativeScoreVo cumulativeScoreVo = new CumulativeScoreVo();
                    Integer cumulativeFlag = ct.getCumulativeFlag();
                    String cumulativeString = PointSourceV1Util.cumulativeInt(cumulativeFlag);
                    cumulativeScoreVo.setCumulativeFlag(ct.getCumulativeFlag());
                    cumulativeScoreVo.setCumulativeName(cumulativeString);
                    cumulativeScoreVo.setCumulativeScore(ct.getCumulativeScore());
                    cumulativeScoreVos.add(cumulativeScoreVo);
                });
                pvos.setStringCumulativeScores(cumulativeScoreVos);
            }
            //+++++++++++=========================
            //会员设置类型
            if (pvos.getSetType() == 1 && memberships != null && memberships.size() != 0) {
                List<MemberScoreVo> memberScoreVos = new ArrayList<>();
                List<MemberScoreQuery> memberScores = pvos.getMemberScores();

                memberScores.stream().forEach(ms -> {
                    memberships.stream().forEach(msp -> {
                        if (ms.getMemberShipId().equals(msp.getId())) {
                            MemberScoreVo memberScoreVo = new MemberScoreVo();
                            memberScoreVo.setMemberShipName(msp.getName());
                            memberScoreVo.setMemberScore(ms.getMemberScore());
                            memberScoreVo.setMemberShipId(ms.getMemberShipId());
                            memberScoreVos.add(memberScoreVo);
                        }
                    });
                });
                pvos.setStringMemberScores(memberScoreVos);
            }
        });
        return BaseResponse.ok(pointSourceV1Vos);
    }


    @Override
    public BaseResponse changeStatusByWay(List<Integer> ways) {
        if (ways == null && ways.size() == 0) {
            return BaseResponse.error("请传入途径类型的值", PointSourceV1ServiceImpl.class.getName());
        }
        UpdateWrapper<PointSourceV1> updateWrapper = new UpdateWrapper<>();
        List<String> strings = new ArrayList<>();
        ways.stream().forEach(w -> {
            String way = PointSourceV1Util.getWay(w);
            strings.add(way);
        });
        updateWrapper.in("way", strings);
        updateWrapper.set("status", 1);
        boolean update = this.update(updateWrapper);
        return update ? BaseResponse.ok() : BaseResponse.error("禁用失败", PointSourceV1ServiceImpl.class.getName());
    }

    @Override
    public BaseResponse enableStatusByWays(List<Integer> ways) {
        if (ways == null && ways.size() == 0) {
            return BaseResponse.error("请传入途径类型的值", PointSourceV1ServiceImpl.class.getName());
        }
        UpdateWrapper<PointSourceV1> updateWrapper = new UpdateWrapper<>();
        List<String> strings = new ArrayList<>();
        ways.stream().forEach(w -> {
            String way = PointSourceV1Util.getWay(w);
            strings.add(way);
        });
        updateWrapper.in("way", strings);
        updateWrapper.set("status", 0);
        boolean update = this.update(updateWrapper);
        return update ? BaseResponse.ok() : BaseResponse.error("启用失败", PointSourceV1ServiceImpl.class.getName());
    }

    /**
     * 根据途径修改数据
     *
     * @param pointSourceV1Query
     * @return
     */
    @Override
    public BaseResponse updateDataByWay(PointSourceV1Query pointSourceV1Query) {
        if (pointSourceV1Query.getSetType() == null) {
            return BaseResponse.error("请传入设置类型", PointSourceV1ServiceImpl.class);
        }
        Integer way1 = pointSourceV1Query.getWay();
        if (way1 == null || way1 < 0 || way1 > 3) {
            return BaseResponse.error("途径类型参数有误", PointSourceV1ServiceImpl.class);
        }
        PointSourceV1 pointSourceV1 = PointSourceV1Util.parseQueryParameter(pointSourceV1Query);
        if (pointSourceV1 == null) {
            return BaseResponse.error("参数解析错误", PointSourceV1ServiceImpl.class);
        }
        //途径
        String way = pointSourceV1.getWay();
        QueryWrapper<PointSourceV1> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("way", way);
        PointSourceV1 one = this.getOne(queryWrapper);
        //没有就新增
        if (one == null) {
            pointSourceV1.setCreateTime(System.currentTimeMillis());
            pointSourceV1.setStatus(0);
            boolean save = this.save(pointSourceV1);
            return save ? BaseResponse.ok(pointSourceV1) : BaseResponse.error("添加数据失败", PointSourceV1ServiceImpl.class.getName());
        }
        //有就修改
        pointSourceV1.setUpdateTime(System.currentTimeMillis());
        UpdateWrapper<PointSourceV1> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("way", way);
        boolean update = this.update(pointSourceV1, updateWrapper);
        return update ? BaseResponse.ok(pointSourceV1) : BaseResponse.error("更新信息失败", pointSourceV1Query);
    }

    @Override
    public Double getScore(Long key, String pathway) {
        //根据路径获取积分信息
        QueryWrapper<PointSourceV1> qw = new QueryWrapper<>();
        if (StrUtil.isEmpty(pathway)) {
            return 0D;
        }
        qw.eq("way", pathway);
        PointSourceV1 pointSourceV1 = this.getOne(qw);
        if (pointSourceV1 == null) {
            return 0D;
        }

        //根据积分类型处理积分信息
        Integer type = pointSourceV1.getSetType();
        AtomicDouble score = new AtomicDouble(0);
        if (type == 0) {
            //统一设置
            score.set(Double.valueOf(pointSourceV1.getUnionScore()));
        } else if (type == 1) {
            if (key == 0L) {
                return 0D;
            }
            //按会员等级获取
            List<MemberScoreQuery> memberScoreQueries = JSON.parseArray(pointSourceV1.getMemberScores(), MemberScoreQuery.class);
            assert memberScoreQueries.size() != 0;
            memberScoreQueries.stream()
                    .filter(arg -> arg.getMemberShipId() == key.longValue())
                    .findFirst()
                    .ifPresent(arg -> score.set(arg.getMemberScore()));
        } else if (type == 2) {
            //不累计设置
            score.set(pointSourceV1.getNonCumulativeScore());
        } else if (type == 3) {
            //按天累计设置
            if (key == 0) {
                return 0D;
            }
            List<CumulativeScoreQuery> cumulativeScoreQueries = JSON.parseArray(pointSourceV1.getCumulativeScores(), CumulativeScoreQuery.class);
            assert cumulativeScoreQueries.size() != 3;
            //清零天数
            CumulativeScoreQuery resetScoreQuery = cumulativeScoreQueries.stream()
                    .filter(arg -> arg.getCumulativeFlag() == 2)
                    .findFirst()
                    .orElse(null);
            //累积天数
            CumulativeScoreQuery accumulateScoreQuery = cumulativeScoreQueries.stream()
                    .filter(arg -> arg.getCumulativeFlag() == 1)
                    .findFirst()
                    .orElse(null);
            //基础分数
            CumulativeScoreQuery dayBaseScoreQuery = cumulativeScoreQueries.stream()
                    .filter(arg -> arg.getCumulativeFlag() == 0)
                    .findFirst()
                    .orElse(null);
            assert resetScoreQuery != null;
            assert accumulateScoreQuery != null;
            assert dayBaseScoreQuery != null;
            //计算累积分数
            Integer accumulateDay = Math.toIntExact(key % resetScoreQuery.getCumulativeScore());
            int d = dayBaseScoreQuery.getCumulativeScore();
            score.set((accumulateDay * d + key * d) + key * (key - 1) / 2 * d);

        } else {
            //评论积分
            WordComment wordComment = JSON.parseObject(pointSourceV1.getWordScores(), WordComment.class);
            if (wordComment.getWords() < key) {
                score.set(wordComment.getWordScore());
            } else {
                score.set(0D);
            }
        }

        return score.get();
    }

}
