package cn.wise.sc.pay.task.timer.order.handler;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.mail.MailService;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.order.model.StatisticsGoods;
import cn.wise.sc.pay.task.timer.order.model.GoodMarketVo;
import com.alibaba.fastjson.JSON;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.thymeleaf.TemplateEngine;
import org.thymeleaf.context.Context;
import javax.mail.MessagingException;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: 每天推送商户商品售卖统计
 * @author: qh
 * @create: 2020-06-12 11:14
 **/
@Slf4j
@Component
public class EveryDaySellerGoodsMarketHandler {

    @Autowired
    IOrdersService iOrdersService;
    @Autowired
    private MailService mailService;
    @Autowired
    private TemplateEngine templateEngine;
    @Autowired
    private ApplicationContext context;
    @Autowired
    IMerchantService iMerchantService;

    @XxlJob("EveryDaySellerGoodsMarketHandler")
    public ReturnT<String> execute(String s) {

        //获取商品卖出统计信息
        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        HashMap<String, Object> eq = new HashMap<>(3);
        pageWrapperQuery.setEq(eq);
        HashMap<String, Object> ge = new HashMap<>(1);
        long start = DateUtil.beginOfDay(DateUtil.date()).getTime();
        long end = DateUtil.endOfDay(DateUtil.date()).getTime();
        ge.put("create_time", start);
        pageWrapperQuery.setGe(ge);
        HashMap<String, Object> lt = new HashMap<>(1);
        lt.put("create_time", end);
        pageWrapperQuery.setLe(lt);
        BaseResponse<List<StatisticsGoods>> baseResponse = iOrdersService.statisticsGoodsMarketMoneyGroupSellerGood(pageWrapperQuery);
        if (baseResponse.getCode() == 400) {
            return ReturnT.FAIL;
        }
        List<StatisticsGoods> goodMarketList = baseResponse.getData();

        //获取商品信息  回去邮件地址
        BaseResponse response = iMerchantService.getMerchList("");
        if (response.getCode() == 400) {
            return ReturnT.FAIL;
        }
        //获取商户邮箱信息
        List<LinkedHashMap> list = (List<LinkedHashMap>) response.getData();

        try {
            FileOutputStream fos = new FileOutputStream("/data/card/good" + DateUtil.date().toDateStr() + ".txt", true);
            for (LinkedHashMap linkedHashMap : list) {
                Merchant merchant = BeanUtil.mapToBean(linkedHashMap, Merchant.class, true);
                List<StatisticsGoods> goodsList = goodMarketList.stream()
                        .filter(arg -> arg.getSellerId().longValue() == merchant.getId())
                        .collect(Collectors.toList());
                List<GoodMarketVo> goodMarketVos = statisticsGood(goodsList);
                if (merchant != null && goodMarketVos.size() != 0) {
                    //true表示在文件末尾追加
                    fos.write(JSON.toJSONString(goodMarketVos).getBytes());
                    //日志
                    log.info("发送邮件=======> {} ======> {}", merchant.getLinkEmail(), JSON.toJSON(goodMarketVos));
                    sendMail(goodMarketVos, merchant.getLinkEmail());
                    log.info("邮件发送成功!");
                }
            }
            fos.close();
        } catch (Exception e) {
            return ReturnT.FAIL;
        }

        return ReturnT.SUCCESS;
    }

    /*
     *
     *
     * */


    /**
     * 统计每件商品的买家和销售额
     *
     * @param goods 商品集合
     * @return List<GoodMarketVo>
     */
    private List<GoodMarketVo> statisticsGood(List<StatisticsGoods> goods) {
        if (goods == null || goods.size() == 0) {
            return new ArrayList<>();
        }
        //商品id  去重
        List<Long> goodIds = new ArrayList<>(goods.size());
        List<GoodMarketVo> rts = new ArrayList<>(goods.size());

        for (StatisticsGoods good : goods) {
            //新的一个商品
            if (!goodIds.contains(good.getId())) {
                GoodMarketVo marketVo = new GoodMarketVo();
                marketVo.setName(good.getName());
                marketVo.setSold(good.getNum());
                marketVo.setTotalPrice(good.getTotalPrice());
                marketVo.setId(good.getId());
                rts.add(marketVo);
                //添加ids
                goodIds.add(good.getId());
            } else {
                //不是一个新的商品
                GoodMarketVo marketVo = rts.stream()
                        .filter(arg -> arg.getId().longValue() == good.getId())
                        .findFirst().orElse(null);

                assert marketVo != null;
                //累加金额和数量
                marketVo.setTotalPrice(marketVo.getTotalPrice().add(good.getTotalPrice()));
                marketVo.setSold(marketVo.getSold() + good.getNum());
            }
        }
        return rts;
    }

    /**
     * 发邮件
     *
     * @param list 商品信息集合
     * @throws MessagingException x
     */
    private void sendMail(List<GoodMarketVo> list, String email) throws MessagingException {
        if (list.size() == 0) {
            return;
        }
        Context context = new Context();
        context.setVariable("Infos", list);
        String emailTemplate = templateEngine.process("welcome", context);
        String dateTime = DateUtil.formatDate(DateUtil.date());
        mailService.sendHtmlMail(email, dateTime + "_商品统计", emailTemplate);
    }
}
