package cn.wise.sc.pay.task.timer.order.handler;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.order.service.IOrderCallExService;
import cn.wise.sc.pay.api.order.service.ISafeguardService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.order.model.OrderCallException;
import cn.wise.sc.pay.domain.order.model.OrderCallExceptionStatus;
import cn.wise.sc.pay.domain.order.model.Safeguard;
import cn.wise.sc.pay.domain.order.model.SafeguardStatus;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 订单退款
 * @author: qh
 * @create: 2020-05-22 11:27
 */
@Component
public class OrderRefundHandler {

    /*
     * 0.审核通过的时候  就该发起退款
     * 1.轮询退款记录表  将审核通过的退款单---时间大于10分钟
     * 2.将10分钟都还未改变的退款单  加入orderCallException
     * 3.轮询CallException  发起主动查询
     *
     * */

    @Autowired
    ISafeguardService iSafeguardService;
    @Autowired
    IOrderCallExService iOrderCallExService;

    /**
     * 轮询支付失败的订单  主动发起支付结果查询
     */
    @XxlJob("OrderRefundHandler")
    public ReturnT<String> execute(String s) {
        PageWrapperQuery wrapperQuery = PageWrapperQueryUtil.create();
        PageQuery pageQuery = new PageQuery();
        pageQuery.setPageNo(0);
        pageQuery.setPageSize(100);
        wrapperQuery.setPageQuery(pageQuery);
        Map<String, Object> eq = new HashMap<>(1);
        eq.put("status", SafeguardStatus.REFUND.name());
        wrapperQuery.setEq(eq);
        Map<String, Object> lt = new HashMap<>(1);
        lt.put("update_time", System.currentTimeMillis() - 60 * 1000);
        wrapperQuery.setLt(lt);

        BaseResponse<Safeguard> response = iSafeguardService.page(wrapperQuery);

        if (response.getCode() == 400) {
            return ReturnT.FAIL;
        }
        // 获取订单集合添加异常记录
        List<Safeguard> list = response.getPage().getRecords();

        List<OrderCallException> insertList = new ArrayList<>(list.size());
        for (Safeguard safeguard : list) {
            if (StrUtil.isNotEmpty(safeguard.getOrderNo())) {
                OrderCallException callException = new OrderCallException();
                callException.setOrderNo(safeguard.getOrderNo());
                callException.setRefundNo(safeguard.getRefundNo());
                callException.setFailCount(0);
                callException.setCreateTime(System.currentTimeMillis());
                callException.setUpdateTime(System.currentTimeMillis());
                callException.setOrderStatus(OrderCallExceptionStatus.APPLY_REFUND_EXCEPTION.name());
                insertList.add(callException);
            }
        }

        BaseResponse exceptionInfoBatch = iOrderCallExService.createExceptionInfoBatch(insertList);
        if (exceptionInfoBatch.getCode() == 400) {
            return ReturnT.FAIL;
        }
        return ReturnT.SUCCESS;
    }

}
